Confluent Control Center
=================

## Initial Setup ##

*Note!* All directions assume you have a similar file structure for your projects and that you're starting at the root:
``` bash
. # root
├── controlcenter
└── kafka
```

First you'll need to build Kafka.

``` bash
brew install maven gradle
git clone git@github.com:apache/kafka.git
cd kafka
gradle
./gradlew install
```

To rebuild:

``` bash
cd kafka
rm -rf .gradle/ && ./gradlew clean jarAll installAll
```

Next install frontend the dependencies:
``` bash
brew install watchman flow
cd controlcenter/js
npm install
```

## Running the Application ##

You'll need to have the following programs running:

1. **ZooKeeper listening at `localhost:2181`**
  ``` bash
  cd kafka
  bin/zookeeper-server-start.sh config/zookeeper.properties
  ```

1. **Kafka**
  ``` bash
  cd kafka
  bin/kafka-server-start.sh config/server.properties
  ```

1. **Connect listening at `localhost:8083`**
  ``` bash
  cd kafka
  bin/connect-distributed.sh config/connect-distributed.properties
  ```
  *Note: you might also want to run the above command with additional connectors in your CLASSPATH. Here is an example with the [hdfs-connector](https://github.com/confluentinc/kafka-connect-hdfs) added:*
  ``` bash
  CLASSPATH=/path/to/root/kafka-connect-hdfs/target/kafka-connect-hdfs-*-package/share/java/kafka-connect-hdfs/* bin/connect-distributed.sh config/connect-distributed.properties
  ```

1. **controlcenter application**
  ``` bash
  cd controlcenter
  mvn package -DskipTests
  ./bin/control-center-start config/control-center-dev.properties
  ```

1. **Start a fake producer / consumer**
  ``` bash
  ./private-bin/rock
  ```
  If you'd like to test with multiple partitions of production/consumption run
  ``` bash
  bin/kafka-topics.sh --zookeeper localhost:2181 --alter --partitions 18 --topic test-topic
  bin/kafka-topics.sh --zookeeper localhost:2181 --alter --partitions 12 --topic another-topic
  ```

  If you're not doing any frontend development then you may now open the application at [localhost:9021](http://localhost:9021)

  If you make changes to only java code and want to do a quicker build you can run
  ```
  mvn -P\!production package -DskipTests
  ```

  *Note: If you are experiencing problems building, you may need to force an update of your local maven cache using the -U flag.*

1. **(Optional) Frontend development hot reload server**
  ``` bash
  cd controlcenter/js
  npm run dev
  ```
  Now you can see frontend changes at [localhost:3001](http://localhost:3001)  

## Redux Devtools Extension (optional but recommended) ##
In development you can use [redux-devtools-extension](https://github.com/zalmoxisus/redux-devtools-extension) to help with debugging. You can install it from the [Chrome Web Store](https://chrome.google.com/webstore/detail/redux-devtools/lmhkpmbekcpmknklioeibfkpmmfibljd).

## RedBox Errors (optional) ##
You can use [redbox-react](https://github.com/KeywordBrain/redbox-react) to catch and display errors in the UI. To enable, set either env variable `USE_REDBOX` or `EDITOR_SCHEME`. Setting `EDITOR_SCHEME` will link stack traces to your editor, e.g. `subl` will open links in SublimeText.

## Editor Setup ##

1. Atom packages
  - Nuclide
  - atom-ternjs
  - language-babel
  - linter
  - linter-eslint
  - linter-shellcheck
  - linter-pep8

<!--1. IntelliJ?-->

## Frontend Generator ##
You can use `npm run g` (an alias to [redux-cli](https://github.com/SpencerCDixon/redux-cli)'s generate) to create new frontend code from predefined templates. Examples:

1. List all possible options for generation (stick to ones under 'controlcenter', though)
``` bash
npm run g -- -h
```

2. Create a pure React component with related css and tests
``` bash
npm run g -- pure my-new-pure-component
```

# Resetting the Application (deleting all data)

```
./bin/control-center-reset config/control-center-dev.properties
```

#Dumping/Loading Data

## Data format

Similar to `kafka-avro-console-producer` and `kafka-avro-console-consumer`, each record is a single line with the fields separated by tab (by default).  All fields are optional except the value.  Possible fields are:

<table>
    <tr>
        <th>Field</th>
        <th>Format</th>
        <th>Description</th>
        <th>Example</th>
    </tr>
    <tr>
        <td>topic</td>
        <td>string</td>
        <td>the topic</td>
        <td>_confluent-monitoring</td>
    </tr>
    <tr>
        <td>partition</td>
        <td>integer</td>
        <td>the partition</td>
        <td>1</td>
    </tr>
    <tr>
        <td>timestamp</td>
        <td><a href="https://en.wikipedia.org/wiki/ISO_8601">ISO 8601 datetime</a></td>
        <td>the message timestamp</td>
        <td>2016-06-09T09:29:30.648-07:00</td>
    </tr>
    <tr>
        <td>key</td>
        <td>JSON (protobuf messages use <a href="https://developers.google.com/protocol-buffers/docs/proto3#json">this mapping</a>)</td>
        <td>the message key</td>
        <td>null</td>
    </tr>
    <tr>
        <td>value</td>
        <td>JSON (protobuf messages use <a href="https://developers.google.com/protocol-buffers/docs/proto3#json">this mapping</a>)</td>
        <td>the message value</td>
        <td>{"clientType":"PRODUCER","clientId":"producer-id-2"}</td>
    </tr>       
</table>

### Getting a sample of monitoring data to load later

```
./bin/control-center-console-consumer config/control-center-dev.properties --topic _confluent-monitoring --from-beginning > monitoring.json
^C
Processed a total of 325 messages
```

Sample data

```
$ head -5 monitoring.json
_confluent-monitoring	0	2016-06-09T09:29:30.633-07:00	null	{"clientType":"PRODUCER","clientId":"producer-id-2","session":"388cb0d9-a647-4481-92bf-63915e96c52c","window":"1465489755000","timestamp":"1465489770465","topic":"another-topic","count":"63","aggregateBytes":"504","aggregateCrc":-9.81604109E8,"samplePeriod":"15000","minWindow":"-1","maxWindow":"-1"}
_confluent-monitoring	0	2016-06-09T09:29:30.648-07:00	null	{"clientType":"PRODUCER","clientId":"producer-id-2","session":"388cb0d9-a647-4481-92bf-63915e96c52c","sequence":"1","window":"1465489770000","timestamp":"1465489770465","topic":"another-topic","count":"3","aggregateBytes":"24","aggregateCrc":9.0084232E8,"samplePeriod":"15000","minWindow":"1465489755000","maxWindow":"1465489755000"}
_confluent-monitoring	0	2016-06-09T09:29:30.650-07:00	null	{"clientType":"PRODUCER","clientId":"producer-id-2","session":"388cb0d9-a647-4481-92bf-63915e96c52c","window":"1465489755000","timestamp":"1465489770465","topic":"test-topic","count":"67","aggregateBytes":"536","aggregateCrc":-4.62237136E8,"samplePeriod":"15000","minWindow":"-1","maxWindow":"-1"}
_confluent-monitoring	0	2016-06-09T09:29:30.651-07:00	null	{"clientType":"PRODUCER","clientId":"producer-id-2","session":"388cb0d9-a647-4481-92bf-63915e96c52c","sequence":"1","window":"1465489770000","timestamp":"1465489770465","topic":"test-topic","count":"3","aggregateBytes":"24","aggregateCrc":-1.773583436E9,"samplePeriod":"15000","minWindow":"1465489755000","maxWindow":"1465489755000"}
_confluent-monitoring	0	2016-06-09T09:29:30.633-07:00	null	{"clientType":"PRODUCER","clientId":"myproducer","session":"d2b2db6f-f56e-4940-9574-6f7cf5c07f19","window":"1465489755000","timestamp":"1465489770465","topic":"test-topic","count":"135","aggregateBytes":"1080","aggregateCrc":1.418194272E9,"samplePeriod":"15000","minWindow":"-1","maxWindow":"-1"}
```

### Loading saved data

```
./private-bin/control-center-console-producer config/control-center-dev.properties --topic _confluent-monitoring --property parse.topic=true < monitoring.json
```

### Dumping data from all C3 topics

```
./bin/control-center-console-consumer config/control-center-dev.properties --from-beginning --topic "_confluent-controlcenter.+"
```

#Generating Monitoring Input Data

#### Generate 5 minutes of data for 2 input partitions (in the future so that it doesn't expire and get filtered by C3)
`python tools/stream_monitoring/generate_monitoring_input.py --start-time-unix=1530898605 tools/stream_monitoring/10-producers-spec.json 2 300 > generated.json`

#### Start C3 before loading data to make sure the monitoring topic is created

```
rm -rf /tmp/confluent/ /tmp/kafka-logs/ /tmp/zookeeper/
#cd kafka
./bin/zookeeper-server-start.sh config/zookeeper.properties
./bin/kafka-server-start.sh config/server.properties
./bin/control-center-start config/control-center-dev.properties
```

#### Load generated data

`./private-bin/control-center-console-producer config/control-center-dev.properties --topic _confluent-monitoring --property parse.timestamp=false --property parse.key=false < generated.json`

### View it in the UI

The generated data file has UI links as comments at the beginning and end of the file

```
head -2 generated.json
# Start first window: 2018-07-06 10:36:45, start last window: 2018-07-06 15:36:30
# C3 UI: http://localhost:9021/monitoring/consumer-groups?fixed-end=1530916605000&fixed-start=1530898605000
```

# Running Sub-Topologies

For debugging or perf testing, you can run a single sub-topology by specifying an input topic for it.

### Examples

#### Run only the first sub-topology that consumes from the monitoring topic
```
./bin/control-center-start --run-tasks-for=_confluent-monitoring config/control-center.properties
```

#### Run the "Verifier" sub-topology that consumes from the re-keyed topic
```
./bin/control-center-start --run-tasks-for=_confluent-controlcenter-1-monitoring-message-rekey config/control-center.properties
```

#### Run the "MonitoringStream" sub-topology
```
./bin/control-center-start --run-tasks-for=_confluent-controlcenter-1-monitoring-aggregate-rekey config/control-center.properties
```

#### Run the "GroupStream" sub-topology
```
./bin/control-center-start --run-tasks-for=_confluent-controlcenter-1-group-stream-extension-rekey config/control-center.properties
```

#### Run the map/combine step of the "ConsumerGroupStream" sub-topology
```
./bin/control-center-start --run-tasks-for=_confluent-controlcenter-1-aggregate-topic-partition config/control-center.properties
```

#### Run the reduce steps of the "ConsumerGroupStream" sub-topology
```
./bin/control-center-start --run-tasks-for=_confluent-controlcenter-1-group-aggregate-topic-FIFTEEN_SECONDS config/control-center.properties
./bin/control-center-start --run-tasks-for=_confluent-controlcenter-1-group-aggregate-topic-ONE_HOUR config/control-center.properties
./bin/control-center-start --run-tasks-for=_confluent-controlcenter-1-group-aggregate-topic-ONE_WEEK config/control-center.properties
```
#### Run the reduce steps of the "monitoring-message-rekey" sub-topology
```
./bin/control-center-start --run-tasks-for=_confluent-controlcenter-0-monitoring-message-rekey config/control-center-dev.properties
```

# Authentication

Testing authentication can be done with a static file, or using ldap.

### file
```
cat <<EOF > /tmp/confluent/login.properties
jay: kafka,Administrators
neha: akfak,Administrators
jun: kafka-
EOF
```


### ldap
Get open ldap running via docker
```
docker run -v /tmp/ldap:/var/lib/ldap \
           -e LDAP_DOMAIN=confluent.io \
           -e LDAP_ORGANISATION="Confluent" \
           -e LDAP_ROOTPASS=kafkaesque \
           -d nickstenning/slapd
```

Add some users to ldap

```
cat <<EOF > /tmp/data.ldif
dn: dc=confluent,dc=io
objectClass: dcObject
objectClass: organization
dc: confluent
o: Confluent
description: We do Kafka

dn: ou=People,dc=confluent,dc=io
objectClass: organizationalunit
objectClass: top
ou: People

dn: ou=Groups,dc=confluent,dc=io
objectClass: organizationalunit
ou: Groups

dn: cn=Administrators,ou=Groups,dc=confluent,dc=io
objectClass: groupofuniquenames
objectClass: top
ou: Groups
cn: Administrators
uniquemember: sn=jay,ou=People,dc=confluent,dc=io
uniquemember: sn=neha,ou=People,dc=confluent,dc=io

dn: sn=jay,ou=People,dc=confluent,dc=io
cn: Jay Kreps
objectClass: inetOrgPerson
sn: jay
userPassword: kafka
ou: Administrators
ou: People

dn: sn=neha,ou=People,dc=confluent,dc=io
cn: Neha Narkhede
objectClass: inetOrgPerson
sn: neha
userPassword: akfak
ou: Administrators
ou: People

dn: sn=jun,ou=People,dc=confluent,dc=io
cn: Jun Rao
objectClass: inetOrgPerson
sn: jun
userPassword: kafka-
ou: People
EOF

ldapadd -h localhost -p 3890 -c -D cn=admin,dc=confluent,dc=io -W -f data.ldif
```
