// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.metrics.reporter;

import io.confluent.metrics.record.ConfluentMetric.KafkaMeasurable;
import io.confluent.metrics.record.ConfluentMetric.KafkaMetricName;
import org.apache.kafka.common.MetricName;
import org.apache.kafka.common.metrics.KafkaMetric;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

public class KafkaMetricsHelper {
    private static final KafkaMetricName.Builder kafkaMetricNameBuilder = KafkaMetricName.newBuilder();
    private static final KafkaMeasurable.Builder kafkaMeasurableBuilder = KafkaMeasurable.newBuilder();

    private KafkaMetricsHelper() {
        // static class; can't instantiate
    }

    public static List<KafkaMeasurable> collectKafkaMetrics(Map<MetricName, KafkaMetric> metricMap, Pattern pattern) {
        List<KafkaMeasurable> kafkaMeasurables = new ArrayList<>();

        for (Map.Entry<MetricName, KafkaMetric> entry : metricMap.entrySet()) {
            MetricName metricName = entry.getKey();
            if (pattern != null && !pattern.matcher(metricName.name()).matches())
                continue;

            kafkaMeasurableBuilder.clear();
            kafkaMeasurableBuilder.setValue(entry.getValue().value());

            kafkaMetricNameBuilder.clear();
            kafkaMetricNameBuilder.setGroup(Utils.notNullOrEmpty(metricName.group()));
            kafkaMetricNameBuilder.setName(Utils.notNullOrEmpty(metricName.name()));
            kafkaMetricNameBuilder.putAllTags(metricName.tags());
            kafkaMeasurableBuilder.setMetricName(kafkaMetricNameBuilder.build());

            kafkaMeasurables.add(kafkaMeasurableBuilder.build());
        }

        return kafkaMeasurables;
    }
}
