// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.kafka.databalancing;

import io.confluent.kafka.databalancing.metric.Metrics;
import io.confluent.kafka.databalancing.report.RebalanceReport;
import io.confluent.kafka.databalancing.topology.Broker;
import io.confluent.kafka.databalancing.topology.ReplicaAssignment;
import kafka.admin.BrokerMetadata;

import java.util.Collection;

public class ProposedRebalance {
    private final ReplicaAssignment currentAssignment;
    private final ReplicaAssignment proposedAssignment;
    private final Metrics metrics;
    private final Collection<Broker> brokersToRemove;
    private final Collection<BrokerMetadata> registeredBrokers;

    public ProposedRebalance(Collection<BrokerMetadata> registeredBrokers,
                             ReplicaAssignment currentAssignment, ReplicaAssignment proposedAssignment,
                             Metrics metrics,
                             Collection<Broker> brokersToRemove) {
        this.registeredBrokers = registeredBrokers;
        this.currentAssignment = currentAssignment;
        this.proposedAssignment = proposedAssignment;
        this.metrics = metrics;
        this.brokersToRemove = brokersToRemove;
    }

    public ReplicaAssignment currentAssignment() {
        return currentAssignment;
    }

    public ReplicaAssignment proposedAssignment() {
        return proposedAssignment;
    }

    public RebalanceReport report() {
        return new RebalanceReport(registeredBrokers, currentAssignment, proposedAssignment, metrics, brokersToRemove);
    }

    /**
     * Returns `true` if the proposed assignment is different from the current assignment. In the future, we may decide
     * to only return true if the imbalance is above a configurable threshold.
     */
    public boolean shouldRebalance() {
        return !currentAssignment.equals(proposedAssignment);
    }
}
