// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.kafka.databalancing.fairness;

import io.confluent.kafka.databalancing.CompareResult;
import io.confluent.kafka.databalancing.topology.Broker;
import org.apache.kafka.common.TopicPartition;

import java.util.List;

/**
 * NB: The definition of fairness must incorporate the use of ceil/floor when dealing with fractional values.
 * Below Par (i.e. below the fair value) is always defined with ceil(). AbovePar is always defined with floor().
 * Thus, where fairness is a fractional value, there will be an overlap between abovePar and belowPar.
 * This is required as the algorithm is not symmetric (it starts at with abovePar brokers then searches
 * belowPar brokers).
 *
 * Consider the an arrangement where the replica counts on each broker are (1, 1, 2, 3, 3). There are 11
 * replicas over 5 brokers so fairness = 2.2. Thus ceil(fairvalue) -> 3 and would not achieve fairness.
 * Rerunning in the opposite direction (belowPar->abovePar) also does not help  as there are no above
 * par options to move to. Hence we choose to apply ceil/floor to the abovePar/belowPar to create an overlapping
 * intersection.
 */
public interface Fairness {
    List<String> aboveParRacks();

    List<String> belowParRacks();

    List<Broker> aboveParBrokers();

    List<Broker> belowParBrokers();

    CompareResult compareBrokerFairness(TopicPartition tp, Broker b1, Broker b2);
    CompareResult compareBrokerFairness(Broker b1, TopicPartition tp1, Broker b2, TopicPartition tp2);

    CompareResult compareRackFairness(TopicPartition tp, Broker b1, Broker b2);
    CompareResult compareRackFairness(Broker b1, TopicPartition tp1, Broker b2, TopicPartition tp2);
}
