// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.kafka.databalancing.metric;

import io.confluent.metrics.record.ConfluentMetric;
import org.apache.kafka.common.TopicPartition;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Extracts the Log size metric from a @ConfluentMetric.MetricsMessage. Assumes that the size is a YammerGauge.
 */
public class LogSizeMetricReader implements MetricReader<Long> {
    private static final String METRIC_GROUP = "kafka.log";
    private static final String METRIC_NAME = "Size";
    private static final String METRIC_TYPE = "Log";
    private static final Pattern MBEAN_NAME_PATTERN = Pattern.compile("^.*,topic=(.*?),partition=(\\d+)$");

    @Override
    public Map<TopicPartition, Long> metricValues(ConfluentMetric.MetricsMessage message) {
        Map<TopicPartition, Long> metricValues = new HashMap<>();
        for (ConfluentMetric.YammerGauge gauge : message.getYammerGaugeList()) {
            ConfluentMetric.YammerMetricName metricName = gauge.getMetricName();
            if (!(metricName.getGroup().equals(METRIC_GROUP)
                    || metricName.getName().equals(METRIC_NAME)
                    || metricName.getType().equals(METRIC_TYPE))) {
                continue;
            }
            String mBeanName = metricName.getMBeanName();
            Matcher matcher = MBEAN_NAME_PATTERN.matcher(mBeanName);
            if (matcher.find()) {
                int partition = Integer.parseInt(matcher.group(2));
                String topic = matcher.group(1);
                TopicPartition topicPartition = new TopicPartition(topic, partition);
                long size = Long.parseLong(gauge.getValue());
                metricValues.put(topicPartition, size);
            }
        }
        return metricValues;
    }
}

