// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.kafka.databalancing.metric;

import io.confluent.metrics.record.ConfluentMetric;
import org.apache.kafka.common.TopicPartition;

import java.util.Map;

/**
 * Defines an interface to extract metric values from a @ConfluentMetric.MetricsMessage. As the number of metrics
 * we need from the metrics topic evolves, we can add implementations to extract different values from each message.
 */
public interface MetricReader<T> {

    /**
     * Extract a specific metric value from a given message, for all topic-partition pairs included in the message.
     * Each class implementing this method will extract a specific value type.
     * @param message The message to read
     * @return The value for each topic/partition pair in the message for which the value is defined. Returns an empty
     *         map if the value doesn't exist in the message.
     */
    Map<TopicPartition, T> metricValues(ConfluentMetric.MetricsMessage message);
}

