// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.kafka.databalancing.topology;

import org.apache.kafka.common.TopicPartition;

import java.util.Objects;

public class Replica {

    private final TopicPartition topicPartition;
    private final Broker broker;

    public Replica(TopicPartition topicPartition, Broker broker) {
        this.topicPartition = Objects.requireNonNull(topicPartition, "topicPartition should not be null");
        this.broker = Objects.requireNonNull(broker, "broker should not be null");
    }

    public Broker broker() {
        return broker;
    }

    public TopicPartition topicPartition() {
        return topicPartition;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;

        Replica replica = (Replica) o;

        return broker.equals(replica.broker) && topicPartition.equals(replica.topicPartition);
    }

    @Override
    public int hashCode() {
        int result = topicPartition.hashCode();
        result = 31 * result + broker.hashCode();
        return result;
    }

    @Override
    public String toString() {
        return "r[" + topicPartition.topic() + ":" + topicPartition.partition() + ":" + broker.id() + "]";
    }

}
