// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.kafka.databalancing.license;

import kafka.zk.EmbeddedZookeeper;
import org.apache.kafka.common.utils.MockTime;
import org.apache.kafka.common.utils.SystemTime;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.Assert;

import java.util.concurrent.TimeUnit;

public class ZkTrialPeriodTest {
    private EmbeddedZookeeper zookeeper;
    private String zkConnect;

    @Before
    public void setUp() {
        zookeeper = new EmbeddedZookeeper();
        zkConnect = "localhost:" + zookeeper.port();
    }

    @After
    public void tearDown() {
        zookeeper.shutdown();
    }

    @Test
    public void testTimeElapsed() {
        MockTime time = new MockTime(System.currentTimeMillis());
        ZkTrialPeriod zkTrialPeriod = new ZkTrialPeriod(zkConnect, time);
        long origTimeRemaining = zkTrialPeriod.startOrVerify();
        time.sleep(5);
        long newTimeRemaining = zkTrialPeriod.startOrVerify();
        Assert.assertTrue("Expected some time to elapse on the trial.", newTimeRemaining < origTimeRemaining);
    }


    @Test
    public void testTrialExpired() {
        MockTime time = new MockTime(System.currentTimeMillis());
        ZkTrialPeriod zkTrialPeriod = new ZkTrialPeriod(zkConnect, time);
        long origTimeRemaining = zkTrialPeriod.startOrVerify();
        Assert.assertEquals("Expected a 30 day trial.", origTimeRemaining, TimeUnit.DAYS.toMillis(30));
        time.sleep(TimeUnit.DAYS.toMillis(35));
        long newTimeRemaining = zkTrialPeriod.startOrVerify();
        Assert.assertEquals("Expected the trial to have expired after 35 days", newTimeRemaining, 0);
    }

}
