// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.controlcenter.data;

import com.google.common.base.Joiner;
import com.google.inject.Inject;
import io.confluent.command.record.Command.CommandConfigType;
import io.confluent.command.record.Command.CommandKey;
import io.confluent.command.record.Command.CommandMessage;
import io.confluent.controlcenter.command.CommandModule.ClusterMetadataStore;
import io.confluent.controlcenter.rest.res.Topic;
import io.confluent.controlcenter.util.KafkaClusterMetadataClientFactory;
import org.apache.kafka.streams.state.ReadOnlyKeyValueStore;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

import static io.confluent.controlcenter.rest.RestModule.IGNORE_TOPICS;

public class KafkaMetadataDao {
  private static final Logger log = LoggerFactory.getLogger(new Object() {}.getClass().getEnclosingClass());
  private final KafkaClusterMetadataClientFactory metadataClientFactory;
  private final ReadOnlyKeyValueStore<CommandKey, CommandMessage> clusterMetadataStore;

  @Inject
  public KafkaMetadataDao(
      KafkaClusterMetadataClientFactory metadataClientFactory,
      @ClusterMetadataStore ReadOnlyKeyValueStore<CommandKey, CommandMessage> clusterMetadataStore
      ) {
    this.metadataClientFactory = metadataClientFactory;
    this.clusterMetadataStore = clusterMetadataStore;
  }

  public List<Topic> getTopics(String clusterId, boolean excludeDefaultTopics) {
    CommandMessage msg = clusterMetadataStore.get(getCommandKey(clusterId));
    if (msg == null) {
      log.info("No cluster metadata for {}", clusterId);
      return null;
    }
    List<String> zkConnectList = msg.getClusterMetadata().getKafka().getZookeeperConnectList();
    if (zkConnectList == null || zkConnectList.size() == 0) {
      log.info("No ZK connect information for {}", msg.getClusterMetadata());
      return null;
    }
    String zkConnect = Joiner.on(',').join(zkConnectList);
    List<Topic> allTopics = metadataClientFactory.create(zkConnect).queryTopics();
    if (excludeDefaultTopics) {
      List<Topic> nonInternalTopics = new ArrayList<>();
      for (Topic topic : allTopics) {
        if (!IGNORE_TOPICS.contains(topic.name)) {
          nonInternalTopics.add(topic);
        }
      }
      return nonInternalTopics;
    }
    return allTopics;
  }

  private CommandKey getCommandKey(String clusterId) {
    return CommandKey.newBuilder()
        .setConfigType(CommandConfigType.CLUSTER_METADATA)
        .setGuid(clusterId)
        .build();
  }
}
