// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.controlcenter.rest;

import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.google.inject.Inject;

import java.security.Principal;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;

@Path("/2.0/auth")
@Produces(MediaType.APPLICATION_JSON)
public class AuthResource {

  private final Gson gson;

  @Inject
  public AuthResource(Gson gson) {
    this.gson = gson;
  }

  @GET
  @Path("/principal")
  public Response principal(
      @Context SecurityContext securityContext
  ) {
    Principal user = securityContext.getUserPrincipal();
    if (user == null) {
      return Response.status(Response.Status.UNAUTHORIZED).build();
    }

    // Strings are a protected class when serialized. need to make this a valid json string manually
    return Response
        .ok(Strings.isNullOrEmpty(user.getName()) ? null : gson.toJson(user.getName()))
        .build();
  }
}
