// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.controlcenter.rest;

import com.google.inject.Inject;
import io.confluent.controlcenter.data.ClusterMetadataDao;
import io.confluent.controlcenter.record.Controlcenter.ClusterContext;
import io.confluent.controlcenter.rest.res.KafkaCluster;
import io.confluent.controlcenter.rest.res.KafkaClusterDisplay;
import io.confluent.controlcenter.rest.res.KafkaClusterListResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.ws.rs.BadRequestException;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.NotFoundException;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Path("/2.0/clusters/")
@Produces(MediaType.APPLICATION_JSON)
public class ClusterResource {
  private static final Logger log = LoggerFactory.getLogger(new Object() {}.getClass().getEnclosingClass());
  private final ClusterMetadataDao clusterMetadata;

  @Inject
  public ClusterResource(
      ClusterMetadataDao clusterMetadata
  ) {
    this.clusterMetadata = clusterMetadata;
  }

  @GET
  @Path("/kafka")
  public List<KafkaCluster> kafka() {
    return clusterMetadata.getKafkaClusters();
  }

  @GET
  @Path("/kafka/display/{context}")
  public KafkaClusterListResponse kafkaDisplay(
      @PathParam("context") final String contextStr
  ) {
    String contextName = contextStr.toUpperCase().replace('-', '_');
    ClusterContext context;
    try {
      context = ClusterContext.valueOf(contextName);
    } catch (IllegalArgumentException e) {
      throw new NotFoundException();
    }
    if (context.equals(ClusterContext.STREAM_MONITORING)) {
      return clusterMetadata.getKafkaClustersForStreamMonitoring();
    }
    if (context.equals(ClusterContext.KAFKA_CONNECT_UI)) {
      return clusterMetadata.getKafkaClustersForConnect();
    }
    return new KafkaClusterListResponse(null, new ArrayList<KafkaClusterDisplay>());
  }

  @PATCH
  @Path("/kafka/{clusterId}")
  @Consumes("application/merge-patch+json")
  public void patchKafka(
      @PathParam("clusterId") String clusterId,
      KafkaCluster updatedCluster
  ) throws Exception {
    if (updatedCluster.clusterId != null && !updatedCluster.clusterId.equals(clusterId)) {
      throw new BadRequestException("The provided clusterId does not match the resource.");
    }
    log.info("Updating metadata for cluster {}: {}", clusterId, updatedCluster);
    try {
      clusterMetadata.updateKafkaCluster(clusterId, updatedCluster, TimeUnit.SECONDS.toMillis(30));
    } catch (Exception e) {
      log.error("Error updating cluster {}", clusterId, e);
      throw e;
    }
  }
}
