package io.confluent.controlcenter.rest;

import com.google.inject.Inject;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import io.confluent.command.CommandStore;
import io.confluent.command.record.Command.CommandConfigType;
import io.confluent.command.record.Command.CommandKey;
import io.confluent.command.record.Command.CommandMessage;
import io.confluent.serializers.ProtoSerde;

@Path("/2.0/command")
@Produces(MediaType.APPLICATION_JSON)
public class CommandResource {

  private final CommandStore commandStore;

  @Inject
  public CommandResource(
      CommandStore commandStore
  ) {
    this.commandStore = commandStore;
  }

  @GET
  @Path("/{commandConfigType}/{guid}")
  public Object get(
      @PathParam("commandConfigType") CommandConfigType commandConfigType,
      @PathParam("guid") String guid
  ) {
    CommandMessage cm = commandStore.type(commandConfigType).get(
        CommandKey.newBuilder()
            .setConfigType(commandConfigType)
            .setGuid(guid)
            .build()
    );
    return new ProtoSerde<>(CommandMessage.getDefaultInstance()).toJSON(cm);
  }
}
