package io.confluent.controlcenter.rest;

import com.google.inject.Inject;

import org.apache.kafka.connect.runtime.rest.entities.CreateConnectorRequest;

import java.util.List;
import java.util.Map;

import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import io.confluent.controlcenter.connect.ConnectService;

@Path("/2.0/management/connect")
@Produces(MediaType.APPLICATION_JSON)
public class ConnectResource {

  private final ConnectService connectService;

  @Inject
  public ConnectResource(ConnectService connectService) {
    this.connectService = connectService;
  }

  @GET
  @Path("connector-plugins")
  public Object connectorPlugins() throws Exception {
    return connectService.getConnectorPlugins();
  }

  @PUT
  @Path("connector-plugins/{connectorType}/config/validate")
  public Object putConnectorPlugins(
      final @PathParam("connectorType") String connType,
      final Map<String, String> connectorConfig
  ) throws Exception {
    return connectService.validateConnectorPlugin(connType, connectorConfig);
  }

  @GET
  @Path("connectors")
  public Object connectors() throws Exception {
    return connectService.getConnectors();
  }

  @POST
  @Path("connectors")
  public Object createConnector(final CreateConnectorRequest createRequest) throws Exception {
    return connectService.createConnector(createRequest);
  }

  @GET
  @Path("connectors/{connector}")
  public Object getConnector(final @PathParam("connector") String connector) throws Exception {
    return connectService.getConnector(connector);
  }

  @PUT
  @Path("connectors/{connector}/pause")
  public Object putConnectorPause(final @PathParam("connector") String connector) throws Exception {
    return connectService.putConnectorPause(connector);
  }

  @PUT
  @Path("connectors/{connector}/resume")
  public Object putConnectorResume(final @PathParam("connector") String connector) throws Exception {
    return connectService.putConnectorResume(connector);
  }

  @GET
  @Path("connectors/{connector}/status")
  public Object getConnectorStatus(final @PathParam("connector") String connector) throws Exception {
    return connectService.getConnectorStatus(connector);
  }

  @GET
  @Path("connectors/{connector}/config")
  public Object getConnectorConfig(
      final @PathParam("connector") String connector
  ) throws Exception {
    return connectService.getConnectorConfig(connector);
  }

  @PUT
  @Path("connectors/{connector}/config")
  public Object putConnectorConfig(
      final @PathParam("connector") String connector,
      final Map<String, String> connectorConfig
  ) throws Exception {
    return connectService.putConnectorConfig(connector, connectorConfig);

  }

  @GET
  @Path("connectors/{connector}/tasks")
  public Object getTaskConfigs(final @PathParam("connector") String connector) throws Exception {
    return connectService.getTaskConfigs(connector);
  }

  @POST
  @Path("connectors/{connector}/tasks")
  public Object putTaskConfigs(
      final @PathParam("connector") String connector,
      final List<Map<String, String>> taskConfigs
  ) throws Exception {
    return connectService.putTaskConfigs(connector, taskConfigs);
  }

  @GET
  @Path("connectors/{connector}/tasks/{task}/status")
  public Object getTaskStatus(
      @PathParam("connector") String connector,
      @PathParam("task") Integer task
  ) throws Exception {
    return connectService.getTaskStatus(connector, task);
  }

  @DELETE
  @Path("connectors/{connector}")
  public Object destroyConnector(final @PathParam("connector") String connector) throws Exception {
    return connectService.destroyConnector(connector);
  }
}
