// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.controlcenter.rest;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import java.lang.management.BufferPoolMXBean;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryPoolMXBean;
import java.lang.management.PlatformManagedObject;
import java.util.*;

@Path("/2.0/status")
@Produces(MediaType.APPLICATION_JSON)
public class StatusResource {

  @GET
  @Path("java/runtime")
  public Map<String, Object> runtime() throws Exception {
    Map<String, Object> metrics = new HashMap<>();
    metrics.put("java.version", System.getProperty("java.version"));
    addObjectMetrics(ManagementFactory.getRuntimeMXBean().getObjectName(), metrics);
    return metrics;
  }

  @GET
  @Path("java/memory")
  public Map<String, Object> memory() throws Exception {
    Map<String, Object> metrics = new HashMap<>();
    metrics.put("HeapMemoryUsage", ManagementFactory.getMemoryMXBean().getHeapMemoryUsage());
    metrics.put("NonHeapMemoryUsage", ManagementFactory.getMemoryMXBean().getNonHeapMemoryUsage());
    addObjectListMetrics("BufferPools", ManagementFactory.getPlatformMXBeans(BufferPoolMXBean.class), metrics);
    addMemoryPoolMetrics(metrics);
    addObjectListMetrics("GarbageCollector", ManagementFactory.getGarbageCollectorMXBeans(), metrics);
    return metrics;
  }

  @GET
  @Path("java/threads")
  public Map<String, Object> threads() throws Exception {
    Map<String, Object> metrics = new HashMap<>();
    addObjectMetrics(ManagementFactory.getThreadMXBean().getObjectName(), metrics);
    return metrics;
  }


  @GET
  @Path("os")
  public Map<String, Object> os() throws Exception {
    Map<String, Object> metrics = new HashMap<>();
    addObjectMetrics(ManagementFactory.getOperatingSystemMXBean().getObjectName(), metrics);
    return metrics;
  }


  @GET
  @Path("streams")
  public Map<String, Object> streams() throws Exception {
    Map<String, Object> metrics = new HashMap<>();
    MBeanServer mBeanServer = ManagementFactory.getPlatformMBeanServer();
    ObjectName streamsMBean = new ObjectName("kafka.streams:*");
    Set<ObjectName> names = mBeanServer.queryNames(streamsMBean, null);
    for (ObjectName name : names) {
      addObjectMetrics(name, metrics);
    }
    return metrics;
  }

  private <T extends PlatformManagedObject> void addObjectListMetrics(String name, List<T> beans, Map<String, Object> metrics) throws Exception {
    List<Map<String, Object>> metricList = new ArrayList<>();
    for (PlatformManagedObject bean : beans) {
      Map<String, Object> metric = new HashMap<>();
      addObjectMetrics(bean.getObjectName(), metric);
      metricList.add(metric);
    }
    metrics.put(name, metricList);
  }

  private void addObjectMetrics(ObjectName name, Map<String, Object> metrics) throws Exception {
    MBeanServer mBeanServer = ManagementFactory.getPlatformMBeanServer();
    Map<String, Object> attrMap = new HashMap<>();
    MBeanInfo info = mBeanServer.getMBeanInfo(name);
    for (MBeanAttributeInfo attrInfo : info.getAttributes()) {
      try {
        if (!attrInfo.getName().equals("ObjectName")) {
          attrMap.put(attrInfo.getName(), mBeanServer.getAttribute(name, attrInfo.getName()));
        }
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    metrics.put(name.getCanonicalName(), attrMap);
  }

  private void addMemoryPoolMetrics(Map<String, Object> metrics) {
    List<Map<String, Object>> memPools = new ArrayList<>();
    for (MemoryPoolMXBean bean : ManagementFactory.getMemoryPoolMXBeans()) {
      Map<String, Object> metric = new HashMap<>();
      metric.put("name", bean.getName());
      metric.put("type", bean.getType());
      metric.put("usage", bean.getUsage());
      memPools.add(metric);
    }
    metrics.put("MemoryPools", memPools);
  }

}
