package io.confluent.controlcenter.rest;

import com.google.common.collect.Maps;
import com.google.common.io.ByteStreams;
import com.google.protobuf.GeneratedMessageV3;
import com.google.protobuf.InvalidProtocolBufferException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.Map;
import java.util.Set;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;

import io.confluent.CustomJsonFormat;
import io.confluent.serializers.UberSerde;

//TODO(norwood): try to get rid of this 
public class UberMessageBodyRW<T extends GeneratedMessageV3> implements MessageBodyReader<T>, MessageBodyWriter<T> {

  private static final Logger log = LoggerFactory.getLogger(UberMessageBodyRW.class);
  private Map<Class<T>, UberSerde<T>> uberSerdes = Maps.newHashMap();

  public UberMessageBodyRW(UberSerde<T> uberSerde) {
    this.uberSerdes.put(uberSerde.type(), uberSerde);
  }

  public UberMessageBodyRW(Set<UberSerde<T>> uberSerdes) {
    for (UberSerde<T> us : uberSerdes) {
      this.uberSerdes.put(us.type(), us);
    }
  }

  private boolean canHandle(Class<?> type, MediaType mediaType) {
    return MediaType.APPLICATION_JSON_TYPE.equals(mediaType) && uberSerdes.containsKey(type);
  }

  @Override
  public boolean isReadable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
    return canHandle(type, mediaType);
  }

  @Override
  public T readFrom(
      Class<T> type,
      Type genericType,
      Annotation[] annotations,
      MediaType mediaType,
      MultivaluedMap<String, String> httpHeaders,
      InputStream entityStream
  ) throws IOException, WebApplicationException {
    return uberSerdes.get(type).deserialize(uberSerdes.get(type).fromJSON(new String(ByteStreams.toByteArray(entityStream))));
  }

  @Override
  public boolean isWriteable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
    return canHandle(type, mediaType);
  }

  @Override
  public long getSize(T proto, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
    try {
      return toJson(proto).length;
    } catch (InvalidProtocolBufferException e) {
      log.warn("unable to serialize proto", e);
    }
    return 0;
  }

  @Override
  public void writeTo(
      T proto,
      Class<?> type,
      Type genericType,
      Annotation[] annotations,
      MediaType mediaType,
      MultivaluedMap<String, Object> httpHeaders,
      OutputStream entityStream
  ) throws IOException, WebApplicationException {
    entityStream.write(toJson(proto));
  }

  private byte[] toJson(T proto) throws InvalidProtocolBufferException {
    return CustomJsonFormat.printer()
        .includingDefaultValueFields()
        .omittingInsignificantWhitespace()
        .print(proto)
        .getBytes();
  }
}
