// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.controlcenter.rest.req;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import io.confluent.controlcenter.Rollup;
import io.confluent.monitoring.record.Monitoring.ClientType;
import io.confluent.monitoring.record.Monitoring.MonitoringMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@JsonIgnoreProperties(ignoreUnknown = true)
public class RangeRequest {
  private static final Logger log = LoggerFactory.getLogger(RangeRequest.class);

  private long startTimeMs;
  private long stopTimeMs;

  public Rollup rollup = Rollup.FIFTEEN_SECONDS;

  public String clusterId;
  public String clientId;
  public ClientType clientType;
  public String group;
  public Integer partition;
  public String topic;

  private MonitoringMessage.Builder createBaseKeySpec() {
    MonitoringMessage.Builder builder = MonitoringMessage.newBuilder();
    builder.setClusterId(clusterId);
    if (clientId != null) {
      builder.setClientId(clientId);
    }
    if (clientType != null) {
      builder.setClientType(clientType);
    }
    if (group != null) {
      builder.setGroup(group);
    }
    if (topic != null) {
      builder.setTopic(topic);
    }
    if (partition != null) {
      builder.setPartition(partition);
    }
    return builder;
  }

  public MonitoringMessage createStartKeySpec() {
    MonitoringMessage.Builder builder = createBaseKeySpec();
    builder.setWindow(getRollupAlignedStartTimeMs());
    return builder.build();
  }

  public long getRollupAlignedStartTimeMs() {
    if (startTimeMs % rollup.getMillis() != 0) {
      startTimeMs = (startTimeMs / rollup.getMillis() + 1) * rollup.getMillis();
    }
    return startTimeMs;
  }

  public long getRollupAlignedStopTimeMs() {
    if (stopTimeMs % rollup.getMillis() != 0) {
      stopTimeMs = (stopTimeMs / rollup.getMillis()) * rollup.getMillis();
    }
    return stopTimeMs;
  }

  public RangeRequest(String clusterId, long startTimeMs, long stopTimeMs) {
    this.clusterId = clusterId;
    this.startTimeMs = startTimeMs;
    this.stopTimeMs = stopTimeMs;
  }
}
