// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.controlcenter.streams;

import com.google.common.collect.ImmutableSet;
import com.google.inject.Inject;
import io.confluent.controlcenter.ControlCenterConfigModule;
import org.apache.kafka.common.Cluster;
import org.apache.kafka.common.KafkaException;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.streams.processor.DefaultPartitionGrouper;
import org.apache.kafka.streams.processor.PartitionGrouper;
import org.apache.kafka.streams.processor.TaskId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * Used for running a single sub-toplogy in development or performance-testing scenarios
 */
public class SingleGroupPartitionGrouper implements PartitionGrouper {
  private static final Logger log = LoggerFactory.getLogger(new Object() {}.getClass().getEnclosingClass());
  //We have to use static injection because this class is instantiated by Kafka Streams
  @Inject @ControlCenterConfigModule.SubTopologyInputTopics private static ImmutableSet<String> inputTopics;
  private PartitionGrouper defaultPartitionGrouper = new DefaultPartitionGrouper();

  @Override
  public Map<TaskId, Set<TopicPartition>> partitionGroups(Map<Integer, Set<String>> topicGroups, Cluster metadata) {
    log.info("All topic groups: " + topicGroups);
    Map<Integer, Set<String>> includedTopicGroups = new HashMap<>();
    for (Map.Entry<Integer, Set<String>> entry : topicGroups.entrySet()) {
      if (entry.getValue().containsAll(inputTopics)) {
        includedTopicGroups.put(entry.getKey(), entry.getValue());
      }
    }
    if (includedTopicGroups.isEmpty()) {
      throw new KafkaException("No topic groups match the input set: " + inputTopics);
    }
    log.info("Included topic groups: " + includedTopicGroups);
    Map<TaskId, Set<TopicPartition>> result = defaultPartitionGrouper.partitionGroups(includedTopicGroups, metadata);
    log.warn(String.format("Running a sub-topology for input topics: %s with map: %s", inputTopics, result));
    return result;
  }
}
