package io.confluent.controlcenter.streams.alert;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.google.protobuf.Descriptors;

import org.apache.commons.mail.Email;
import org.apache.commons.mail.EmailException;
import org.apache.kafka.common.utils.Bytes;
import org.apache.kafka.streams.kstream.ForeachAction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.confluent.command.record.alert.CommandAlert;
import io.confluent.controlcenter.ControlCenterConfig;
import io.confluent.controlcenter.alert.record.Alert.ActionInfo;
import io.confluent.controlcenter.alert.record.Alert.AlertInfo;
import io.confluent.controlcenter.rest.RestModule.RestEndpoint;

public class SendEmail implements ForeachAction<Bytes, AlertInfo> {
  private static final Logger log = LoggerFactory.getLogger(SendEmail.class);

  private static final Descriptors.FieldDescriptor
      EMAIL_FIELD = ActionInfo.getDescriptor().findFieldByNumber(ActionInfo.EMAIL_FIELD_NUMBER);

  private final Provider<Email> emailProvider;
  private final ControlCenterConfig config;
  private final String restEndpoint;

  @Inject
  public SendEmail(
      Provider<Email> emailProvider,
      ControlCenterConfig config,
      @RestEndpoint String restEndpoint
  ) {
    this.emailProvider = emailProvider;
    this.config = config;
    this.restEndpoint = restEndpoint;
  }

  @Override
  public void apply(Bytes key, AlertInfo value) {
    for (ActionInfo ai : value.getActionsList()) {
      if (ai.hasField(EMAIL_FIELD)) {
        CommandAlert.EmailAction emailAction = ai.getEmail();
        if (config.getBoolean(ControlCenterConfig.CONTROL_CENTER_EMAIL_ENABLE)) {
          try {
            Email email = emailProvider.get();
            email.addTo(emailAction.getAddress());
            email.setSubject(emailAction.getSubject());
            email.setMsg(String.format("%s/alerts/overview/history/%d-%s", restEndpoint, value.getTimestamp(), value.getGuid()));
            String messageId = email.send();
            log.debug("sending=success email to={} with subject={} from trigger={} messageId={}", emailAction.getAddress(), emailAction.getSubject(), value.getMonitoringTrigger().getGuid(), messageId);
          } catch (EmailException e) {
            log.warn("sending=failure email to={} with subject={} from trigger={} {}", emailAction.getAddress(), emailAction.getSubject(), value.getMonitoringTrigger().getGuid(), e);
          }
        } else {
          log.debug("sending=prevented email to={} with subject={} from trigger={} messageId={}", emailAction.getAddress(), emailAction.getSubject(), value.getMonitoringTrigger().getGuid());
        }
      }
    }
  }
}
