// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.controlcenter.util;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

public class LRUSet<E> implements Set<E> {
  private final LinkedHashMap<E, Void> cache;

  public LRUSet(final int maxSize) {
    cache = new LinkedHashMap<E, Void>(16, .75f, true) {
      @Override
      protected boolean removeEldestEntry(Map.Entry<E, Void> eldest) {
        return size() > maxSize;
      }
    };
  }

  @Override
  public int size() {
    return cache.size();
  }

  @Override
  public boolean isEmpty() {
    return cache.isEmpty();
  }

  @Override
  public boolean contains(Object o) {
    return cache.containsKey(o);
  }

  @Override
  public Iterator<E> iterator() {
    return cache.keySet().iterator();
  }

  @Override
  public Object[] toArray() {
    return cache.keySet().toArray();
  }

  @Override
  public <T> T[] toArray(T[] a) {
    return cache.keySet().toArray(a);
  }

  @Override
  public boolean add(E e) {
    if (cache.containsKey(e)) {
      return false;
    }
    cache.put(e, null);
    return true;
  }

  @Override
  public boolean remove(Object o) {
    boolean wasPresent = cache.containsKey(o);
    cache.remove(o);
    return wasPresent;
  }

  @Override
  public boolean containsAll(Collection<?> c) {
    return cache.keySet().containsAll(c);
  }

  @Override
  public boolean addAll(Collection<? extends E> c) {
    throw new RuntimeException("Not implemented yet");
  }

  @Override
  public boolean retainAll(Collection<?> c) {
    throw new RuntimeException("Not implemented yet");
  }

  @Override
  public boolean removeAll(Collection<?> c) {
    throw new RuntimeException("Not implemented yet");
  }

  @Override
  public void clear() {
    cache.clear();
  }
}
