package io.confluent.controlcenter.rest;

import org.junit.Before;
import org.junit.Test;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import javax.ws.rs.core.MediaType;

import io.confluent.controlcenter.alert.record.Alert.TriggerInfo;
import io.confluent.serializers.ProtoSerde;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class UberMessageBodyRWTest {

  private UberMessageBodyRW<TriggerInfo> uberMessageBodyRW;
  private final String asString = "{\"window\":\"28980\",\"longValue\":\"489\"}";
  private final TriggerInfo asProto = TriggerInfo.newBuilder()
      .setWindow(28980L)
      .setLongValue(489)
      .build();

  @Before
  public void setUp() {
    this.uberMessageBodyRW = new UberMessageBodyRW<>(new ProtoSerde<>(TriggerInfo.getDefaultInstance()));
  }

  @Test
  public void testOnlyReadsJson() {
    assertTrue(uberMessageBodyRW.isReadable(TriggerInfo.class, null, null, MediaType.APPLICATION_JSON_TYPE));
    assertFalse(uberMessageBodyRW.isReadable(TriggerInfo.class, null, null, MediaType.APPLICATION_XML_TYPE));
  }

  @Test
  public void testOnlyReadsProtoType() {
    assertTrue(uberMessageBodyRW.isReadable(TriggerInfo.class, null, null, MediaType.APPLICATION_JSON_TYPE));
    assertFalse(uberMessageBodyRW.isReadable(Object.class, null, null, MediaType.APPLICATION_JSON_TYPE));
  }

  @Test
  public void testOnlyWritesJson() {
    assertTrue(uberMessageBodyRW.isWriteable(TriggerInfo.class, null, null, MediaType.APPLICATION_JSON_TYPE));
    assertFalse(uberMessageBodyRW.isWriteable(TriggerInfo.class, null, null, MediaType.APPLICATION_XML_TYPE));
  }

  @Test
  public void testOnlyWritesProtoType() {
    assertTrue(uberMessageBodyRW.isWriteable(TriggerInfo.class, null, null, MediaType.APPLICATION_JSON_TYPE));
    assertFalse(uberMessageBodyRW.isWriteable(Object.class, null, null, MediaType.APPLICATION_JSON_TYPE));
  }

  @Test
  public void testWritesToStream() throws Exception {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    uberMessageBodyRW.writeTo(
        asProto,
        TriggerInfo.class,
        null,
        null,
        null,
        null,
        baos
    );
    baos.close();
    byte[] out = baos.toByteArray();
    String a = new String(out);
    assertTrue(a.length() > 0);
    assertEquals(asString, a);
  }

  @Test
  public void testReadsFromStream() throws Exception {
    ByteArrayInputStream bais = new ByteArrayInputStream(asString.getBytes());
    TriggerInfo out = uberMessageBodyRW.readFrom(
        TriggerInfo.class,
        null,
        null,
        null,
        null,
        bais
    );
    assertTrue(out.isInitialized());
    assertTrue(out.equals(asProto));
  }

  @Test
  public void testGetSize() throws Exception {
    long out = uberMessageBodyRW.getSize(
        asProto,
        null,
        null,
        null,
        null
    );
    assertEquals(asString.length(), out);
  }

}
