// (Copyright) [2016 - 2016] Confluent, Inc.
package io.confluent.controlcenter.serialization;

import com.google.gson.Gson;
import io.confluent.controlcenter.record.Controlcenter;
import io.confluent.serializers.ProtoSerde;
import org.apache.kafka.streams.state.StateSerdes;
import org.apache.kafka.streams.state.internals.WindowStoreUtils;
import org.junit.Test;

import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.*;

public class WindowStoreSerdeTest {
  final ProtoSerde<Controlcenter.TopicPartition> tpSerde = new ProtoSerde<>(Controlcenter.TopicPartition.getDefaultInstance());
  final WindowStoreSerde<Controlcenter.TopicPartition> serde = new WindowStoreSerde<>(tpSerde);
  final Controlcenter.TopicPartition tp = Controlcenter.TopicPartition.newBuilder()
      .setTopic("foo")
      .setPartition(1)
      .build();
  final long timestamp = 99L;
  final int seqNum = 8;
  final WindowStoreSerde.WindowedKey<Controlcenter.TopicPartition> wk = new WindowStoreSerde.WindowedKey<>(tp, timestamp, seqNum);


  @Test
  public void fromJSON() throws Exception {
    String json = "{\"key\":{\"topic\":\"foo\",\"partition\":1.0},\"timestamp\":99,\"seq_num\":8}";
    byte[] bytes = serde.fromJSON(json);
    Controlcenter.TopicPartition decodedTp = WindowStoreUtils.keyFromBinaryKey(bytes, new StateSerdes<>("dummy", tpSerde, tpSerde));
    assertEquals(tp, decodedTp);
    assertEquals(timestamp, WindowStoreUtils.timestampFromBinaryKey(bytes));
  }

  @Test
  public void toJSON() throws Exception {
    String json = serde.toJSON(wk);
    Gson gson = new Gson();
    Map<String, Object> obj = (Map<String, Object>) gson.fromJson(json, HashMap.class);
    Map<String, Object> key = (Map<String, Object>) obj.get("key");
    assertEquals("foo", key.get("topic").toString());
    assertEquals(1, ((Double)key.get("partition")).intValue());
    assertEquals(timestamp, ((Double) obj.get("timestamp")).longValue());
    assertEquals(seqNum, ((Double) obj.get("seq_num")).intValue());
  }

}