package io.confluent.controlcenter.streams;

import com.google.common.base.Predicate;
import com.google.common.collect.Collections2;
import com.google.common.collect.ImmutableSet;
import com.google.inject.Guice;
import com.google.inject.Injector;
import io.confluent.controlcenter.ControlCenterConfig;
import io.confluent.controlcenter.ControlCenterConfigModule;
import io.confluent.controlcenter.serialization.SerializationModule;
import org.junit.Test;

import java.util.Collection;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class TopicStoreMasterTest {
  private final ControlCenterConfig config = new ControlCenterConfig();
  private final Injector injector = Guice.createInjector(
      new ControlCenterConfigModule(config),
      new StreamsConfigModule(),
      new SerializationModule(),
      new TopicStoreModule()
  );
  private final TopicStoreMaster topicStoreMaster = injector.getInstance(TopicStoreMaster.class);
  private final static String DEFAULT_CMD_TOPIC_NAME = "_confluent-command";
  private final static String DEFAULT_MONITORING_TOPIC_NAME = "_confluent-monitoring";

  @Test
  public void getInputTopicNames() throws Exception {
    ImmutableSet<String> inTopics = topicStoreMaster.getInputTopicNames();
    assertEquals(2, inTopics.size());
    assertTrue(inTopics.contains(DEFAULT_MONITORING_TOPIC_NAME));
    assertTrue(inTopics.contains(DEFAULT_CMD_TOPIC_NAME));
  }

  @Test
  public void getIntermediateTopicNames() throws Exception {
    ImmutableSet<String> thruTopics = topicStoreMaster.getIntermediateTopicNames();
    assertTrue(thruTopics.size() > 0);
    //No change-log topics should be in the intermediate topic list
    for (String thruTopic : thruTopics) {
      assertFalse(thruTopic.endsWith("-changelog"));
    }
    //No input topics should be in the intermediate topic list
    for (String inTopic : topicStoreMaster.getInputTopicNames()) {
      assertFalse(thruTopics.contains(inTopic));
    }
  }

  @Test
  public void getInternalTopicNames() throws Exception {
    ImmutableSet<String> internalTopics = topicStoreMaster.getInternalTopicNames();
    //All intermediate topics are internal
    assertTrue(internalTopics.containsAll(topicStoreMaster.getIntermediateTopicNames()));
    //No input topics should be in the internal topic list
    for (String inTopic : topicStoreMaster.getInputTopicNames()) {
      assertFalse(internalTopics.contains(inTopic));
    }
    //There should be at least one change-log topic in the list
    Collection<String> changeLogTopics = Collections2.filter(internalTopics, new Predicate<String>() {
      @Override
      public boolean apply(String t) {
        return t.endsWith("-changelog");
      }
    });
    assertTrue(changeLogTopics.size() > 0);
  }

}