package io.confluent.controlcenter.streams.alert;

import java.util.List;

import io.confluent.command.record.alert.CommandAlert.ActionConfig;
import io.confluent.command.record.alert.CommandAlert.EmailAction;
import io.confluent.command.record.alert.CommandAlert.MonitoringTriggerConfig;
import io.confluent.command.record.alert.CommandAlert.TriggerMetricType;
import io.confluent.command.record.alert.CommandAlert.TriggerCondition;

public class CommandAlertUtils {
  public static final String DEFAULT_EMAIL_ADDRESS = "a@b";
  public static final boolean DEFAULT_ACTION_ENABLED = true;
  public static final Long DEFAULT_TRIGGER_LAG_MS = 60000L;

  public static MonitoringTriggerConfig makeTriggerConfig(
      String clusterId,
      String group,
      String guid,
      TriggerMetricType metricType,
      TriggerCondition condition,
      Long value,
      Long lagMs
  ) {
    return MonitoringTriggerConfig.newBuilder()
        .setClusterId(clusterId)
        .setGroup(group)
        .setGuid(guid)
        .setName(guid)
        .setMetric(metricType)
        .setCondition(condition)
        .setLongValue(value)
        .setLagMs(lagMs)
        .build();
  }

  public static MonitoringTriggerConfig makeTriggerConfig(
      String clusterId,
      String group,
      String guid,
      TriggerMetricType metricType,
      TriggerCondition condition,
      Long value
  ) {
    return makeTriggerConfig(clusterId, group, guid, metricType, condition, value, DEFAULT_TRIGGER_LAG_MS);
  }

  public static ActionConfig makeActionConfig(
      String guid,
      int sendRateNumerator,
      long interval,
      List<String> triggerGuids
  ) {
    return ActionConfig.newBuilder()
        .setEnabled(DEFAULT_ACTION_ENABLED)
        .setGuid(guid)
        .setEmail(
            EmailAction.newBuilder()
                .setSubject(guid)
                .setAddress(DEFAULT_EMAIL_ADDRESS)
                .build()
        )
        .setMaxSendRateNumerator(sendRateNumerator)
        .setIntervalMs(interval)
        .addAllTriggerGuid(triggerGuids)
        .build();
  }

}
