EC2 Container Service Deployment
=================

## Launch ##

1. Upload task definitions

	```bash
./define_tasks
```
1. Schedule tasks

	```bash
./start
```

## Shutdown ##

```bash
./stop
```


## Initial Setup ##

1. Install j2 and jq

	```bash
brew install jq
pip install j2cli[yaml]
```

1. Get your AWS keys and export them as environment variables: `AWS_ACCESS_KEY_ID` and `AWS_SECRET_ACCESS_KEY`

1. Install [ECS cli](http://docs.aws.amazon.com/AmazonECS/latest/developerguide/ECS_CLI_installation.html) and configure it.

	```bash
ecs-cli configure --region us-west-2 --access-key $AWS_ACCESS_KEY_ID --secret-key $AWS_SECRET_ACCESS_KEY --cluster c3-integration
```
  
1. Launch a cluster of size 4

  ``` bash
ecs-cli up --keypair c3-dev-keys --capability-iam --size 4 --instance-type m3.medium
```

1. Go to AWS Cloud Formation console to make sure everything was created successfully and take note of

	* VPC id (e.g. vpc-f5822b91)
	* subnets (e.g. subnet-d2e450b6, subnet-445c9232)
	* Security Group (e.g sg-1b2f2c7c)

1. In the AWS console

	* Add an SSH rule to the security group that was created
	* Enable DNS support in the VPC that was created

1. In the EC2 console, get public IPs of the cluster VMs
	* ssh to these VMs with user `ec2-user` and the `c3-dev-keys` keypair.

1. Assign one of them to host Consul
	* Create a private hosted zone in Route 53 for the VPC called `c3.internal`
	* Add an A record for `consul.c3.internal` to point to the Consul IP

1. Create an EBS volume for each cluster node and attach it to the VM
	* Mount, format, and create directories

	```bash
sudo mkfs.ext4 /dev/sdf && sudo mkdir /mnt/ecs && sudo mount /dev/sdf /mnt/ecs
sudo rm -rf /mnt/ecs/* && sudo mkdir -p /mnt/ecs/zookeeper/{1..3}/data  /mnt/ecs/zookeeper/{1..3}/log /mnt/ecs/kafka/{1..3}/data /mnt/ecs/kafka/{1..3}/logs /mnt/ecs/control-center/{1..1}/data
```
	* You must reboot the docker after this for it to see the new mount (see below)

1. Install New Relic (optional) on all nodes

	```bash
sudo rpm -Uvh https://yum.newrelic.com/pub/newrelic/el5/x86_64/newrelic-repo-5-3.noarch.rpm
sudo yum install -y newrelic-sysmond
sudo nrsysmond-config --set license_key=<your license key>
#Amazon Linux uses a non-standard path (/cgroup) for cgroup metrics
sudo vi /etc/newrelic/nrsysmond.cfg
cgroup_root=/cgroup
sudo usermod -a -G docker newrelic
sudo service newrelic-sysmond start
```

1. Restart docker (and the AWS agent) on all nodes

	```bash
sudo service docker restart
docker rm -f ecs-agent; docker run --name ecs-agent \
    --restart always -d \
    -v /var/run/docker.sock:/var/run/docker.sock \
    -v /var/log/ecs:/log \
    -v /var/lib/ecs/data:/data \
    -v /var/lib/docker:/var/lib/docker \
    -v /sys/fs/cgroup:/sys/fs/cgroup:ro \
    -v /var/run/docker/execdriver/native:/var/lib/docker/execdriver/native:ro \
    -p 127.0.0.1:51678:51678 \
    --env-file /etc/ecs/ecs.config \
    -e ECS_LOGFILE=/log/ecs-agent.log \
    -e ECS_DATADIR=/data/ \
    amazon/amazon-ecs-agent
```

1. Launch Consul

	```bash
docker run -d --name=consul --net=host --restart=always gliderlabs/consul-server -bootstrap -advertise=$(curl -s http://169.254.169.254/latest/meta-data/local-ipv4)
```

1. Launch registrator on all nodes

	```bash
docker run -d --name=consul-registrator  --restart=always  -v /var/run/docker.sock:/tmp/docker.sock -h $(curl -s http://169.254.169.254/latest/meta-data/instance-id) gliderlabs/registrator:latest -ip $(curl -s http://169.254.169.254/latest/meta-data/local-ipv4) consul://consul.c3.internal:8500
```
