// @flowfixme
// (Copyright) [2016 - 2016] Confluent, Inc.
import { baseWebpackConfig } from './webpack.config';
process.env.TZ = 'America/Los_Angeles';

export default function karmaES6(config) {
  config.set({
    basePath: '',
    singleRun: true,
    frameworks: ['mocha'],
    reporters: ['mocha', 'coverage'],
    browsers: ['PhantomJS'],
    phantomjsLauncher: {
      // Have phantomjs exit if a ResourceError is encountered (useful if karma exits without killing phantom)
      exitOnResourceError: false
    },

    files: [
      './node_modules/phantomjs-polyfill/bind-polyfill.js',
      {
        pattern: './tests/testEntry.js',
        watched: false,
        served: true,
        included: true
      }
    ],
    preprocessors: {
      './tests/testEntry.js': ['webpack']
    },
    webpack: {
      ...baseWebpackConfig,
      devtool: 'cheap-module-source-map',
      module: {
        ...baseWebpackConfig.module,
        noParse: [
          /\/sinon\.js/
        ],
        preLoaders: [{
          test: /\.(js|jsx)$/,
          include: /src/,
          exclude: /node_modules/,
          loader: 'isparta'
        }],
        loaders: baseWebpackConfig.module.loaders.concat([
          {
            test: /sinon(\\|\/)pkg(\\|\/)sinon\.js/,
            loader: 'imports?define=>false,require=>false'
          }
        ])
      },
      resolve: {
        ...baseWebpackConfig.resolve,
        alias: {
          sinon: 'sinon/pkg/sinon.js'
        }
      },
      externals: {
        'react/addons': true,
        'react/lib/ExecutionEnvironment': true,
        'react/lib/ReactContext': 'window'
      }
    },
    webpackMiddleware: {
      noInfo: true,
      quiet: false,
      stats: {
        color: true,
        chunkModules: false,
        modules: false
      }
    },
    coverageReporter: {
      instrumenterOptions: {
        istanbul: {
          noCompact: true
        }
      },
      dir: 'coverage',
      reporters: [
        {
          type: 'text-summary'
        },
        {
          type: 'html',
          subdir: 'report-html'
        },
        {
          type: 'lcov',
          subdir: 'report-lcov'
        }
      ]
    }
  });
}
