// (Copyright) [2016 - 2016] Confluent, Inc.
// @flow
import browserSync from 'browser-sync';
import express from 'express';
import webpack from 'webpack';
import { webpackConfig } from './webpack.dev';
import webpackHotMiddleware from 'webpack-hot-middleware';
import webpackDevMiddleware from 'webpack-dev-middleware';

const compiler = webpack(webpackConfig);

const app = express();

app.use(require('morgan')('short'));

app.use(express.static('dist'));

app.get('*', (req, res) => {
  res.sendFile(`${__dirname}/dist/index.html`);
});

app.listen(() => {
  browserSync({
    files: [
      'dist/app.css'
    ],
    ghostMode: false,
    open: false,
    proxy: {
      middleware: [
        webpackDevMiddleware(compiler, {
          colors: true,
          publicPath: webpackConfig.output.publicPath,
          noInfo: true,
          quiet: false,
          stats: {
            colors: true
          }
        }),
        webpackHotMiddleware(compiler, {
          log: console.log.bind(console), // eslint-disable-line no-console
          path: '/__webpack_hmr',
          heartbeat: 10 * 1000
        })
      ],
      target: 'http://localhost:9021/',
      proxyReq: [
        function fakeAuthorizationForBrowserSync(proxyReq) {
          if (!proxyReq.getHeader('Authorization')) {
            proxyReq.setHeader('Authorization', 'Basic amF5OmthZmth'); // this is valid auth for jay's user/pass as defined in the README
          }
        }
      ]
    }
  });
});
