// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { makeAction } from '../util';

import type {
  TriggerSummary,
  TriggerInfo,
  AlertSummary,
  ActionSummary,
  ActionInfo,
  TimeGuid
} from 'types/alerts';
import type { Action } from 'types';

import type { MonitoringDataType } from 'types/monitoring';
import type { ClusterId } from 'types/clusters';


export const ALERTS_PREFILL_NEW_TRIGGER = 'ALERTS_PREFILL_NEW_TRIGGER';
export type AlertsPrefillNewTriggerAction = Action<'ALERTS_PREFILL_NEW_TRIGGER', { consumerGroupName: string, clusterId: ClusterId, type: MonitoringDataType }>
export const alertsPrefillNewTrigger = (consumerGroupName: string, clusterId: ClusterId, type: MonitoringDataType): AlertsPrefillNewTriggerAction => makeAction(ALERTS_PREFILL_NEW_TRIGGER, { consumerGroupName, clusterId, type });

export const ALERTS_UPDATE_TRIGGER = 'ALERTS_UPDATE_TRIGGER';
export type AlertsUpdateTriggerAction = Action<'ALERTS_UPDATE_TRIGGER', { guid: string }>
export const alertsUpdateTrigger = (guid: string): AlertsUpdateTriggerAction => makeAction(ALERTS_UPDATE_TRIGGER, { guid });

export const ALERTS_UPDATE_TRIGGER_FROM_STATE = 'ALERTS_UPDATE_TRIGGER_FROM_STATE';
export type AlertsUpdateTriggerFromStateAction = Action<'ALERTS_UPDATE_TRIGGER_FROM_STATE', { guid: string }>
export const alertsUpdateTriggerFromState = (guid: string): AlertsUpdateTriggerFromStateAction => makeAction(ALERTS_UPDATE_TRIGGER_FROM_STATE, { guid });

export const ALERTS_TRIGGER_TO_DELETE = 'ALERTS_TRIGGER_TO_DELETE';
export type AlertsTriggerToDeleteAction = Action<'ALERTS_TRIGGER_TO_DELETE', { guid: string }>
export const alertsTriggerToDelete = (guid: string): AlertsTriggerToDeleteAction => makeAction(ALERTS_TRIGGER_TO_DELETE, { guid });

export const ALERTS_DELETE_TRIGGER = 'ALERTS_DELETE_TRIGGER';
export type AlertsDeleteTriggerAction = Action<'ALERTS_DELETE_TRIGGER', { guid: string }>
export const alertsDeleteTrigger = (guid: string): AlertsDeleteTriggerAction => makeAction(ALERTS_DELETE_TRIGGER, { guid });

export const ALERTS_CREATE_ACTION = 'ALERTS_CREATE_ACTION';
export type AlertsCreateActionAction = Action<'ALERTS_CREATE_ACTION', void>
export const alertsCreateAction = (): AlertsCreateActionAction => makeAction(ALERTS_CREATE_ACTION);

export const ALERTS_CREATE_ACTION_FROM_STATE = 'ALERTS_CREATE_ACTION_FROM_STATE';
export type AlertsCreateActionFromStateAction = Action<'ALERTS_CREATE_ACTION_FROM_STATE', void>
export const alertsCreateActionFromState = (): AlertsCreateActionFromStateAction => makeAction(ALERTS_CREATE_ACTION_FROM_STATE);

export const ALERTS_UPDATE_ACTION = 'ALERTS_UPDATE_ACTION';
export type AlertsUpdateActionAction = Action<'ALERTS_UPDATE_ACTION', { guid: string }>
export const alertsUpdateAction = (guid: string): AlertsUpdateActionAction => makeAction(ALERTS_UPDATE_ACTION, { guid });

export const ALERTS_UPDATE_ACTION_FROM_STATE = 'ALERTS_UPDATE_ACTION_FROM_STATE';
export type AlertsUpdateActionFromStateAction = Action<'ALERTS_UPDATE_ACTION_FROM_STATE', { guid: string }>
export const alertsUpdateActionFromState = (guid: string): AlertsUpdateActionFromStateAction => makeAction(ALERTS_UPDATE_ACTION_FROM_STATE, { guid });

export const ALERTS_ACTION_TO_DELETE = 'ALERTS_ACTION_TO_DELETE';
export type AlertsActionToDeleteAction = Action<'ALERTS_ACTION_TO_DELETE', { guid: string }>
export const alertsActionToDelete = (guid: string): AlertsActionToDeleteAction => makeAction(ALERTS_ACTION_TO_DELETE, { guid });

export const ALERTS_DELETE_ACTION = 'ALERTS_DELETE_ACTION';
export type AlertsDeleteActionAction = Action<'ALERTS_DELETE_ACTION', { guid: string }>
export const alertsDeleteAction = (guid: string): AlertsDeleteActionAction => makeAction(ALERTS_DELETE_ACTION, { guid });

export const ALERTS_CREATE_TRIGGER_FROM_STATE = 'ALERTS_CREATE_TRIGGER_FROM_STATE';
export type AlertsCreateTriggerFromStateAction = Action<'ALERTS_CREATE_TRIGGER_FROM_STATE', void>
export const alertsCreateTriggerFromState = (): AlertsCreateTriggerFromStateAction => makeAction(ALERTS_CREATE_TRIGGER_FROM_STATE);

export const ALERTS_TRIGGER_LAST_CREATED = 'ALERTS_TRIGGER_LAST_CREATED';
export type AlertsTriggerLastCreatedAction = Action<'ALERTS_TRIGGER_LAST_CREATED', { triggerInfo: TriggerInfo }>
export const alertsTriggerLastCreated = (triggerInfo: TriggerInfo): AlertsTriggerLastCreatedAction => makeAction(ALERTS_TRIGGER_LAST_CREATED, { triggerInfo });

export const ALERTS_TRIGGER_ADD_ACTIONS_FROM_STATE = 'ALERTS_TRIGGER_ADD_ACTIONS_FROM_STATE';
export type AlertsTriggerAddActionsFromStateAction = Action<'ALERTS_TRIGGER_ADD_ACTIONS_FROM_STATE', { triggerGuid: string }>
export const alertsTriggerAddActionsFromState = (triggerGuid: string): AlertsTriggerAddActionsFromStateAction => makeAction(ALERTS_TRIGGER_ADD_ACTIONS_FROM_STATE, { triggerGuid });


export const ALERTS_REQUEST_TRIGGERS = 'ALERTS_REQUEST_TRIGGERS';
export type AlertsRequestTriggersAction = Action<'ALERTS_REQUEST_TRIGGERS', void>
export const ALERTS_RECEIVE_TRIGGERS = 'ALERTS_RECEIVE_TRIGGERS';
export type AlertsReceiveTriggersAction = Action<'ALERTS_RECEIVE_TRIGGERS', { data: Array<TriggerSummary> }>
export const alertsTriggers = {
  request: (): AlertsRequestTriggersAction => makeAction(ALERTS_REQUEST_TRIGGERS),
  receive: (triggers: Array<TriggerSummary>): AlertsReceiveTriggersAction => makeAction(ALERTS_RECEIVE_TRIGGERS, { data: triggers })
};

export const TriggerStateUpdateMethods = {
  REDUX_FORM: 'REDUX_FORM',
  SELECTABLE_LIST: 'SELECTABLE_LIST'
};

export type TriggerStateUpdateMethod = $Enum<typeof TriggerStateUpdateMethods>;

export const ALERTS_REQUEST_TRIGGER = 'ALERTS_REQUEST_TRIGGER';
export type AlertsRequestTriggerAction = Action<'ALERTS_REQUEST_TRIGGER', { id: string, stateUpdateMethod: TriggerStateUpdateMethod }>
export const ALERTS_RECEIVE_TRIGGER = 'ALERTS_RECEIVE_TRIGGER';
export type AlertsReceiveTriggerAction = Action<'ALERTS_RECEIVE_TRIGGER', { data: TriggerInfo }>
export const alertsTrigger = {
  request: (id: string, stateUpdateMethod: TriggerStateUpdateMethod): AlertsRequestTriggerAction => makeAction(ALERTS_REQUEST_TRIGGER, { id, stateUpdateMethod }),
  receive: (trigger: TriggerInfo): AlertsReceiveTriggerAction => makeAction(ALERTS_RECEIVE_TRIGGER, { data: trigger })
};

export const ALERTS_REQUEST_ACTIONS = 'ALERTS_REQUEST_ACTIONS';
export type AlertsRequestActionsAction = Action<'ALERTS_REQUEST_ACTIONS', void>
export const ALERTS_RECEIVE_ACTIONS = 'ALERTS_RECEIVE_ACTIONS';
export type AlertsReceiveActionsAction = Action<'ALERTS_RECEIVE_ACTIONS', { data: Array<ActionSummary> }>
export const alertsActions = {
  request: (): AlertsRequestActionsAction => makeAction(ALERTS_REQUEST_ACTIONS),
  receive: (actions: Array<ActionSummary>): AlertsReceiveActionsAction => makeAction(ALERTS_RECEIVE_ACTIONS, { data: actions })
};

export const ALERTS_REQUEST_ACTION = 'ALERTS_REQUEST_ACTION';
export type AlertsRequestActionAction = Action<'ALERTS_REQUEST_ACTION', { id: string }>
export const ALERTS_RECEIVE_ACTION = 'ALERTS_RECEIVE_ACTION';
export type AlertsReceiveActionAction = Action<'ALERTS_RECEIVE_ACTION', { data: ActionInfo }>
export const alertsAction = {
  request: (id: string): AlertsRequestActionAction => makeAction(ALERTS_REQUEST_ACTION, { id }),
  receive: (action: ActionInfo): AlertsReceiveActionAction => makeAction(ALERTS_RECEIVE_ACTION, { data: action })
};

export type HistoryRequestType = 'MAIN' | 'NEW' | 'SPECIFIC';

export type FetchHistorySpecType = {
  type: HistoryRequestType,
  itemId: ?TimeGuid,
  count: number
}

export const ALERTS_REQUEST_HISTORY = 'ALERTS_REQUEST_HISTORY';
export type AlertsRequestHistoryAction = Action<'ALERTS_REQUEST_HISTORY', FetchHistorySpecType>
export const ALERTS_RECEIVE_HISTORY = 'ALERTS_RECEIVE_HISTORY';
export type AlertsReceiveHistoryAction = Action<'ALERTS_RECEIVE_HISTORY', { spec: FetchHistorySpecType, data: Array<Object> }>
export const alertsHistory = {
  request: (spec: FetchHistorySpecType): AlertsRequestHistoryAction => makeAction(ALERTS_REQUEST_HISTORY, spec),
  receive: (spec: FetchHistorySpecType, data: Array<AlertSummary>): AlertsReceiveHistoryAction => makeAction(ALERTS_RECEIVE_HISTORY, { spec, data })
};

export type FetchAlertsMonitoringDataType = {
  startTimeMs: number,
  stopTimeMs: number,
  clusterId: ClusterId,
  consumerGroup: string
};

export const ALERTS_REQUEST_MONITORING_DATA = 'ALERTS_REQUEST_MONITORING_DATA';
export type AlertsRequestMonitoringDataAction = Action<'ALERTS_REQUEST_MONITORING_DATA', FetchAlertsMonitoringDataType>
export const ALERTS_RECEIVE_MONITORING_DATA = 'ALERTS_RECEIVE_MONITORING_DATA';
export type AlertsReceiveMonitoringDataAction = Action<'ALERTS_RECEIVE_MONITORING_DATA', { spec: FetchAlertsMonitoringDataType, data: Object }>
export const alertsMonitoringData = {
  request: (spec: FetchAlertsMonitoringDataType): AlertsRequestMonitoringDataAction => makeAction(ALERTS_REQUEST_MONITORING_DATA, spec),
  receive: (spec: FetchAlertsMonitoringDataType, data: Object): AlertsReceiveMonitoringDataAction => makeAction(ALERTS_RECEIVE_MONITORING_DATA, { spec, data })
};


export const ALERTS_PREPEND_NEW_HISTORY = 'ALERTS_PREPEND_NEW_HISTORY';
export type AlertsPrependNewHistoryAction = Action<'ALERTS_PREPEND_NEW_HISTORY', void>
export const alertsPrependNewHistory = (): AlertsPrependNewHistoryAction => makeAction(ALERTS_PREPEND_NEW_HISTORY);

export const ALERTS_CLUSTERID_CONSUMER_GROUP_OPTIONS_LOADED = 'ALERTS_CLUSTERID_CONSUMER_GROUP_OPTIONS_LOADED';
export type AlertsClusterIdConsumerGroupOptionsLoadedAction = Action<'ALERTS_CLUSTERID_CONSUMER_GROUP_OPTIONS_LOADED', void>
export const alertsClusterIdConsumerGroupOptionsLoaded = (): AlertsClusterIdConsumerGroupOptionsLoadedAction => makeAction(ALERTS_CLUSTERID_CONSUMER_GROUP_OPTIONS_LOADED);

export const ALERTS_CREATE_FIRST_ACTION = 'ALERTS_CREATE_FIRST_ACTION';
export type AlertsCreateFirstAction = Action<'ALERTS_CREATE_FIRST_ACTION', void>
export const alertsCreateFirstAction = (): AlertsCreateFirstAction => makeAction(ALERTS_CREATE_FIRST_ACTION);
