// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { makeAction } from 'actions/util';

import type { Action } from 'types';
import type {
  Cluster,
  ClusterId,
  ClusterList,
  ClusterType,
  ClustersResponse
} from 'types/clusters';

export const CLUSTERS_FETCH_FAILURE = 'CLUSTERS_FETCH_FAILURE';
export const CLUSTERS_FETCH_REQUEST = 'CLUSTERS_FETCH_REQUEST';
export const CLUSTERS_FETCH_SUCCESS = 'CLUSTERS_FETCH_SUCCESS';

export type ClusterFetchFailureAction = Action<'CLUSTERS_FETCH_FAILURE', { error: Object }>;
export type ClusterFetchRequestAction = Action<'CLUSTERS_FETCH_REQUEST', { clusterType: ClusterType }>;
export type ClusterFetchSuccessAction = Action<'CLUSTERS_FETCH_SUCCESS', ClustersResponse>;

export const clustersFetch = {
  failure: (error: Error): ClusterFetchFailureAction => makeAction(CLUSTERS_FETCH_FAILURE, { error }),
  request: (clusterType: ClusterType): ClusterFetchRequestAction => makeAction(CLUSTERS_FETCH_REQUEST, { clusterType }),
  success: (clusterType: ClusterType, defaultClusterId: ClusterId, clusters: ClusterList): ClusterFetchSuccessAction => (
    makeAction(CLUSTERS_FETCH_SUCCESS, { clusters, clusterType, defaultClusterId })
  )
};

export const LOAD_CLUSTERS = 'LOAD_CLUSTERS';
export type LoadClustersAction = Action<'LOAD_CLUSTERS', { clusterType: ClusterType }>;
export const loadClusters = (clusterType: ClusterType): LoadClustersAction => makeAction(LOAD_CLUSTERS, { clusterType });

export const SELECT_CLUSTER = 'SELECT_CLUSTER';
export type ClusterSelectedAction = Action<'SELECT_CLUSTER', { clusterType: ClusterType, selectedClusterId: ClusterId}>;
export const selectCluster = (clusterType: ClusterType, selectedClusterId: ClusterId): ClusterSelectedAction => makeAction(SELECT_CLUSTER, { clusterType, selectedClusterId });

export const ALL_CLUSTERS_FETCH_FAILURE = 'ALL_CLUSTERS_FETCH_FAILURE';
export const ALL_CLUSTERS_FETCH_REQUEST = 'ALL_CLUSTERS_FETCH_REQUEST';
export const ALL_CLUSTERS_FETCH_SUCCESS = 'ALL_CLUSTERS_FETCH_SUCCESS';

export type AllClustersFetchFailureAction = Action<'ALL_CLUSTERS_FETCH_FAILURE', { error: Object }>;
export type AllClustersFetchRequestAction = Action<'ALL_CLUSTERS_FETCH_REQUEST', void>;
export type AllClustersFetchSuccessAction = Action<'ALL_CLUSTERS_FETCH_SUCCESS', { clusters: ClusterList }>;

export const allClustersFetch = {
  failure: (error: Error): AllClustersFetchFailureAction => makeAction(ALL_CLUSTERS_FETCH_FAILURE, { error }),
  request: (): AllClustersFetchRequestAction => makeAction(ALL_CLUSTERS_FETCH_REQUEST),
  success: (clusters: ClusterList): AllClustersFetchSuccessAction => (
    makeAction(ALL_CLUSTERS_FETCH_SUCCESS, { clusters })
  )
};

export const SAVE_CLUSTER_FAILURE = 'SAVE_CLUSTER_FAILURE';
export const SAVE_CLUSTER_REQUEST = 'SAVE_CLUSTER_REQUEST';
export const SAVE_CLUSTER_SUCCESS = 'SAVE_CLUSTER_SUCCESS';

export type SaveClusterFailureAction = Action<'SAVE_CLUSTER_FAILURE', { error: Object }>;
export type SaveClusterRequestAction = Action<'SAVE_CLUSTER_REQUEST', { cluster: $Shape<Cluster> }>;
export type SaveClusterSuccessAction = Action<'SAVE_CLUSTER_SUCCESS', { cluster: $Shape<Cluster> }>;

export const saveCluster = {
  failure: (error: Error): SaveClusterFailureAction => makeAction(SAVE_CLUSTER_FAILURE, { error }),
  request: (cluster: $Shape<Cluster>): SaveClusterRequestAction => makeAction(SAVE_CLUSTER_REQUEST, { cluster }),
  success: (cluster: $Shape<Cluster>): SaveClusterSuccessAction => (
    makeAction(SAVE_CLUSTER_SUCCESS, { cluster })
  )
};

export const LOAD_CLUSTER_INDEX = 'LOAD_CLUSTER_INDEX';
export type LoadClusterIndexAction = Action<'LOAD_CLUSTER_INDEX', void>;
export const loadClusterIndex = (): LoadClusterIndexAction => makeAction(LOAD_CLUSTER_INDEX);

export const LOAD_CLUSTER_EDIT_FORM = 'LOAD_CLUSTER_EDIT_FORM';
export type LoadClusterEditFormAction = Action<'LOAD_CLUSTER_EDIT_FORM', { clusterId: ClusterId }>;
export const loadClusterEditForm = (clusterId: ClusterId): LoadClusterEditFormAction => makeAction(LOAD_CLUSTER_EDIT_FORM, { clusterId });

export const SAVE_CLUSTER_EDIT_FORM = 'SAVE_CLUSTER_EDIT_FORM';
export type SaveClusterEditFormAction = Action<'SAVE_CLUSTER_EDIT_FORM', void>;
export const saveClusterEditForm = (): SaveClusterEditFormAction => makeAction(SAVE_CLUSTER_EDIT_FORM);
