// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import debug from 'debug';
import {
  getJson,
  makeAction
} from './util';

import type {
  Action,
  Dispatchable,
  Dispatcher
} from 'types';

const log = debug('controlcenter:actions:license');


export const RECEIVE_LICENSE_DATA = 'RECEIVE_LICENSE_DATA';
export type ReceiveLicenseDataAction = Action<'RECEIVE_LICENSE_DATA', { expiration?: number }>

export function loadLicenseData(): Dispatchable {
  return (dispatch: Dispatcher) => {
    const reqPath = '/2.0/license';
    return getJson(reqPath)
      .then(data => {
        dispatch(makeAction(RECEIVE_LICENSE_DATA, data));
      }).catch(err => {
        log(`error looking up path=${reqPath} status=${err}`);
        dispatch(makeAction(RECEIVE_LICENSE_DATA, {}));
      });
  };
}
