// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import 'babel-polyfill';

import React from 'react';
import ReactDOM from 'react-dom';
import { browserHistory } from 'react-router';
import { syncHistoryWithStore } from 'react-router-redux';
import debug from 'debug';

import './styles/global.css';
import './styles/react-virtualized.less';

import AppContainer from './containers/AppContainer';

import createStore from './store/createStore';
import rootSaga from './sagas';

// ========================================================
// Store and History Instantiation
// ========================================================
// Create redux store and sync with react-router-redux. We have installed the
// react-router-redux reducer under the routerKey "router" in src/routes/index.js,
// so we need to provide a custom `selectLocationState` to inform
// react-router-redux of its location.
const initialState = window.__INITIAL_STATE__;
const store = createStore(initialState, browserHistory);
const history = syncHistoryWithStore(browserHistory, store);

// ========================================================
// Developer Tools Setup
// ========================================================
if (__DEV__) {
  window.debug = debug;
  debug.enable('controlcenter:*');
}

// ========================================================
// Render Setup
// ========================================================
const MOUNT_NODE = document.getElementById('controlcenter');

store.runSaga(rootSaga);

let render = (routerKey = null) => {

  // const routes = require('./routes/index').default(store);

  ReactDOM.render(
    <AppContainer
      history={history}
      routerKey={routerKey}
      store={store}
    />,
    MOUNT_NODE
  );
};

// Enable HMR and catch runtime errors in RedBox
// This code is excluded from production bundle
if (__DEV__ && module.hot && __USE_REDBOX__) {
  const renderApp = render;
  const renderError = (error) => {
    const RedBox = require('redbox-react').default;

    ReactDOM.render(
      <RedBox
        error={error}
        editorScheme={process.env.EDITOR_SCHEME}
        useLines={false}
        useColumns={false}
        style={{
          redbox: {
            fontFamily: 'inherit',
            position: 'fixed',
            padding: '2rem',
            top: 0,
            left: 0,
            bottom: 0,
            right: 0,
            width: '100%',
            background: '#ef5730',
            color: '#ffffff',
            zIndex: 9999,
            fontSize: '2.2rem',
            lineHeight: 1.4
          },
          message: {
            fontWeight: '400'
          },
          stack: {
            fontFamily: 'menlo',
            marginTop: '2em'
          },
          frame: {
            marginTop: '1rem'
          },
          file: {
            fontSize: '1.6rem',
            color: '#a7a7a8'
          },
          linkToFile: {
            color: '#5d5d60'
          }
        }}
      />, MOUNT_NODE
    );
  };
  render = () => {
    try {
      renderApp(Math.random());
    } catch (error) {
      renderError(error);
    }
  };
}

// ========================================================
// Go!
// ========================================================
render();
