// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { FlexColumn } from 'react-virtualized';
import { browserHistory } from 'react-router';
import { connect } from 'react-redux';

import * as clusterActions from 'actions/clusters';
import styles from './ClusterIndex.css';
import { BigDataTable, CellContextMenu, CellPillRenderer } from 'components/table';
import { ContentPanel } from 'components/layout';
import { clusterIndexSelector } from 'selectors/clusters';

import type { CellRenderer } from 'types/flex-table';
import type { ClusterList } from 'types/clusters';

export type ClusterIndexProps = {
  clusters: ClusterList,
  loadClusterIndex: Function
};

const contextMenuRenderer = ({ rowData }: CellRenderer) => (
  <CellContextMenu
    menuItems={[
      {
        icon: 'settingsSm',
        label: 'Edit',
        onActivate: () => browserHistory.push(`/management/clusters/${rowData.clusterId}/edit`)
      }
    ]}
  />
);

const dataKeyOrClusterIdRenderer = ({ rowData }) => rowData.displayName || rowData.clusterId;

export class ClusterIndex extends Component {
  props: ClusterIndexProps;

  componentDidMount() {
    this.props.loadClusterIndex();
  }

  render() {
    const { props } = this;
    return (
      <div className={styles['cluster-index-root']}>
        <ContentPanel>
          <BigDataTable
            height={400}
            rowData={props.clusters}
          >
            <FlexColumn
              rowHeader={true}
              label="Cluster name"
              dataKey="nothing"
              cellDataGetter={dataKeyOrClusterIdRenderer}
              width={190}
            />
            <FlexColumn
              disableSort={true}
              rowHeader={true}
              dataKey="nothing"
              width={25}
              cellRenderer={contextMenuRenderer}
            />
            <FlexColumn
              rowHeader={false}
              label="Zookeeper servers"
              dataKey="zookeeperConnect"
              cellRenderer={CellPillRenderer}
              width={300}
            />
            <FlexColumn
              rowHeader={false}
              label="Bootstrap servers"
              dataKey="bootstrapServers"
              cellRenderer={CellPillRenderer}
              width={300}
            />
            <FlexColumn
              rowHeader={false}
              label="GUID"
              dataKey="clusterId"
              width={190}
            />
          </BigDataTable>
        </ContentPanel>
      </div>
    );
  }
}

const mapStateToProps = (state) => ({
  clusters: clusterIndexSelector(state)
});

export default connect(
  mapStateToProps,
  clusterActions
)(ClusterIndex);
