// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { connect } from 'react-redux';

import * as clusterActions from 'actions/clusters';
import Dropdown from 'components/Dropdown';
import styles from './ClusterSelectorDropdown.css';
import {
  getClusterOptionsSelector,
  getSelectedClusterOptionSelector
} from 'selectors/clusters';

import type { ActionCreator } from 'types';
import type { ClusterType, ClusterOption, ClusterOptions } from 'types/clusters';

export type ClusterSelectorDropdownProps = {
  clusterType: ClusterType,
  clusters: ClusterOptions,
  loadClusters: ActionCreator,
  selectCluster: ActionCreator,
  selectedCluster: ClusterOption
};

export class ClusterSelectorDropdown extends Component {
  props: ClusterSelectorDropdownProps;

  componentDidMount() {
    this.props.loadClusters(this.props.clusterType);
  }

  render() {
    const {
      clusterType,
      clusters,
      selectCluster,
      selectedCluster
    } = this.props;

    if (!clusters.length) {
      return null;
    }

    return (
      <div className={styles['cluster-selector-dropdown-root']}>
        <Dropdown
          valueRenderer={option => `Cluster: ${option.label}`}
          searchable={false}
          clearable={false}
          onChange={option => selectCluster(clusterType, option.value)}
          multi={false}
          disabled={clusterType === 'CONNECT'}
          options={clusters}
          value={selectedCluster}
        />
      </div>
    );
  }
}

const mapStateToProps = (state: Object, { clusterType }: ClusterSelectorDropdownProps) => ({
  clusters: getClusterOptionsSelector(clusterType)(state),
  selectedCluster: getSelectedClusterOptionSelector(clusterType)(state)
});

export default connect(
  mapStateToProps,
  clusterActions
)(ClusterSelectorDropdown);
