// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import Sticky from 'react-stickynode';
import { connect } from 'react-redux';

import ClusterSelectorDropdown from 'components/ClusterSelectorDropdown';
import LicenseExpired from 'components/LicenseExpired';
import styles from './ConnectLayout.css';
import { Header, Breadcrumbs } from 'components/header';
import { showLicenseExpiredSelector } from 'selectors/license';

export type ConnectLayoutProps = {
  content?: any,
  params: Object,
  routes: Array<Object>,
  showLicenseExpired?: boolean,
  subnav?: any,
  topLinks?: any
};

export const ConnectLayout = ({
  content,
  params,
  routes,
  showLicenseExpired,
  subnav,
  topLinks
}: ConnectLayoutProps) => showLicenseExpired ? (
  <LicenseExpired />
) : (
  <div className={styles.root}>
    <Header
      bottomLeft={<Breadcrumbs routes={routes} params={params} />}
      topRight={<ClusterSelectorDropdown clusterType="CONNECT" />}
    />
    <Sticky top={0}>
      <div className={styles.sticky}>
        {subnav}
        {topLinks && (
          <div className={styles['top-links']}>
            {topLinks}
          </div>
        )}
      </div>
    </Sticky>
    {content}
  </div>
);

export default connect(
  (state) => ({ showLicenseExpired: showLicenseExpiredSelector(state) })
)(ConnectLayout);
