// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { FlexTable, FlexColumn } from 'react-virtualized';

import styles from './PopoverMenu.css';
import { Icon, IconNames } from 'components/icon';
import { Popover } from 'components/tooltip';

import type { MenuItem } from 'types';

export type PopoverMenuProps = {
  preferPlace?: null | 'above' | 'right' | 'below' | 'left' | 'row' | 'column' | 'start' | 'end',
  hasIcons: boolean,
  children?: any,

  menuItems: Array<MenuItem>
};

export class PopoverMenu extends Component {

  props: PopoverMenuProps;

  state: {
    isOpen: boolean
  };

  static defaultProps = {
    hasIcons: false,
    menuItems: []
  };

  constructor() {
    super();
    this.state = {
      isOpen: false
    };
  }

  renderList() {
    const { menuItems, hasIcons } = this.props;
    const rowHeight = 26;
    const height = Math.min(menuItems.length * rowHeight, 400);

    return (
      <div className={styles.list}>
        <FlexTable
          disableHeader={true}
          rowClassName={({ index }) => {
            const item = menuItems[index];
            return item && typeof item.onActivate === 'function' ? styles.clickableRow : null;
          }}
          onRowClick={({ index }) => {
            const item = menuItems[index];
            if (item && typeof item.onActivate === 'function') {
              item.onActivate();
            }
          }}
          rowGetter={({ index }) => menuItems[index]}
          rowWrapperClassName={({ index }) => `Grid__cell-${(index + 1) % 2 === 0 ? 'even' : 'odd'}`}
          estimatedRowHeight={rowHeight}
          height={height}
          rowData={menuItems}
          rowHeight={rowHeight}
          width={132}
          rowCount={menuItems.length}
        >
          {hasIcons && (
            <FlexColumn
              dataKey="icon"
              cellRenderer={({ cellData }) => (cellData in IconNames) ? (
                <Icon
                  name={cellData}
                  size={16}
                />
              ) : null}
              width={16}
              flexGrow={0}
            />
          )}
          <FlexColumn
            dataKey="label"
            width={hasIcons ? 132 - 36 : 132}
          />
        </FlexTable>
      </div>
    );
  }

  render() {
    return (
      <Popover
        className={styles['menu-popover']}
        body={this.renderList()}
        preferPlace={typeof this.props.preferPlace === 'undefined' ? null : this.props.preferPlace}
        isOpen={this.state.isOpen}
        onOuterAction={() => this.setState({ isOpen: false })}
      >
        <span
          className={styles[this.state.isOpen ? 'button-open' : 'button']}
          onClick={() => this.setState({ isOpen: !this.state.isOpen })}
        >
          {this.props.children}
        </span>
      </Popover>
    );
  }
}

export default PopoverMenu;
