// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { connect } from 'react-redux';
import React, { Component } from 'react';
import { createSelector } from 'reselect';
import { sumArrayUntil } from 'utils';
import styles from 'components/aggregate-stats/AggregateStats.css';

import {
  consumedLatencyDataSelector,
  ignoreDeliveryDiscrepancyAfterIndexSelector
} from 'selectors/monitoring/data';

import type {
  MultiTimeSeries,
  RawLatencyData
} from 'types/monitoring/data';


export function calculateAggregateLatency(consumedLatency: MultiTimeSeries<RawLatencyData>, ignoreAfterIndex: ?number): ?number {
  if (consumedLatency && ignoreAfterIndex) {
    const latencyTotals = consumedLatency.values.totalLatency;
    const consumedCountValues = consumedLatency.values.count;
    const totalMessageCount = sumArrayUntil(consumedCountValues, ignoreAfterIndex);
    const totalLatency = sumArrayUntil(latencyTotals, ignoreAfterIndex);
    return totalMessageCount > 0
      ? totalLatency / totalMessageCount / 1000
      : null;
  }
  return null;
}

export const aggregateLatencySelector = createSelector(
  [consumedLatencyDataSelector, ignoreDeliveryDiscrepancyAfterIndexSelector],
  (consumedLatency: Object, ignoreAfterIndex: ?number): ?number => {
    return calculateAggregateLatency(consumedLatency, ignoreAfterIndex);
  }
);

export class AggregateLatency extends Component {
  props: {
    aggregateLatency: ?number
  };

  render() {
    return (
      <div>
        {this.props.aggregateLatency && (
          <div className={styles['item']}>
            <span className={styles['item-stat']}>{`${this.props.aggregateLatency.toFixed(2)}s`}</span>
            <span>Average latency</span>
          </div>
        )}
      </div>
    );
  }
}

function mapStateToProps(state) {
  return {
    aggregateLatency: aggregateLatencySelector(state)
  };
}

export default connect(mapStateToProps)(AggregateLatency);
