// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { Link } from 'react-router';
import { FlexColumn } from 'react-virtualized';
import { CellOverflowHidden } from 'components/table';
import { ContentPanel, TableContentPanel } from 'components/layout';
import EmptyListPrompt from 'components/EmptyListPrompt';
import ConfirmationModal from 'components/modal/ConfirmationModal';
import {
  alertsActions,
  alertsDeleteAction,
  alertsActionToDelete
} from 'actions/alerts';
import { ACTIONS_INDEX_PATHNAME, NEW_ACTION_PATHNAME } from 'constants/alerts';
import styles from './Actions.css';
import {
  minutesToFrequencySpec,
  frequencySpecToString
} from 'utils/time';
import { setVisibleModal } from 'actions/layout';
import { actionToDelete } from 'selectors/alerts';

export class Actions extends Component {
  props: {
    requestActions: Function,
    deleteAction: Function,
    setVisibleModal: Function,
    setActionToDelete: Function,
    actions: Array<Object> | null,
    toDelete: Object
  };

  handleDeleteClicked(guid: string, e: SyntheticMouseEvent) {
    e.preventDefault();
    this.props.setActionToDelete(guid);
    this.props.setVisibleModal('actionDelete');
  }

  handleDelete() {
    this.props.deleteAction(this.props.toDelete.guid);
    this.props.setVisibleModal(null);
  }

  componentDidMount() {
    this.props.requestActions();
  }

  render() {
    const actions = this.props.actions;
    const rowGetter = ({ index }) => actions ? actions[index] : null;
    return (
      <div>
        {actions != null && actions.length === 0
          ? <ContentPanel>
              <EmptyListPrompt
                buttonRoute={{ pathname: NEW_ACTION_PATHNAME }}
                buttonText="Add an action"
                header="You haven't added any actions yet"
                text="Take action on the cluster events you care about"
              />
            </ContentPanel>
          : actions == null
              ? null
              : <TableContentPanel rowData={actions}>
                  <FlexColumn label="Name" dataKey="name" rowGetter={rowGetter} width={50} flexGrow={1} cellRenderer={CellOverflowHidden} />
                  <FlexColumn label="Max frequency" dataKey="maxFrequency" rowGetter={rowGetter} width={100} />
                  <FlexColumn label="Triggers" dataKey="triggerCount" rowGetter={rowGetter} width={100} />
                  <FlexColumn label="Status" dataKey="status" rowGetter={rowGetter} width={100} />
                  <FlexColumn dataKey="guid" width={80} cellRenderer={(item) =>
                    <div>
                      <Link to={`${ACTIONS_INDEX_PATHNAME}/${item.cellData}/edit`}>Edit</Link>
                      <a
                        className={styles['delete-link']}
                        href="#"
                        onClick={this.handleDeleteClicked.bind(this, item.cellData)}
                      >
                        Delete
                      </a>
                    </div>
                  } />
                </TableContentPanel>
        }
        <ConfirmationModal
          id="actionDelete"
          text={`Are you sure you want to delete action: ${this.props.toDelete.name}?`}
          onConfirm={this.handleDelete.bind(this)}
        />
      </div>
    );
  }
}

export default connect(
  (state) => {
    const actions = state.alerts.actions == null
      ? null
      : state.alerts.actions.map(action => ({
          guid: action.guid,
          name: action.name,
          maxFrequency: frequencySpecToString(minutesToFrequencySpec(action.maxFrequencyPerDay)),
          triggerCount: action.triggerCount,
          status: action.status
        }));
    const result = {
      actions,
      toDelete: actionToDelete(state)
    };
    return result;
  },
  {
    requestActions: alertsActions.request,
    deleteAction: alertsDeleteAction,
    setActionToDelete: alertsActionToDelete,
    setVisibleModal
  }
)(Actions);
