// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { FlexColumn } from 'react-virtualized';
import { Link } from 'react-router';

import withSelectionList, { selectors, actions } from 'components/withSelectionList';

import { SizeEmittingContentPanel } from 'components/layout';
import {
  BigDataTable,
  CellOverflowHidden
} from 'components/table';
import { ActionBar } from 'components/navigation';
import {
  ButtonPrimaryLarge,
  ButtonSecondaryLarge
} from 'components/button';

import {
  alertsActions,
  alertsTrigger,
  alertsTriggerAddActionsFromState,
  TriggerStateUpdateMethods
} from 'actions/alerts';

import {
  ROW_HEIGHT_PX,
  HEADER_HEIGHT_PX,
  calculateTableHeight
} from 'components/layout/TableContentPanel';

import { TRIGGERS_INDEX_PATHNAME } from 'constants/alerts';

import styles from './AddActions.css';


export const LIST_NAME_ACTIONS = 'ACTIONS';

const HeightAdjustableTable = (actions, availableHeight) => {
  const rowGetter = ({ index }) => actions[index];

  const Table = (props) =>
    <BigDataTable
      rowData={props.rowData}
      height={calculateTableHeight(availableHeight, props.rowData.length)}
      rowHeight={ROW_HEIGHT_PX}
      headerHeight={HEADER_HEIGHT_PX}
    >
      <FlexColumn
        width={25}
        {...props.flexColumnProps}
      />
      <FlexColumn
        label="Action Name"
        dataKey="name"
        rowGetter={rowGetter}
        width={110}
        flexGrow={1}
        cellRenderer={CellOverflowHidden}
      />
    </BigDataTable>;

  const SelectableTable = withSelectionList(Table, 'guid', LIST_NAME_ACTIONS);

  return (
    <div className={styles.root}>
      <SelectableTable rowData={actions} />
    </div>
  );
};

export class AddActions extends Component {

  handleSetActionsClick() {
    this.props.alertsTriggerAddActionsFromState(this.props.routeParams.triggerid);
  }

  componentDidMount() {
    const { requestActions } = this.props;
    if (!requestActions) {
      return;
    }
    this.props.initializeSelectedList(LIST_NAME_ACTIONS);
    this.props.requestActions();
    this.props.requestTrigger(this.props.routeParams.triggerid, TriggerStateUpdateMethods.SELECTABLE_LIST);
  }

  render() {
    const actionCount = this.props.selectedItems == null
      ? 0
      : Object.getOwnPropertyNames(this.props.selectedItems).length;

    return (
      <div>
        <SizeEmittingContentPanel>
          {HeightAdjustableTable.bind(null, this.props.actions)}
        </SizeEmittingContentPanel>
        <ActionBar>
          <ButtonPrimaryLarge
            text={`Set ${actionCount === 0 ? '' : actionCount} ${actionCount === 1 ? 'action' : 'actions'}`}
            disabled={actionCount === 0}
            onClick={this.handleSetActionsClick.bind(this)}
          />
          <Link to={TRIGGERS_INDEX_PATHNAME} className={styles['button-link']}>
            <ButtonSecondaryLarge text="Cancel" />
          </Link>
        </ActionBar>
      </div>
    );
  }
}

export default connect(
  (state) => ({
    selectedItems: selectors.selectedItemsSelector(LIST_NAME_ACTIONS, state),
    actions: state.alerts.actions,
    trigger: state.alerts.trigger
  }),
  {
    requestActions: alertsActions.request,
    requestTrigger: alertsTrigger.request,
    initializeSelectedList: actions.initialize,
    alertsTriggerAddActionsFromState
  }
)(AddActions);
