// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { connect } from 'react-redux';
import InlineSVG from 'svg-inline-react';
import { Link } from 'react-router';
import styles from './TriggerSavedModal.css';
import checkIcon from 'static/icons/sm/check.svg';
import {
  ButtonPrimaryLarge,
  ButtonSecondaryLarge
} from 'components/button';
import {
  TRIGGERS_INDEX_PATHNAME,
  TriggerMethods
} from 'constants/alerts';
import { setVisibleModal } from 'actions/layout';
import { alertsCreateFirstAction } from 'actions/alerts';

import type { AlertSummary } from 'types/alerts';


export class TriggerSavedModal extends Component {
  props: {
    actions: Array<AlertSummary>,
    triggerGuid: string,
    setVisibleModal: Function,
    createFirstAction: Function
  };

  handleCreateAction() {
    this.props.setVisibleModal(null);
    this.props.createFirstAction();
  }

  render() {
    const haveActions = this.props.actions.length > 0;

    return (
      <div className={styles.root}>
        <div className={styles.heading}>
          <InlineSVG src={checkIcon} className={styles.icon} />
          Trigger saved
        </div>
        <div className={styles.description}>
          Set actions to perform when your trigger goes off.
        </div>
        <div className={styles.hl} />
        <div className={styles.buttons}>
          <div className={styles['button-left']}>
            {haveActions
              ? <Link to={`${TRIGGERS_INDEX_PATHNAME}/${this.props.triggerGuid}/${TriggerMethods.ACTIONS}`}>
                  <ButtonPrimaryLarge
                    text="Choose actions"
                    onClick={() => this.props.setVisibleModal(null)}
                  />
                </Link>
              : <ButtonPrimaryLarge
                  text="Create an action"
                  onClick={this.handleCreateAction.bind(this)}
                />
            }
          </div>
          <div className={styles['button-right']}>
            <Link to={TRIGGERS_INDEX_PATHNAME}>
              <ButtonSecondaryLarge
                text={'I\'ll do this later'}
                onClick={() => this.props.setVisibleModal(null)}
              />
            </Link>
          </div>
        </div>
      </div>
    );
  }
}

export default connect(
  state => ({
    actions: state.alerts.actions, // note: this is requested on trigger page load, so unless the user is *really* quick, it'll exist here.
    triggerGuid: state.alerts.triggerLastCreated.guid
  }),
  {
    setVisibleModal,
    createFirstAction: alertsCreateFirstAction
  }
)(TriggerSavedModal);
