// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { Link } from 'react-router';
import { FlexColumn } from 'react-virtualized';
import { CellOverflowHidden } from 'components/table';
import { ContentPanel, TableContentPanel } from 'components/layout';
import EmptyListPrompt from 'components/EmptyListPrompt';
import ConfirmationModal from 'components/modal/ConfirmationModal';
import {
  alertsTriggers,
  alertsDeleteTrigger,
  alertsTriggerToDelete
} from 'actions/alerts';
import {
  TRIGGERS_INDEX_PATHNAME,
  NEW_TRIGGER_PATHNAME,
  TriggerMethods
} from 'constants/alerts';
import { setVisibleModal } from 'actions/layout';
import { triggerToDelete } from 'selectors/alerts';
import styles from './Triggers.css';


export class Triggers extends Component {
  props: {
    requestTriggers: Function,
    deleteTrigger: Function,
    setVisibleModal: Function,
    setTriggerToDelete: Function,
    triggers: Array<Object> | null,
    toDelete: Object
  };

  handleDeleteClicked(guid: string, e: SyntheticMouseEvent) {
    e.preventDefault();
    this.props.setTriggerToDelete(guid);
    this.props.setVisibleModal('triggerDelete');
  }

  handleDelete() {
    this.props.deleteTrigger(this.props.toDelete.guid);
    this.props.setVisibleModal(null);
  }

  componentDidMount() {
    this.props.requestTriggers();
  }

  render() {
    const triggers = this.props.triggers;
    const rowGetter = ({ index }) => triggers ? triggers[index] : null;
    return (
      <div>
        {triggers != null && triggers.length === 0
          ? <ContentPanel>
              <EmptyListPrompt
                buttonRoute={{ pathname: NEW_TRIGGER_PATHNAME }}
                buttonText="Add a trigger"
                header="You haven't added any triggers yet"
                text="Watch for specific events that occur in your cluster"
              />
            </ContentPanel>
          : triggers == null
              ? null
              : <TableContentPanel rowData={triggers}>
                  <FlexColumn label="Name" dataKey="name" rowGetter={rowGetter} width={50} flexGrow={1} cellRenderer={CellOverflowHidden} />
                  <FlexColumn label="Assigned actions" dataKey="actionCount" rowGetter={rowGetter} width={100} />
                  <FlexColumn dataKey="guid" width={80} cellRenderer={(item) =>
                    <div>
                      <Link to={`${TRIGGERS_INDEX_PATHNAME}/${item.cellData}/${TriggerMethods.EDIT}`}>Edit</Link>
                      <a
                        className={styles['delete-link']}
                        href="#"
                        onClick={this.handleDeleteClicked.bind(this, item.cellData)}
                      >
                        Delete
                      </a>
                    </div>
                  } />
                </TableContentPanel>
        }
        <ConfirmationModal
          id="triggerDelete"
          text={`Are you sure you want to delete trigger: ${this.props.toDelete.name}?`}
          onConfirm={this.handleDelete.bind(this)}
        />
      </div>
    );
  }
}

export default connect(
  (state) => ({
    triggers: state.alerts.triggers,
    toDelete: triggerToDelete(state)
  }),
  {
    requestTriggers: alertsTriggers.request,
    deleteTrigger: alertsDeleteTrigger,
    setTriggerToDelete: alertsTriggerToDelete,
    setVisibleModal
  }
)(Triggers);
