// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import InlineSVG from 'svg-inline-react';

import styles from './Button.css';

export const ButtonClassTypes = {
  PRIMARY: 'PRIMARY',
  SECONDARY: 'SECONDARY'
};

export const ButtonSizes = {
  LARGE: 'LARGE',
  MEDIUM: 'MEDIUM'
};

export type ButtonClassType = $Enum<typeof ButtonClassTypes>;
export type ButtonSize = $Enum<typeof ButtonSizes>;


type ButtonProps = {
  icon?: string,
  size: ButtonSize,
  text?: string,
  classType: ButtonClassType
};

export class Button extends Component {
  props: ButtonProps;

  static defaultProps = {
    size: ButtonSizes.MEDIUM,
    classType: ButtonClassTypes.PRIMARY
  };

  render() {
    const { icon, size, text, classType, ...buttonProps } = this.props;

    return (
      <button type="button" {...buttonProps} className={styles[`button-${classType.toLowerCase()}-${size.toLowerCase()}`]}>
        {icon && <InlineSVG className={styles.icon} src={icon} />}
        {text && <span className={styles.text}>{text}</span>}
      </button>
    );
  }
}


type TypedButtonProps = {
  icon?: string,
  text?: string
}

export class ButtonPrimaryMedium extends Component {
  props: TypedButtonProps;
  render() {
    return <Button {...this.props} size={ButtonSizes.MEDIUM} classType={ButtonClassTypes.PRIMARY} />;
  }
}

export class ButtonPrimaryLarge extends Component {
  props: TypedButtonProps;
  render() {
    return <Button {...this.props} size={ButtonSizes.LARGE} classType={ButtonClassTypes.PRIMARY} />;
  }
}

export class ButtonSecondaryMedium extends Component {
  props: TypedButtonProps;
  render() {
    return <Button {...this.props} size={ButtonSizes.MEDIUM} classType={ButtonClassTypes.SECONDARY} />;
  }
}

export class ButtonSecondaryLarge extends Component {
  props: TypedButtonProps;
  render() {
    return <Button {...this.props} size={ButtonSizes.LARGE} classType={ButtonClassTypes.SECONDARY} />;
  }
}

export default Button;
