// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';

import { ButtonPopup } from 'components/button-popup';
import type { Alignment } from 'components/button-popup';
import { PopupMenuItem } from './PopupMenuItem';
import { FluidTable } from 'components/table/FluidTable';
import { FlexColumn } from 'react-virtualized';
import { rootElementId } from 'constants';

import styles from './PopupMenu.less';

import {
  itemsHasFlaggedItem,
  estimateMaxMenuItemWidth
} from './linkSpecUtil';


export type PopupMenuProps = {
  alignment: Alignment,
  items: Array<Object>,
  rowHeightPx: number,
  popupMaxWidthPx: number,
  onMenuClosed?: () => void,
  onMenuOpened?: () => void
};

export class PopupMenu extends Component {

  props: PopupMenuProps;
  buttonPopup: any;

  static defaultProps = {
    rowHeightPx: 30,
    popupMaxWidthPx: 350
  };

  handleMenuItemClick() {
    if (this.buttonPopup) {
      this.buttonPopup.close();
    }
  }

  render() {
    const { alignment, items, popupMaxWidthPx, rowHeightPx } = this.props;

    if (!items || items.length === 0) {
      return null;
    }

    const averageCharWidthEstimatePx = 8; // TODO(mhowlett): guess this better.
    const popupWidthPx = estimateMaxMenuItemWidth(items, averageCharWidthEstimatePx, popupMaxWidthPx);

    const heightPx = rowHeightPx * Math.min(items.length, 3) + 4;

    return (
      <ButtonPopup
        ref={el => this.buttonPopup = el}
        rootElementId={rootElementId}
        flag={itemsHasFlaggedItem(items)}
        alignment={alignment}
        label={`${items.length}`}
        popupHeight={heightPx}
        onMenuOpened={this.props.onMenuOpened}
        onMenuClosed={this.props.onMenuClosed}
      >
        <div style={{ width: `${popupWidthPx}px`, height: `${heightPx}px` }}>
          <FluidTable
            rowHeight={rowHeightPx}
            headerHeight={0}
            rowData={items}
            extraClass={styles['table-menu']}
          >
            <FlexColumn
              dataKey="none"
              cellDataGetter={({ rowData }) => rowData}
              cellRenderer={({ cellData }) => (
                <PopupMenuItem
                  linkSpec={cellData}
                  onClick={() => this.handleMenuItemClick()}
                />
              )}
              width={popupWidthPx}
            />
          </FluidTable>
        </div>
      </ButtonPopup>
    );
  }

}
