// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { destroy } from 'redux-form';

import { ContentPanel } from 'components/layout';
import ConnectedForm from './ConnectedForm';
import ActionFormBar from './ActionFormBar';
import { ConnectedModal } from 'components/modal';
import { setVisibleModal } from 'actions/layout';

import type { FormType, FieldValue } from 'types/form';
import type { Dispatcher } from 'types';


export class ActionForm extends Component {
  props: {
    canSave: boolean,
    route: {
      routedFormData: {
        formType: FormType,
        fieldSectionsSelector: Function,
        initialChangeValues: Array<FieldValue>,
        actionBarButtonsSelector: Function,
        formMountedHandler: Function,
        formSubmitHandler: Function,
        modalComponent?: React$Element<any>
      }
    },
    dispatch: Dispatcher,
    routeParams?: Object
  };

  componentDidMount() {
    this.props.route.routedFormData.formMountedHandler(this.props);
  }

  componentWillUnmount() {
    this.props.dispatch(destroy(this.props.route.routedFormData.formType));
    this.props.dispatch(setVisibleModal(null));
  }

  render() {
    const { canSave } = this.props;
    const { formType, fieldSectionsSelector, initialChangeValues, actionBarButtonsSelector, modalComponent, formSubmitHandler } = this.props.route.routedFormData;
    return (
      <form onSubmit={(e) => {
        e.preventDefault();
        if (canSave) {
          formSubmitHandler(this.props.dispatch, this.props.routeParams);
        }
      }}>
        <ContentPanel>
          <ConnectedForm formType={formType} fieldSectionsSelector={fieldSectionsSelector} initialChangeValues={initialChangeValues} />
        </ContentPanel>
        <ActionFormBar actionBarButtonsSelector={actionBarButtonsSelector} routeParams={this.props.routeParams} />
        {modalComponent
          ? <ConnectedModal id="ActionFormModal" clickOutsideCloses={false}>
              {modalComponent}
            </ConnectedModal>
          : null}
      </form>
    );
  }
}

export default connect(
  (state, props) => (
    {
      canSave: props.route.routedFormData.canSaveSelector(state)
    }
  )
)(ActionForm);
