// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';

import styles from './FieldHelp.css';

export const FieldHelp = (props: {
  defaultValue: ?string,
  documentation: ?string,
  error: ?string
}) => {
  const documentation = props.documentation || '';
  const defaultValue = props.defaultValue || '';
  const error = props.error || '';

  const mostCharacters = Math.max(documentation.length, defaultValue.length, error.length);

  // extend to 300px, if necessary
  const maxWidth = `${Math.max(40, mostCharacters / 20)}rem`;

  return (
    <div className={styles.root} style={{ maxWidth }}>
      {Boolean(documentation) && (
        <div className={styles.documentation}>
          {documentation}
        </div>
      )}

      {Boolean(error) && (
        <div className={styles.error}>
          {error}
        </div>
      )}

      {Boolean(defaultValue) && (
        <div className={styles['default-value']}>
          Default value: {defaultValue}
        </div>
      )}
    </div>
  );
};

export default FieldHelp;
