// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import FlipMove from 'react-flip-move';
import { Field } from 'redux-form';

import { get } from 'lodash';

import styles from './Form.css';
import type { FieldSections } from 'types/form';

import FieldInput from 'components/FieldInput';

export class Form extends Component {
  props: {
    errorsAreVisible: boolean,
    fieldSections: FieldSections,
    optionsByField?: Object,
    showTitles: boolean
  };

  static defaultProps = {
    fieldSections: [],
    showTitles: true
  };

  render() {
    const { props } = this;
    return (
      <div className={styles['form-root']}>
        <FlipMove>
          {props.fieldSections.map(section => (
            <div
              key={section.title}
              role="group"
              className={styles['fieldset']}
            >
              {section.title && props.showTitles && (
                <div className={styles.legend}>{section.title}</div>
              )}
              {section.fields.map(field => (
                <Field
                  key={field.id}
                  props={{
                    errorsAreVisible: props.errorsAreVisible,
                    sectionField: {
                      ...field,
                      options: get(props.optionsByField, field.id, get(field, 'options', []))
                    }
                  }}
                  component={FieldInput}
                  name={field.id}
                />
              ))}
            </div>
          ))}
        </FlipMove>
      </div>
    );
  }
}

export default Form;
