// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';

import { Icon } from 'components/icon';

import styles from './Label.css';


export class Label extends Component {
  props: {
    children?: any,
    disabled: boolean,
    id: string,
    showLabel: boolean,
    staticPlaceholder: boolean,
    text?: ?string,
    showInfoIcon: boolean
  };

  static defaultProps = {
    disabled: false,
    showInfoIcon: false,
    showLabel: false,
    staticPlaceholder: false
  };

  label: HTMLLabelElement;

  handleClick: (e: SyntheticMouseEvent) => void;

  handleClick(e: SyntheticMouseEvent) {
    // the label `for` attribute is presumably linked
    // to the actual form element's `id`, which means this
    // is the same as clicking the form element directly
    if (e.target !== this.label) {
      this.label.click();
    }
  }

  constructor() {
    super();
    this.handleClick = this.handleClick.bind(this);
  }

  render() {
    const { props } = this;
    return (
      <div
        onClick={this.handleClick}
        className={styles[`label-root${props.disabled ? '-disabled' : ''}`]}
      >
        {props.children}
        <label
          ref={(el: HTMLLabelElement) => this.label = el}
          htmlFor={props.id}
          className={styles[`${
            props.showLabel && !props.staticPlaceholder ? 'label' : (
              `placeholder${props.staticPlaceholder ? '-static' : ''}`
            )
          }-mode`]}
        >
          {props.text}
          {props.showInfoIcon && (
            <span>
              {' '}
              <Icon name="info" size={16} />
            </span>
          )}
        </label>
      </div>
    );
  }
}

export default Label;
