// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';

import Label from './Label';
import styles from './LabeledTextInput.css';
import { constructPlaceholderText } from 'components/FieldInput/FieldInput';

import type { FormField, FieldSectionField } from 'types/form';

export class LabeledTextInput extends Component {
  props: {
    active: boolean,
    reduxFormInputProps: FormField,
    sectionField: FieldSectionField,
    showErrorIndicator: boolean,
    showInfoIcon: boolean,
    type: string
  };

  static defaultProps = {
    showErrorIndicator: false,
    showInfoIcon: false,
    type: 'text'
  };

  render() {
    const {
      active,
      reduxFormInputProps,
      sectionField,
      showErrorIndicator,
      showInfoIcon,
      type
    } = this.props;

    const disabled = sectionField.locked || reduxFormInputProps.disabled;

    const placeholder = constructPlaceholderText(sectionField);

    return (
      <Label
        id={sectionField.id}
        disabled={disabled}
        showLabel={active || Boolean(reduxFormInputProps.value)}
        text={placeholder}
        showInfoIcon={showInfoIcon}
      >
        <input
          {...reduxFormInputProps}
          disabled={disabled}
          autoComplete="off"
          className={styles[showErrorIndicator ? 'input-error' : 'input']}
          id={sectionField.id}
          placeholder={placeholder}
          type={type}
        />
      </Label>
    );
  }
}

export default LabeledTextInput;
