// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { connect } from 'react-redux';
import React, { Component } from 'react';

import { consumerGroupsSelector } from 'selectors/monitoring/data';
import { Link } from 'react-router';

import styles from './Breadcrumbs.css';

export class Breadcrumbs extends Component {
  props: {
    elements: Array<Array<Object>>
  };

  render() {
    return (
      <ul className={styles.crumbs}>
        {this.props.elements.map(
          (arr, i) =>
            <li key={i}>
              {arr.map(
                (el, j) =>
                  <Link key={j} to={el.path}>{el.name + (j !== arr.length-1 ? ', ' : '')}</Link>
              )}
            </li>
        )}
      </ul>
    );
  }
}


export function constructDisplayElements(params: Object, routes: Array<Object>, consumerGroups: ?Array<string>) {

  const routeElements = routes
    .filter(route => route.name || route.names);

  const namedElements = routeElements
    .map((element, i) => element.names
      ? (i === routeElements.length-1
          ? { name: element.names.terminal, path: element.path }
          : { name: element.names.middle, path: element.path }
        )
      : element
    );

  if (params.id) {
    namedElements.push({ name: params.id, path: params.id });
  }

  const constructPaths = elements =>
    elements.reduce(
      (accum, current) => {
        const el = {
          name: current.name,
          path: `${(accum.length === 0 ? '' : accum[accum.length-1].path)}/${current.path}`
        };
        accum.push(el);
        return accum;
      },
      []
    ).map(el => [el]);

  const generateConsumerElements = (elements, consumerGroups): Array<Array<Object>> => {
    if (elements.length !== 3) {
      return [];
    }
    return [
      [{ name: 'Stream Monitoring', path: '/monitoring' }],
      [{ name: 'Message Delivery', path: '/monitoring/consumer-groups' }],
      consumerGroups.map(cg => ({ name: cg, path: `/monitoring/consumer-groups/${cg}` })),
      [{ name: elements[2].name, path: `/monitoring/consumers/${elements[2].path}` }]
    ];
  };

  const elements = consumerGroups && (namedElements.length > 1)
    ? generateConsumerElements(namedElements, consumerGroups)
    : constructPaths(namedElements);

  return elements;
}


export default connect(
  (state, props) => ({
    elements: constructDisplayElements(
      props.params,
      props.routes,
      consumerGroupsSelector(state)
    )
  })
)(Breadcrumbs);
