// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { ContentPanel } from 'components/layout';

import styles from './SizeEmittingContentPanel.css';


export class SizeEmittingContentPanel extends Component {

  props: {
    children: Function
  }

  state: {
    availableHeight: number
  };

  static defaultProps = {
    children: () => null
  }

  _measure: Function;
  contentElement: HTMLElement;

  constructor() {
    super();
    this._measure = this._measure.bind(this);
    this.state = { availableHeight: 0 };
  }

  _measure() {
    const paddingPx = 70;
    this.setState({ availableHeight: window.innerHeight - this.contentElement.getBoundingClientRect().top - paddingPx });
  }

  componentDidMount() {
    setTimeout(this._measure, 0);
    window.addEventListener('resize', this._measure);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this._measure);
  }

  render() {
    return (
      <div className={styles['root']} ref={el => this.contentElement = el}>
        <ContentPanel>
          {this.state.availableHeight > 0 && this.props.children(this.state.availableHeight)}
        </ContentPanel>
      </div>
    );
  }
}

export default SizeEmittingContentPanel;
