// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import { bindActionCreators } from 'redux';
import { connect } from 'react-redux';

import { connectorStatusByNameSelector } from 'selectors/management';
import * as ConnectorActions from 'actions/connectors';
import type { ConnectorOrTaskStatus } from 'actions/connectors';


type GenericConnectorAction = (connctorName: string) => any;

const options = [
  {
    icon: 'pauseSm',
    label: 'Pause',
    value: 'PAUSED'
  },
  {
    icon: 'playSm',
    label: 'Resume',
    value: 'RUNNING'
  }
];

const isLoadingStatus = (status: string): boolean => (
  !status || status === 'PAUSING' || status === 'RESUMING' || status === 'DELETING' || status === 'DELETED'
);

export class ConnectorStatusActions extends React.Component {
  props: {
    actions: {
      loadConnectorStatus: GenericConnectorAction,
      pauseConnector: GenericConnectorAction,
      resumeConnector: GenericConnectorAction
    },
    children: (status: ConnectorOrTaskStatus, isTransitioning: boolean, pause: GenericConnectorAction, resume: GenericConnectorAction, options: Array<Object>) => any,
    name: string,
    status: ConnectorOrTaskStatus
  };

  static defaultProps = {
    isLoading: true
  };

  pause: () => void;
  pollingIntervalId: number;
  resume: () => void;
  statusChangeReceived: (status: ConnectorOrTaskStatus) => void;

  constructor() {
    super();
    this.pause = this.pause.bind(this);
    this.resume = this.resume.bind(this);
  }


  pause() {
    this.props.actions.pauseConnector(this.props.name);
  }

  resume() {
    this.props.actions.resumeConnector(this.props.name);
  }

  statusChangeReceived(status: ConnectorOrTaskStatus) {
    const isLoading = isLoadingStatus(status);

    if (this.pollingIntervalId) {
      clearInterval(this.pollingIntervalId);
    }

    if (isLoading) {
      this.pollingIntervalId = setInterval(() => {
        this.props.actions.loadConnectorStatus(this.props.name);
      }, 1000);
    }

  }

  componentWillMount() {
    this.props.actions.loadConnectorStatus(this.props.name);
  }

  componentWillReceiveProps(nextProps: Object) {
    if (typeof nextProps.status !== 'undefined') {
      this.statusChangeReceived(nextProps.status);
    }
  }

  componentWillUnmount() {
    clearInterval(this.pollingIntervalId);
  }

  render() {
    const {
      children,
      status
    } = this.props;

    const isTransitioning = isLoadingStatus(status);

    return children(status, isTransitioning, this.pause, this.resume, options);
  }
}

const mapStateToProps = (state, ownProps) => {
  return {
    status: connectorStatusByNameSelector(state)[ownProps.name]
  };
};

// TODO(@dylanharrington): write this in the superior object style
const mapDispatchToProps = dispatch => ({
  actions: bindActionCreators(ConnectorActions, dispatch)
});

export default connect(
  mapStateToProps,
  mapDispatchToProps
)(ConnectorStatusActions);
