// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { browserHistory } from 'react-router';

import { ActionBar } from 'components/navigation';
import { ButtonPrimaryLarge, ButtonSecondaryLarge } from 'components/button';

import type {
  FormStateSteps,
  FormStepKey,
  FormSteps
} from 'types/form';

export type Props = {
  completionRedirectPathname: string,
  currentStep: FormStepKey,
  errorCount: number,
  firstStep: FormStepKey,
  invalid: boolean,
  lastStep: FormStepKey,
  pathname: string,
  statefulSteps: FormStateSteps,
  steps: FormSteps,
  submitting: boolean,
  showErrors: () => any,
  valid: boolean
};

export class ManagementFormActionBar extends Component {
  props: Props;

  render() {
    const {
      completionRedirectPathname,
      currentStep,
      errorCount,
      firstStep,
      invalid,
      lastStep,
      pathname,
      showErrors,
      statefulSteps,
      steps,
      submitting
    } = this.props;

    const disabledByErrors = steps[currentStep].zeroErrorsBeforeSubmit && errorCount > 0 || invalid;

    const isFirstStep = firstStep === currentStep;
    const isLastStep = lastStep === currentStep;
    const previousStep = steps[currentStep].previousStep;

    return (
      <ActionBar
        navLinks={Object.keys(steps).map(stepKey => ({
          props: {
            disabled: !statefulSteps[stepKey].canAccess
          },
          label: steps[stepKey].navLabel,
          location: { pathname, query: { step: stepKey } }
        }))}
      >
        <ButtonSecondaryLarge
          disabled={submitting}
          onClick={() => {
            if (isFirstStep) {
              browserHistory.push(completionRedirectPathname);
            } else {
              browserHistory.push({
                pathname,
                query: {
                  step: previousStep
                }
              });
            }
          }}
          text={isFirstStep ? 'Cancel' : 'Back'}
        />
        <ButtonPrimaryLarge
          onClick={() => {
            if (disabledByErrors) {
              showErrors();
            }
          }}
          disabled={submitting}
          type={disabledByErrors ? 'button' : 'submit'}
          text={isLastStep ? 'Save & Finish' : 'Continue'}
        />
      </ActionBar>
    );
  }
}

export default ManagementFormActionBar;
