// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import React, { Component } from 'react';
import { browserHistory } from 'react-router';
import { connect } from 'react-redux';

import { ManagementForm } from 'components/management';

import type { Props as ManagementFormProps } from './ManagementForm';
import type { Props as FormActionBarProps } from './ManagementFormActionBar';

import {
  currentFieldSectionsSelector,
  currentFieldsSelector,
  currentStepIsAccessibleSelector,
  currentStepKeySelector,
  currentStepsSelector
} from 'selectors/management';

import { FormTypes } from 'types/form';
import { STEP1 } from 'constants/management';

export class ManagementFormContainer extends Component {
  props: {
    canAccessStep: boolean,
    formActionBarProps: FormActionBarProps,
    formProps: ManagementFormProps,
    pathname: string,
    route: Object
  };

  componentWillMount() {
    const { canAccessStep, pathname } = this.props;
    if (!canAccessStep) {
      browserHistory.replace({
        pathname,
        query: {
          step: STEP1
        }
      });
    }
  }

  render() {
    const {
      canAccessStep,
      formActionBarProps,
      formProps,
      route
    } = this.props;

    if (!canAccessStep) {
      return null;
    }

    return (
      <ManagementForm
        {...formProps}
        formActionBarProps={formActionBarProps}
        route={route}
      />
    );
  }
}

const mapStateToProps = state => ({
  canAccessStep: currentStepIsAccessibleSelector(state),
  completed: state.management[FormTypes.CONNECTOR_FORM].completed,
  currentStepKey: currentStepKeySelector(state),
  errorCount: state.management[FormTypes.CONNECTOR_FORM].errorCount,
  statefulFieldSections: currentFieldSectionsSelector(state),
  statefulFields: currentFieldsSelector(state),
  statefulSteps: currentStepsSelector(state),
  initialValues: state.management[FormTypes.CONNECTOR_FORM].initialValues
});

const mergeProps = (stateProps, dispatchProps, ownProps) => {
  const {
    canAccessStep,
    completed,
    errorCount,
    currentStepKey,
    initialValues,
    statefulFieldSections,
    statefulSteps
  } = stateProps;

  const {
    location,
    route
  } = ownProps;

  const { pathname } = location;

  const { routedFormData } = route;
  const {
    firstStep,
    lastStep,
    redirectPathname,
    steps
  } = routedFormData;

  if (!canAccessStep) {
    return {
      canAccessStep,
      pathname
    };
  }

  const isLastStep = lastStep === currentStepKey;

  const statefulFieldSectionsBlacklist = [
    ...(steps[currentStepKey].fieldBlacklist || []),
    ...steps[currentStepKey].fields
  ];

  const result = {
    canAccessStep,
    route,
    formActionBarProps: {
      completionRedirectPathname: redirectPathname,
      errorCount,
      pathname,
      currentStep: currentStepKey,
      firstStep: firstStep,
      lastStep: lastStep,
      statefulSteps,
      steps
    },

    formProps: {
      ...steps[currentStepKey],
      completionRedirectPathname: redirectPathname,
      completed,
      initialValues,
      showSummary: isLastStep,
      statefulFieldSections,
      statefulFieldSectionsBlacklist
    }
  };
  return result;
};

export default connect(mapStateToProps, {}, mergeProps)(ManagementFormContainer);
