// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { connect } from 'react-redux';
import ConnectedModal from './ConnectedModal';
import {
  ButtonPrimaryLarge,
  ButtonSecondaryLarge
} from 'components/button';
import { setVisibleModal } from 'actions/layout';

import styles from './ConfirmationModal.css';


export class ConfirmationModal extends Component {
  props: {
    id: string,
    text: string,
    okText: string,
    cancelText: string,
    onConfirm: Function,
    onCancel: Function,
    setVisibleModal: Function
  };

  static defaultProps = {
    text: 'Are you sure?',
    okText: 'OK',
    cancelText: 'Cancel'
  }

  handleOkAction() {
    if (this.props.onConfirm) {
      this.props.onConfirm();
    }
  }

  handleCancelAction() {
    if (this.props.onCancel) {
      this.props.onCancel();
      return;
    }
    this.props.setVisibleModal(null);
  }

  render() {
    return (
      <ConnectedModal id={this.props.id}>
        <div className={styles.root}>
          <div className={styles.text}>
            {this.props.text}
          </div>
          <div className={styles.buttons}>
            <div className={styles['button-left']}>
              <ButtonPrimaryLarge
                text={this.props.okText}
                onClick={this.handleOkAction.bind(this)}
              />
            </div>
            <div className={styles['button-right']}>
              <ButtonSecondaryLarge
                text={this.props.cancelText}
                onClick={this.handleCancelAction.bind(this)}
              />
            </div>
          </div>
        </div>
      </ConnectedModal>
    );
  }
}

export default connect(
  null,
  { setVisibleModal }
)(ConfirmationModal);
