// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { SubNavigation } from 'components/navigation';
import { connect } from 'react-redux';
import Sticky from 'react-stickynode';
import { ContentPanel } from 'components/layout';
import { DataLayoutTypes } from 'types/monitoring/data';

import { aComponentIsFocusedSelector, focusedComponentSelector } from 'selectors/layout';
import { ComponentTypes } from 'actions/layout';
import type { FocusedSpec } from 'reducers/layout';

import styles from './MonitoringBody.css';


export type MonitoringBodyProps = {
  children?: any,
  links?: any,
  menuDisabled: boolean,
  focusedComponent: ?FocusedSpec
};

export class MonitoringBody extends Component {
  props: MonitoringBodyProps;

  render() {
    const { links, children, focusedComponent } = this.props;
    return (
      <div className={styles['monitoring-body-root']}>
        {links && (
          <Sticky top={56}>
            <SubNavigation
              links={links}
              padded={true}
              disabled={this.props.menuDisabled}
            />
          </Sticky>
        )}
        <ContentPanel deemphasize={focusedComponent != null && focusedComponent.type === ComponentTypes.MONITORING_CHART}>
          {children}
        </ContentPanel>
      </div>
    );
  }
}

function select(state, props) {
  const result = {
    links: [],
    menuDisabled: aComponentIsFocusedSelector(state),
    focusedComponent: focusedComponentSelector(state)
  };

  switch (props.route.routedDataLayoutType) {
    case DataLayoutTypes.CONSUMER_GROUPS:
    case DataLayoutTypes.CONSUMER_GROUPS_TOPICS:
      result.links = [
        {
          label: 'consumer groups',
          location: '/monitoring/consumer-groups'
        },
        {
          label: 'topics',
          location: '/monitoring/consumer-groups-topics'
        }
      ];
      break;
    case DataLayoutTypes.CONSUMER_GROUP:
    case DataLayoutTypes.CONSUMER_GROUP_TOPICS:
      result.links = [
        {
          label: 'consumers',
          location: `/monitoring/consumer-groups/${props.routeParams.id}`
        },
        {
          label: 'topic partitions',
          location: `/monitoring/consumer-groups-topics/${props.routeParams.id}`
        }
      ];
      break;
  }

  return result;
}

export default connect(select)(MonitoringBody);
