// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import moment from 'moment';
import styles from './DefaultRanges.css';
import { RangeTypes } from 'constants/monitoring/timeframe';
import type { RangeType } from 'constants/monitoring/timeframe';

export type DisplayRanges = Array<Array<{
  startTime: number,
  endTime: number,
  rangeType: RangeType
}>>

export const defaultGetRanges = (): DisplayRanges => {
  return [
    [
      {
        // Last 10 minutes
        startTime: moment().subtract(10, 'minutes').unix(),
        endTime: moment().unix(),
        rangeType: RangeTypes.ROLLING
      }, {
        // Last 30 minutes
        startTime: moment().subtract(30, 'minutes').unix(),
        endTime: moment().unix(),
        rangeType: RangeTypes.ROLLING
      }, {
        // Last 4 hours
        startTime: moment().subtract(4, 'hours').unix(),
        endTime: moment().unix(),
        rangeType: RangeTypes.ROLLING
      }, {
        // Last 12 hours
        startTime: moment().subtract(12, 'hours').unix(),
        endTime: moment().unix(),
        rangeType: RangeTypes.ROLLING
      }, {
        // Last 24 hours
        startTime: moment().subtract(24, 'hours').unix(),
        endTime: moment().unix(),
        rangeType: RangeTypes.ROLLING
      }
    ], [
      {
        // Yesterday
        startTime: moment().startOf('day').subtract(24, 'hours').unix(),
        endTime: moment().endOf('day').subtract(24, 'hours').unix(),
        rangeType: RangeTypes.FIXED
      }, {
        // Today
        startTime: moment().startOf('day').unix(),
        endTime: moment().endOf('day').unix(),
        rangeType: RangeTypes.FIXED
      }, {
        // This week
        startTime: moment().startOf('week').unix(),
        endTime: moment().endOf('week').unix(),
        rangeType: RangeTypes.FIXED
      }, {
        // This month
        startTime: moment().startOf('month').unix(),
        endTime: moment().endOf('month').unix(),
        rangeType: RangeTypes.FIXED
      }
    ], [
      {
        // Today so far
        startTime: moment().startOf('day').unix(),
        endTime: moment().unix(),
        rangeType: RangeTypes.GROWING_DAY
      }, {
        // Week so far
        startTime: moment().startOf('week').unix(),
        endTime: moment().unix(),
        rangeType: RangeTypes.GROWING_WEEK
      }, {
        // Month so far
        startTime: moment().startOf('month').unix(),
        endTime: moment().unix(),
        rangeType: RangeTypes.GROWING_MONTH
      }
    ], [
      {
        // Today last week
        startTime: moment().startOf('day').subtract(1, 'week').unix(),
        endTime: moment().endOf('day').subtract(1, 'week').unix(),
        rangeType: RangeTypes.FIXED
      }, {
        // Today last month
        startTime: moment().startOf('day').subtract(1, 'month').unix(),
        endTime: moment().endOf('day').subtract(1, 'month').unix(),
        rangeType: RangeTypes.FIXED
      }
    ]
  ];
};

export type DefaultRangesProps = {
  getRanges: () => DisplayRanges,
  selectedRangeLabel: string,
  onHandleDefaultRangeClick: (startTime: number, endTime: number, rangeType: RangeType) => any,
  onHandleDefaultRangeHover: (startTime: number, endTime: number, rangeType: RangeType, hovered: boolean) => any,
  onHandleDefaultRangeBlur: () => any,
  getRangeLabel: (startTime: number, endTime: number, rangeType: RangeType) => any
};

class DefaultRanges extends Component {
  props: DefaultRangesProps;

  static defaultProps = {
    getRanges: defaultGetRanges
  };

  render() {
    const {
      getRanges,
      selectedRangeLabel,
      onHandleDefaultRangeClick,
      onHandleDefaultRangeHover,
      onHandleDefaultRangeBlur,
      getRangeLabel
    } = this.props;
    return (
      <div className={styles['default-ranges-root']}>
        {getRanges().map((ranges, i) => (
          <ul className={styles['range-col']} key={i}>
          {ranges.map((range, j) => (
            <li className={styles['range-item']} key={j}>
              <div
                className={styles[selectedRangeLabel === getRangeLabel(range.startTime, range.endTime, range.rangeType)
                  ? 'selected-range'
                  : 'range'
                ]}
                onClick={() => onHandleDefaultRangeClick(range.startTime, range.endTime, range.rangeType)}
                onMouseEnter={() => onHandleDefaultRangeHover(range.startTime, range.endTime, range.rangeType, true)}
                onMouseLeave={() => onHandleDefaultRangeBlur()}
              >
                {getRangeLabel(range.startTime, range.endTime, range.rangeType)}
              </div>
            </li>
          ))}
          </ul>
        ))}
      </div>
    );
  }
}

export default DefaultRanges;
