// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';

export type MouseCatcherProps = {
  rootElementId: string,
  children?: any,
  onClose?: Function
};


export class MouseCatcher extends Component {

  props: MouseCatcherProps;

  rootDiv: HTMLElement;
  _handleMouseDown: EventHandler;

  constructor() {
    super();
    this._handleMouseDown = this.handleMouseDown.bind(this);
  }

  handleMouseDown(e: Event): void {
    // $FlowFixMe
    if (this.props.onClose && !this.mouseIsOverInner((e: MouseEvent))) {
      e.stopPropagation();
      if (this.props.onClose) {
        this.props.onClose();
      }
    }
  }

  mouseIsOverInner(e: MouseEvent): boolean {
    if (!this.rootDiv.firstChild) {
      return false;
    }

    // $FlowFixMe
    const boundingRect = this.rootDiv.firstChild.getBoundingClientRect();

    return (
      e.clientX > boundingRect.left &&
      e.clientX < boundingRect.right &&
      e.clientY > boundingRect.top &&
      e.clientY < boundingRect.bottom
    );

    // TODO(mhowlett) at this point, this is nearly what we want. However, if
    // the popup is covered (possibly partially) by another element, we really
    // want clicking within that area to close the popup. From here, this
    // could be achieved using document.elementFromPoint + traversing the
    // popup node tree. However I think the current functionality is good
    // enough for our purposes and there are higher priorities.
  }

  componentDidMount() {
    document.getElementById(this.props.rootElementId)
      .addEventListener('mousedown', this._handleMouseDown, true);
  }

  componentWillUnmount() {
    document.getElementById(this.props.rootElementId)
      .removeEventListener('mousedown', this._handleMouseDown, true);
  }

  render() {
    return (
      <div ref={el => this.rootDiv = el}>
        {this.props.children}
      </div>
    );
  }

}

export default MouseCatcher;
