// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import styles from './SelectionCellRenderer.css';

export type SelectionCellRendererProps = {
  columnData: {
    deselectItem: Function,
    endRangeSelect: Function,
    isRangeSelecting: boolean,
    selectItem: Function,
    selectedIdMap: {},
    startRangeSelect: Function,
    tableRowData: Array<any>
  },
  cellData: any,
  dataKey: string,
  rowIndex: number
};

const SelectionCellRenderer = ({
  columnData: {
    deselectItem,
    endRangeSelect,
    isRangeSelecting,
    selectItem,
    selectedIdMap,
    startRangeSelect,
    tableRowData
  },
  cellData,
  dataKey,
  rowIndex
}: SelectionCellRendererProps) => {

  const isSelected = Boolean(selectedIdMap[cellData]);

  return (
    <input
      className={styles.checkbox}
      readOnly={true}
      onClick={event => {
        event.stopPropagation();
        if (event.shiftKey && isRangeSelecting) {
          endRangeSelect(rowIndex, tableRowData, dataKey);
        } else {
          if (isSelected) {
            deselectItem(cellData);
          } else {
            selectItem(cellData);
          }
          startRangeSelect(rowIndex);
        }
      }}
      type="checkbox"
      checked={isSelected}
    />
  );
};

export default SelectionCellRenderer;
