// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import classNames from 'classnames';

import styles from './SelectionHeaderRenderer.css';
import { SelectionStatuses } from './selectionListActions';

import type { SelectionStatus } from './selectionListActions';

const { ALL, NONE, SOME } = SelectionStatuses;

export type SelectionHeaderRendererProps = {
  columnData: {
    selectionStatus: SelectionStatus,
    deselectAll: Function,
    selectAll: Function,
    tableRowData: Array<any>
  },
  dataKey: string
};

const SelectionHeaderRenderer = ({
  columnData: {
    deselectAll,
    selectionStatus,
    selectAll,
    tableRowData
  },
  dataKey
}: SelectionHeaderRendererProps) => (
  <input
    readOnly={true}
    className={classNames(styles.checkbox, {
      [styles.hasSelection]: selectionStatus === SOME
    })}
    onClick={event => {
      event.stopPropagation();
      if (selectionStatus === NONE) {
        selectAll(tableRowData, dataKey);
      } else {
        deselectAll();
      }
    }}
    type="checkbox"
    checked={selectionStatus === ALL}
  />
);

export default SelectionHeaderRenderer;
