// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { bindActionCreators } from 'redux';
import { connect } from 'react-redux';
import { get } from 'lodash';

import { selectedItemsSelector } from './selectionListSelectors';
import * as selectionListActions from './selectionListActions';
import SelectionCellRenderer from './SelectionCellRenderer';
import SelectionHeaderRenderer from './SelectionHeaderRenderer';

import type { SelectionStatus } from './selectionListActions';


export const REDUX_MOUNT_POINT = 'selectionList';

const { ALL, NONE, SOME } = selectionListActions.SelectionStatuses;

function withSelectionList<WithSelectionListOptions>(
  ComposedComponent: ReactClass<WithSelectionListOptions>,
  dataKey: string,
  listName: string
): ReactClass<{}> {

  class SelectionListWrapped extends Component {
    componentWillMount() {
      if (!this.props.selectedItems) {
        this.props.initialize(listName);
      }
    }

    render() {
      const mapStateToProps = (state, ownProps) => {
        const selectionList = get(state, REDUX_MOUNT_POINT, {});
        const {
          startRangeIndex,
          selectedIdMap
        } = get(selectionList, listName, {});

        const selectionCount = Object.keys(selectedIdMap).length;

        const selectionStatus: SelectionStatus = selectionCount === ownProps.rowData.length
          ? ALL
          : selectionCount > 0
            ? SOME
            : NONE;

        return {
          selectionStatus,
          startRangeIndex,
          selectedIdMap
        };
      };

      const mapDispatchToProps = dispatch => {
        const actions = bindActionCreators(selectionListActions, dispatch);

        return {
          deselectAll: actions.deselectAll.bind(null, listName),
          deselectItem: actions.deselectItem.bind(null, listName),
          endRangeSelect: actions.endRangeSelect.bind(null, listName),
          selectAll: actions.selectAll.bind(null, listName),
          selectItem: actions.selectItem.bind(null, listName),
          startRangeSelect: actions.startRangeSelect.bind(null, listName)
        };
      };

      const mergeProps = (stateProps, dispatchProps, ownProps) => ({
        ...ownProps,
        ...stateProps,
        flexColumnProps: {
          cellRenderer: SelectionCellRenderer,
          columnData: {
            ...dispatchProps,
            selectionStatus: stateProps.selectionStatus,
            isRangeSelecting: stateProps.startRangeIndex !== -1,
            selectedIdMap: stateProps.selectedIdMap,
            startRangeIndex: stateProps.startRangeIndex,
            tableRowData: ownProps.rowData
          },
          dataKey,
          headerRenderer: SelectionHeaderRenderer,
          label: ''
        }
      });

      const connected = connect(mapStateToProps, mapDispatchToProps, mergeProps)(ComposedComponent);

      return React.createElement(connected, this.props);
    }
  }

  return connect(
    (state) => ({
      selectedItems: selectedItemsSelector(listName, state)
    }),
    { initialize: selectionListActions.initialize }
  )(SelectionListWrapped);
}

export default withSelectionList;
