// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import * as ConnectorActions from 'actions/connectors';

import type { Action } from 'types';

import type { ConnectorListState } from 'actions/connectors';

type StatusByName = {};

type ConnectorsState = {
  list: ConnectorListState,
  statusByName: StatusByName
};

const initialState: ConnectorsState = {
  list: {
    isFresh: false,
    items: []
  },
  statusByName: {}
};

const list = (
  state: ConnectorListState = initialState.list,
  action: Action<*, *>
): ConnectorListState => {
  switch (action.type) {
    case ConnectorActions.CONNECTOR_SAVE_SUCCESS:
      return {
        isFresh: false,
        items: state.items
      };
    case ConnectorActions.CONNECTOR_LIST_FETCH_SUCCESS:
      return {
        isFresh: true,
        items: [...action.payload.connectors]
      };
    default:
      return state;
  }
};

const statusByName = (
  state: StatusByName = initialState.statusByName,
  action: Action<*, *>
): StatusByName => {
  switch (action.type) {
    case ConnectorActions.CONNECTOR_DESTROY_REQUEST:
      return {
        ...state,
        [action.payload.name]: 'DELETING'
      };
    case ConnectorActions.CONNECTOR_DESTROY_SUCCESS:
      return {
        ...state,
        [action.payload.name]: 'DELETED'
      };
    case ConnectorActions.CONNECTOR_PAUSE_REQUEST:
      return {
        ...state,
        [action.payload.name]: 'PAUSING'
      };
    case ConnectorActions.CONNECTOR_RESUME_REQUEST:
      return {
        ...state,
        [action.payload.name]: 'RESUMING'
      };
    case ConnectorActions.CONNECTOR_STATUS_SUCCESS:
      return {
        ...state,
        [action.payload.name]: action.payload.connectorStatus.connector.state
      };
    default:
      return state;
  }
};

const reducer = (state: ConnectorsState = initialState, action: Action<*, *>): ConnectorsState => ({
  list: list(state.list, action),
  statusByName: statusByName(state.statusByName, action)
});

export default reducer;
