// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import type { Action } from 'types';

import {
  SET_TIME_PICKER_VISIBILITY,
  SET_CHART_AREA_WIDTH,
  SET_MOUSE_OVER_TIME,
  SET_PAGE_SCROLL_INFO
} from 'actions/monitoring/layout';

import { MIN_CHART_AREA_WIDTH_PX } from 'constants/monitoring/layout';

import type {
  SetTimePickerVisibilityAction,
  ChartAreaWidthAction,
  MouseOverTimeAction,
  SetPageScrollInfoAction
} from 'actions/monitoring/layout';

export type LayoutState = {
  timePickerVisible: boolean,
  chartAreaWidthPx: number,
  mouseOverTime: ?Date
};

export const defaultState: LayoutState = {
  timePickerVisible: false,
  chartAreaWidthPx: MIN_CHART_AREA_WIDTH_PX,
  mouseOverTime: null
};

function setTimePickerVisibility(state: LayoutState, action: SetTimePickerVisibilityAction): LayoutState {
  return {
    ...state,
    timePickerVisible: action.payload.isVisible
  };
}

function setChartAreaWidth(state: LayoutState, action: ChartAreaWidthAction): LayoutState {
  if (action.payload.widthPixels < MIN_CHART_AREA_WIDTH_PX) {
    return state;
  }
  return {
    ...state,
    chartAreaWidthPx: action.payload.widthPixels
  };
}

function setMouseOverTime(state: LayoutState, action: MouseOverTimeAction): LayoutState {
  return {
    ...state,
    mouseOverTime: action.payload.mouseOverTime
  };
}

function setPageScrollInfo(state: LayoutState, action: SetPageScrollInfoAction): LayoutState {
  return {
    ...state,
    pageScrollOffsetY: action.payload.offsetY,
    pageHeight: action.payload.pageHeight,
    windowHeight: action.payload.windowHeight
  };
}

export function layout(state: LayoutState = defaultState, action: Action<*, *>): LayoutState {
  switch (action.type) {
    case SET_TIME_PICKER_VISIBILITY:
      return setTimePickerVisibility(state, action);
    case SET_CHART_AREA_WIDTH:
      return setChartAreaWidth(state, action);
    case SET_MOUSE_OVER_TIME:
      return setMouseOverTime(state, action);
    case SET_PAGE_SCROLL_INFO:
      return setPageScrollInfo(state, action);
    default:
      return state;
  }
}
