// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import debug from 'debug';
import { fork, select } from 'redux-saga/effects';
import { actionTypes } from 'redux-form';
import { takeEvery } from 'redux-saga';

import alertsNewActionFormSaga from './alerts/newAction';
import alertsNewTriggerFormSaga from './alerts/newTrigger';
import { FormTypes } from 'types/form';
import { clusterFormChangeSaga } from './clusters';

import type { FormChangeAction } from 'types/form';

const log = debug('controlcenter:sagas:form');


// note: CONNECTOR_FORM form change events are handled by management sagas.
export const FormChangeSagas = {
  [FormTypes.ALERTS_ACTION_FORM]: alertsNewActionFormSaga,
  [FormTypes.ALERTS_TRIGGER_FORM]: alertsNewTriggerFormSaga,
  [FormTypes.CLUSTER_FORM]: clusterFormChangeSaga,
  [FormTypes.NONE]: null
};

export function* formChangedSaga(): Generator<*, *, *> {
  yield* takeEvery(actionTypes.CHANGE, function* change(action: FormChangeAction) {
    const currentForm = action.meta.form;
    const formChangeSaga = FormChangeSagas[currentForm];
    if (formChangeSaga) {
      yield fork(formChangeSaga, action);
    } else {
      const route: any = yield select(state => state.routing.locationBeforeTransitions.pathname);
      log(`no change saga defined for path ${route}`);
    }
  });
}

export default formChangedSaga;
