// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { createSelector } from 'reselect';

export function historyItemSelector(state: Object, itemId: string) {
  const historyItem = state.alerts.history.main.find(item => itemId.endsWith(item.guid));
  if (historyItem === undefined) {
    if (state.alerts.history.item.alertInfo != null && itemId.endsWith(state.alerts.history.item.alertInfo.guid)) {
      return state.alerts.history.item.alertInfo;
    } else {
      return null;
    }
  }
  return historyItem;
}

// most recent history item out of new and main lists.
export const mostRecentHistoryItemSelector = (state: Object) => {
  if (state.alerts.history.new && state.alerts.history.new.length > 0) {
    return state.alerts.history.new[0];
  }
  return state.alerts.history.main && state.alerts.history.main.length > 0
    ? state.alerts.history.main[0]
    : null;
};


export const earliestHistoryItemSelector = (state: Object) =>
  state.alerts.history.main && state.alerts.history.main.length > 0
    ? state.alerts.history.main[state.alerts.history.main.length-1]
    : null;

export const newHistoryCount = (state: Object) =>
  state.alerts.history.new
    ? state.alerts.history.new.length
    : 0;

const alertsSelector = state => state.alerts;

export const triggerToDelete = createSelector(
  [alertsSelector],
  (alerts: Object): Object => {
    const triggers = alerts.triggers == null ? [] : alerts.triggers;
    const selected = triggers.filter(
      trigger => trigger.guid === alerts.toDelete.triggerGuid
    );
    if (selected.length === 0) {
      return { guid: null, name: null };
    }
    return selected[0];
  }
);

export const actionToDelete = createSelector(
  [alertsSelector],
  (alerts: Object): Object => {
    const actions = alerts.actions == null ? [] : alerts.actions;
    const selected = actions.filter(
      action => action.guid === alerts.toDelete.actionGuid
    );
    if (selected.length === 0) {
      return { guid: null, name: null };
    }
    return selected[0];
  }
);
