// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { createSelector } from 'reselect';
import { get, memoize, sortBy } from 'lodash';
import { locationBeforeTransitionsSelector } from './routing';

import type {
  Cluster,
  ClusterId,
  ClusterList,
  ClusterOption,
  ClusterOptions,
  ClusterType,
  ClustersState
} from 'types/clusters';

// TODO(@dylanharrington): put in a common place since this is duplicated in the saga
const getClusterQueryParam = (clusterType: ClusterType) => clusterType === 'MONITORING' ? 'monitoringClusterId' : 'connectClusterId';

// TODO(@dylanharrington): differentiate between keys on the cluster reducer and "ClusterType"... for now memoize
// hides the errors
const stateClusterSelectorFactory = (clusterType: ClusterType) => state => state.clusters[clusterType];

const getSortedClusterListSelector = memoize((clusterType: ClusterType) => createSelector(
  [stateClusterSelectorFactory(clusterType)],
  (clusters: ClustersState): ClusterList => sortBy(clusters, 'displayName')
));

export const getClusterOptionsSelector = memoize((clusterType: ClusterType) => createSelector(
  [getSortedClusterListSelector(clusterType)],
  (clusters: ClusterList): ClusterOptions => clusters.map((cluster: Cluster) => ({
    label: cluster.displayName,
    value: cluster.clusterId
  }))
));

export const getSelectedClusterIdSelector = memoize((clusterType: ClusterType) => createSelector(
  [locationBeforeTransitionsSelector],
  ({ query }: Object): ClusterId => get(query, getClusterQueryParam(clusterType), null)
));

export const getSelectedClusterOptionSelector = memoize((clusterType: ClusterType) => createSelector(
  [getClusterOptionsSelector(clusterType), getSelectedClusterIdSelector(clusterType)],
  (clusterOptions: ClusterOptions, selectedClusterId: ClusterId): ?ClusterOption => clusterOptions.find(
    (cluster: ClusterOption) => cluster.value === selectedClusterId
  )
));

export const clusterIndexSelector = getSortedClusterListSelector('ALL');
