// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { flatten, get } from 'lodash';
import { createSelector } from 'reselect';

import { FormTypes } from 'types/form';

const managementSelector = state => state.management;
const connectorFormSelector = createSelector(managementSelector, management => management[FormTypes.CONNECTOR_FORM]);

export const connectorStatusByNameSelector = createSelector(managementSelector, management => management.connectors.statusByName);

export const currentFormTypeSelector = createSelector(connectorFormSelector, form => get(form, 'type', ''));

export const currentStepKeySelector = createSelector(connectorFormSelector, form => get(form, 'currentStep', ''));

export const currentStepsSelector = createSelector(connectorFormSelector, form => get(form, 'steps', {}));

export const currentStepSelector = createSelector([currentStepsSelector, currentStepKeySelector], (steps, stepKey) => get(steps, stepKey, {}));

export const currentStepIsAccessibleSelector = createSelector(currentStepSelector, currentStep => get(currentStep, 'canAccess', false));

export const currentFieldSectionsSelector = createSelector(currentStepSelector, step => get(step, 'fieldSections', []));

export const errorsAreVisibleSelector = createSelector(currentStepSelector, step => get(step, 'errorsAreVisible'));

function fieldSectionsToFields(fieldSections) {
  return flatten(
    fieldSections.map(section => (
      section.fields.map(field => field.id)
    ))
  );
}

export const currentFieldsSelector = createSelector(currentFieldSectionsSelector, fieldSections => fieldSectionsToFields(fieldSections));

export const allFieldsSelector = createSelector(currentStepsSelector, steps => flatten(
  Object.keys(steps).map(stepKey => fieldSectionsToFields(steps[stepKey].fieldSections))
));

export const optionsByFieldSelector = createSelector(connectorFormSelector, form => get(form, 'optionsByField'));
