// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import createLogger from 'redux-logger';
import createSagaMiddleware, { END } from 'redux-saga';
import thunkMiddleware from 'redux-thunk';
import { applyMiddleware, compose, createStore } from 'redux';
import { routerMiddleware } from 'react-router-redux';

import makeRootReducer from 'reducers';

import type { Store } from 'types';

export default (initialState: Object = {}, history: Object): Store => {

  // ======================================================
  // Middleware Configuration
  // ======================================================
  const sagaMiddleware = createSagaMiddleware();

  const middleware = [
    sagaMiddleware,
    thunkMiddleware,
    routerMiddleware(history)
  ];

  if (__DEV__) {
    middleware.push(createLogger({ collapsed: true }));
  }

  // ======================================================
  // Store Enhancers
  // ======================================================
  const enhancers = [];

  if (__DEV__) {
    const devToolsExtension = window.devToolsExtension;
    if (typeof devToolsExtension === 'function') {
      enhancers.push(devToolsExtension());
    }
  }

  // ======================================================
  // Store Instantiation and HMR Setup
  // ======================================================
  const store: Store = createStore(
    makeRootReducer(),
    initialState,
    compose(
      applyMiddleware(...middleware),
      ...enhancers
    )
  );

  if (typeof module.hot === 'object' && module.hot !== null && typeof module.hot.accept === 'function') {
    module.hot.accept('../reducers', () => {
      // eslint-disable-next-line import/newline-after-import
      const reducers = require('../reducers').default;
      store.replaceReducer(reducers);
    });
  }

  store.runSaga = sagaMiddleware.run;
  store.close = () => store.dispatch(END);

  return store;
};
