// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import type { EnabledDisabledType } from 'types';


export const ActionTypes = {
  SEND_EMAIL: 'SEND_EMAIL'
};

export type ActionType = $Enum<typeof ActionTypes>;


export const AlertSources = {
  CONSUMER_GROUP: 'CONSUMER_GROUP',
  KAFKA_CONNECTOR: 'KAFKA_CONNECTOR'
};

export type AlertSourceType = $Enum<typeof AlertSources>;


export const MetricTypes = {
  MAX_LATENCY: 'MAX_LATENCY',
  AVG_LATENCY: 'AVG_LATENCY',
  CONSUMPTION_DIFF: 'CONSUMPTION_DIFF',
  CONNECTOR_STATE: 'CONNECTOR_STATE'
};

export type MetricType = $Enum<typeof MetricTypes>;


export const ConditionTypes = {
  GREATER_THAN: 'GREATER_THAN',
  LESS_THAN: 'LESS_THAN',
  EQUAL: 'EQUAL',
  NOT_EQUAL: 'NOT_EQUAL'
};

export type ConditionType = $Enum<typeof ConditionTypes>;


export const SelectOptionListIds = {
  CONNECTORS: 'CONNECTORS',
  CONSUMER_GROUPS: 'CONSUMER_GROUPS'
};


export const TriggerFieldIds = {
  TRIGGER_NAME: 'TRIGGER_NAME',
  ALERT_SOURCE: 'ALERT_SOURCE',
  CONSUMER_GROUP_TARGET: 'CONSUMER_GROUP_TARGET',
  CONNECTOR_TARGET: 'CONNECTOR_TARGET',
  METRIC: 'METRIC',
  CONDITION: 'CONDITION',
  VALUE: 'VALUE',
  BUFFER: 'BUFFER'
};


export type TriggerSummary = {
  guid: string,
  name: string,
  actionCount: number
};

export type Guid = string;

export type TriggerInfo = {
  guid: string,
  name: string,
  alertSource: AlertSourceType,
  group: string | null,
  connector: string | null,
  metric: MetricType,
  condition: ConditionType,
  longValue: number,
  lagMs: number,
};

export type TriggerInstance = Object;

export type ActionSummary = {
  guid: string,
  name: string,
  maxFrequencyPerDay: number,
  triggers: Array<Object>,
  triggerCount: number,
  status: EnabledDisabledType
};

export type ActionShortSummary = {
  guid: string,
  name: string
};

export type AlertSummary = {
  guid: string,
  timestamp: number,
  value: number,
  monitoringTrigger: TriggerInfo,
  actions: Array<ActionShortSummary>,
  triggers: Array<TriggerInstance>
};

export type ActionInfo = {
  giud: string,
  name: string,
  enabled: boolean,
  triggerGuid: Array<Guid>,
  actionType: ActionType,
  email: {
    address: string,
    subject: string
  },
  maxSendRateNumerator: number,
  intervalMs: number
};

export type TimeGuid = {
  timestamp: number,
  guid: string
};
