// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import type { InputType } from 'constants/management';
import type { ConnectorType } from 'actions/connectors';

export type FormValue = any;

export type EventOrValue = SyntheticEvent | FormValue;
export type EventOrSubmit = any;

export type FormField = {
  checked?: boolean,
  disabled: boolean,
  name: string,
  onBlur: (eventOrValue: EventOrValue) => void,
  onChange: (eventOrValue: EventOrValue) => void,
  onDragStart: Function,
  onDrop: Function,
  onFocus: Function,
  onUpdate: Function,
  pristine: boolean,
  touched: boolean,
  valid: boolean,
  value: FormValue,
  visited: boolean
};

export type ReduxFormPassedInput = FormField;

export type Option = {
  label: string,
  value: any
};

type FieldOption = Array<Option>;
type FieldOptions = { [key: string]: FieldOption };

export type ReduxFormContainerProps = {
  asyncValidate: Function,
  asyncValidating: string | boolean,
  destroy: Function,
  dirty: boolean,
  error: any,
  form: string,
  handleSubmit: (eventOrSubmit: EventOrSubmit) => any,
  initialize: (data: Object) => any,
  invalid: boolean,
  pristine: boolean,
  reset: Function,
  submitFailed: boolean,
  submitting: boolean,
  touch: Function,
  untouch: Function,
  valid: boolean
};

type todoAny = any;

export type FieldSectionField = {
  colSpan: number,
  defaultValue?: any,
  documentation?: string,
  errors: Array<string>,
  group: string | null,
  id: string,
  importance: 'HIGH' | 'MEDIUM' | 'LOW',
  order: number,
  inputProps?: FormField,
  inputType: InputType,
  instantErrorFeedback: boolean,
  locked: boolean,
  multi?: boolean,
  offText?: string,
  onText?: string,
  options?: FieldOption,
  placeholder: string,
  required: boolean,
  value: todoAny,
  visible: boolean,
  dependents: todoAny
};

export type FieldSections = Array<{
  title: string,
  fields: Array<FieldSectionField>
}>;

// ---------------------------------
export type FormStep = {
  canAccess?: boolean,
  fieldSections: FieldSections,
  navLabel: string,
  nextStep?: FormStepKey,
  previousStep?: FormStepKey,
  onSubmit: (values: Object, dispatch: Function) => void,
  asyncValidate?: (values: Object, dispatch: Function) => Promise<*>,
  validate?: (values: Object) => Promise<*> | Object
};

export type FormStepKey = string;

export type FormSteps = { [key: FormStepKey]: FormStep };

export type FormConfig = {
  firstStep: FormStepKey,
  isEdit: boolean,
  lastStep: FormStepKey,
  pathname?: string,
  redirectPathname: string,
  steps: FormSteps
};

export type FormConfigs = { [key: string]: FormConfig };

// ---------------------------------
export type Fields = Array<string>;

type FormStateStep = {
  canAccess: boolean,
  errorsAreVisible: boolean,
  fieldSections: FieldSections,
  fields: Fields
};

export type FormStateSteps = { [key: string]: $Shape<FormStateStep> };

export type FormState = {
  completed: boolean,
  currentPathname: string,
  currentStep: FormStepKey,
  errorCount: number,
  initialValues: Object,
  isEdit: boolean,
  loadingConnector: boolean,
  optionsByField: FieldOptions,
  steps: FormStateSteps,
  type: ConnectorType
};

export type FormStates = { [key: string]: FormState };

export type ReduxFormAction = {
  meta: {
    field: string,
    form: string
  },
  payload: FormValue
};

export type FormBlurAction = ReduxFormAction;
export type FormChangeAction = ReduxFormAction;


export const FormTypes = {
  NONE: 'NONE',
  CONNECTOR_FORM: 'CONNECTOR_FORM',
  CLUSTER_FORM: 'CLUSTER_FORM',
  ALERTS_TRIGGER_FORM: 'ALERTS_TRIGGER_FORM',
  ALERTS_ACTION_FORM: 'ALERTS_ACTION_FORM'
};

export type FormType = $Enum<typeof FormTypes>;

export type FieldValue = {
  field: string,
  value: string
};
