// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

export type RangeRequest = {
  clientType: string,
  topic: string,
  clientId: string,
  rollup: string,
  metrics: Array<string>,
  startTimeMs: number,
  stopTimeMs: number
};

export type MetricData = {
  metricName: string,
  values: Array<number>
};

export type RangeResult = {
  id: string,
  clientId: string,
  clientType: string,
  group: string,
  partition: ?number,
  rollup: string,
  topic: string,
  startTimeMs: number,
  stepMs: number,
  data: Array<MetricData>
};

export type TimeSeries = {
  startTimeMs: number,
  stepMs: number,
  values: Array<?number>
};

export type RawLatencyData = {
  totalLatency: Array<?number>,
  count: Array<?number>,
  minLatency: Array<?number>,
  maxLatency: Array<?number>
}

export type LatencyData = {
  averageLatency: Array<?number>,
  minLatency: Array<?number>,
  maxLatency: Array<?number>
}

export type MultiTimeSeries<T> = {
  startTimeMs: number,
  stepMs: number,
  values: T
}

export const DataLayoutTypes = {
  CONSUMER_GROUPS: 'CONSUMER_GROUPS',
  CONSUMER_GROUP: 'CONSUMER_GROUP',
  CONSUMER_GROUPS_TOPICS: 'CONSUMER_GROUPS_TOPICS',
  CONSUMER_GROUP_TOPICS: 'CONSUMER_GROUP_TOPICS',
  CONSUMER: 'CONSUMER'
};

export type DataLayoutType = $Enum<typeof DataLayoutTypes>;

export const RequestTypes = {
  MEMBER_LIST: 'MEMBER_LIST',
  MEMBER: 'MEMBER',
  OVERVIEW: 'OVERVIEW'
};

export type RequestType = $Enum<typeof RequestTypes>;
