// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import type { FrequencySpec } from 'types/time';
import { FrequencySpecUnits } from 'types/time';

export const MILLISECONDS_ONE_MINUTE = 60 * 1000;
export const MILLISECONDS_ONE_HOUR = 60 * MILLISECONDS_ONE_MINUTE;
export const MILLISECONDS_FOUR_HOURS = 4 * MILLISECONDS_ONE_HOUR;
export const MILLISECONDS_EIGHT_HOURS = 2 * MILLISECONDS_FOUR_HOURS;
export const MILLISECONDS_ONE_DAY = 3 * MILLISECONDS_EIGHT_HOURS;

export function frequencySpecToString(frequencySpec: FrequencySpec): string {
  let postfix = '';
  switch (frequencySpec.units) {
    case FrequencySpecUnits.MINUTES:
      postfix = 'per minute';
      break;
    case FrequencySpecUnits.HOUR:
      postfix = 'per hour';
      break;
    case FrequencySpecUnits.FOUR_HOURS:
      postfix = 'per 4 hours';
      break;
    case FrequencySpecUnits.EIGHT_HOURS:
      postfix = 'per 8 hours';
      break;
    case FrequencySpecUnits.DAY:
      postfix = 'per day';
      break;
  }

  return `${frequencySpec.value} ${postfix}`;
}

export function frequencySpecToMinutes(frequencySpec: FrequencySpec): number {
  switch (frequencySpec.units) {
    case FrequencySpecUnits.MINUTES:
      return frequencySpec.value;
    case FrequencySpecUnits.HOUR:
      return frequencySpec.value * 60;
    case FrequencySpecUnits.FOUR_HOURS:
      return frequencySpec.value * 60 * 4;
    case FrequencySpecUnits.EIGHT_HOURS:
      return frequencySpec.value * 60 * 8;
    case FrequencySpecUnits.DAY:
      return frequencySpec.value * 60 * 24;
  }
  throw new Error(`invalid units type: ${frequencySpec.units}`);
}

export function minutesToFrequencySpec(perDay: number): FrequencySpec {
  const per8Hours = perDay / 3;
  const per4Hours = per8Hours / 2;
  const perHour = per4Hours / 4;
  const perMinute = perHour / 60;

  if (per8Hours < 0 || Math.floor(per8Hours) !== per8Hours) {
    return { value: perDay, units: FrequencySpecUnits.DAY };
  }

  if (per4Hours < 0 || Math.floor(per4Hours) !== per4Hours) {
    return { value: per8Hours, units: FrequencySpecUnits.EIGHT_HOURS };
  }

  if (perHour < 0 || Math.floor(perHour) !== perHour) {
    return { value: per4Hours, units: FrequencySpecUnits.FOUR_HOURS };
  }

  if (perMinute < 0 || Math.floor(perMinute) !== perMinute) {
    return { value: perHour, units: FrequencySpecUnits.HOUR };
  }

  return { value: perMinute, units: FrequencySpecUnits.MINUTES };
}
