// @flowfixme
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import { storiesOf, action } from '@kadira/storybook';

import Dropdown from 'components/Dropdown';
import { Select } from 'components/form';

import twoThousandRows from './mock-data/list.json';

const options = twoThousandRows.slice(0, 100).map(item => ({
  label: item.name,
  value: item._id
}));

const selectPropAdapter = props => ({
  clearable: props.clearable,
  reduxFormInputProps: {
    onBlur: props.onBlur,
    onChange: props.onChange,
    onFocus: props.onFocus,
    value: props.value
  },
  sectionField: {
    id: `${Math.random()}`,
    locked: props.locked,
    multi: props.multi,
    options: props.options,
    placeholder: props.placeholder
  },
  searchable: props.searchable
});

const dropdownPropAdapter = props => ({
  clearable: props.clearable,
  disabled: props.locked,
  multi: props.multi,
  onBlur: props.onBlur,
  onChange: props.onChange,
  onFocus: props.onFocus,
  options: props.options,
  placeholder: props.placeholder,
  searchable: props.searchable,
  value: props.value
});

const renderDropdown = props => (
  <div key={`dropdown-${props.placeholder}`} style={{ minHeight: '11rem' }}>
    <h3 title={JSON.stringify(selectPropAdapter({ ...props, options: '[ ... ]' }), null, 2)}>{props.placeholder}</h3>
    <Dropdown {...dropdownPropAdapter(props)} />
  </div>
);

const renderSelect = props => (
  <div key={`select-${props.placeholder}`} style={{ minHeight: '11rem' }}>
    <h3 title={JSON.stringify(selectPropAdapter({ ...props, options: '[ ... ]' }), null, 2)}>{props.placeholder}</h3>
    <Select {...selectPropAdapter(props)} />
  </div>
);

const baseProps = {
  active: false,
  clearable: false,
  locked: false,
  multi: false,
  onBlur: action('onBlur'),
  onChange: action('onChange'),
  onFocus: action('onFocus'),
  options: options,
  placeholder: 'placeholder',
  searchable: false,
  showErrorIndicator: false,
  showInfoIcon: false,
  value: options[0].value
};

const allProps = {
  // active: { ...baseProps, active: true, placeholder: 'active' },
  clearable: { ...baseProps, clearable: true, placeholder: 'clearable' },
  locked: { ...baseProps, locked: true, placeholder: 'locked' },
  multi: { ...baseProps, multi: true, placeholder: 'multi' },
  multiEmpty: { ...baseProps, multi: true, placeholder: 'multi empty', value: undefined },
  searchable: { ...baseProps, searchable: true, placeholder: 'searchable' }
};

const allPropsArr = Object.keys(allProps).map(key => allProps[key]);

const renderAllSelects = () => (
  <div
    style={{
      // background: 'orange'
    }}
  >
    {allPropsArr.map(renderSelect)}
  </div>
);

const renderAllDropdowns = () => (
  <div
    style={{
      // background: 'red'
    }}
  >
    {allPropsArr.map(renderDropdown)}
  </div>
);

storiesOf('Dropdown', module)
  .add('Side by side', () => (
    <div
      style={{
        background: '#444',
        padding: '10px',
        margin: '10px'
      }}
    >
      <div style={{ verticalAlign: 'top', display: 'inline-block', width: '50%', padding: '0 5px 0 10px' }}>
        <h2>Dropdowns</h2>
        {renderAllDropdowns(baseProps)}
      </div>
      <div style={{ verticalAlign: 'top', display: 'inline-block', width: '50%', padding: '0 10px 0 5px' }}>
        <h2>Selects</h2>
        {renderAllSelects(baseProps)}
      </div>
      <p>Note: Hover item titles to see the props that were passed</p>
    </div>
  ))
  .add('Dropdown', renderAllDropdowns)
  .add('Form Select', renderAllSelects)
;
