// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React, { Component } from 'react';
import { storiesOf } from '@kadira/storybook';

import { PillBox, Pill } from 'components/pills';
import twoThousandRows from './mock-data/list.json';

const renderPillBox = (pillData) => (
  <PillBox>
    {pillData.map(row => (
       <Pill key={row._id}>
         {row.name}
       </Pill>
    ))}
  </PillBox>
);

type PillRendererState = {
  pillCount: number
};

class PillRenderer extends Component {
  props: {
    width: string | number,
    pillData: Array<any>,
    children?: Function
  };

  state: PillRendererState;

  static defaultProps = {
    pillData: twoThousandRows
  };

  constructor(props) {
    super(props);

    this.state = {
      pillCount: props.pillData.length
    };
  }

  render() {
    const { children } = this.props;
    return (
      <div
        style={{
          background: '#444',
          margin: '10px 0',
          padding: '10px',
          width: this.props.width
        }}
      >
        <input
          style={{
            width: '100%'
          }}
          max={this.props.pillData.length}
          min={0}
          step={1}
          type="range"
          value={this.state.pillCount}
          onChange={(event) => this.setState({ pillCount: event.target.value })}
        />
        {typeof children === 'function' && children({
          pills: this.props.pillData.slice(0, this.state.pillCount)
        })}
      </div>
    );
  }
}

storiesOf('Pillbox', module)
  .add('Yo', () => (
    <div
      style={{
        padding: '10px',
        margin: '10px'
      }}
    >
      <div style={{ verticalAlign: 'top', padding: '0 10px 0 5px' }}>

        <h2>Static width container</h2>

        <PillRenderer width={100} pillData={twoThousandRows.slice(0, 100)}>
          {({ pills }) => renderPillBox(pills)}
        </PillRenderer>

        <PillRenderer width={200} pillData={twoThousandRows.slice(0, 200)}>
          {({ pills }) => renderPillBox(pills)}
        </PillRenderer>

        <PillRenderer width={400} pillData={twoThousandRows.slice(0, 400)}>
          {({ pills }) => renderPillBox(pills)}
        </PillRenderer>

        <PillRenderer width={800} pillData={twoThousandRows.slice(0, 600)}>
          {({ pills }) => renderPillBox(pills)}
        </PillRenderer>

        <h2>Dynamic width container</h2>
        <PillRenderer width="100%">
          {({ pills }) => renderPillBox(pills)}
        </PillRenderer>

      </div>
    </div>
  ))
;
