// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import { shallow } from 'enzyme';

import {
  calculateAggregateLatency,
  AggregateLatency
} from 'components/aggregate-stats/AggregateLatency';

import styles from 'components/aggregate-stats/AggregateStats.css';


describe('#getAggregateLatency', () => {
  it('returns overall latency', () => {
    const val1 = calculateAggregateLatency(
      {
        startTimeMs: 0,
        stepMs: 15000,
        values: {
          totalLatency: [6789, 6789, 6789, 6789, 6789, 6789],
          count: [1, 1, 1, 1, 1, 1],
          minLatency: [],
          maxLatency: []
        }
      },
      Infinity
    );
    const val2 = calculateAggregateLatency(
      {
        startTimeMs: 0,
        stepMs: 15000,
        values: {
          totalLatency: [6789, 6789, 6789, 6789, 6789, 6789],
          count: [10, 10, 10, 10, 10, 10],
          minLatency: [],
          maxLatency: []
        }
      },
      Infinity
    );
    expect(val1).to.not.be.null;
    expect(val2).to.not.be.null;
    if (val1 != null && val2 != null) {
      expect(`${val1.toFixed(2)}`).to.eql('6.79');
      expect(`${val2.toFixed(2)}`).to.eql('0.68');
    }
  });
});

describe('<AggregateLatency />', () => {
  it('renders', () => {
    const aggregateLatency = 100;

    const wrapper = shallow(
      <AggregateLatency
        aggregateLatency={aggregateLatency}
      />
    );
    expect(wrapper.find({ className: styles['item-stat'] })).to.have.length(1);
  });
});
