// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { expect } from 'chai';
import React from 'react';
import { shallow } from 'enzyme';

import styles from 'components/charting/Axis.css';
import { Axis } from 'components/charting/Axis';

describe('<Axis />', () => {
  const xScale = (x) => x instanceof Date ? x.getTime() : x;
  xScale.ticks = () => [new Date(0), new Date(50), new Date(100)];
  xScale.domain = () => [new Date(0), new Date(100)];
  xScale.range = () => [0, 100];

  it('renders', () => {
    const wrapper = shallow(
      <Axis xScale={xScale} missingRegions={[]} />
    );
    expect(wrapper.find({ className: styles['root'] })).to.be.present();
  });

  describe('ticks', () => {
    it('renders ticks', () => {
      const wrapper = shallow(
        <Axis xScale={xScale} missingRegions={[]} />
      );
      const ticks = wrapper.find({ className: styles['axis-label'] });
      expect(ticks).to.have.length(3);
      ticks.forEach(tick => expect(tick.childAt(0).props().className).to.eql(styles['axis-tick-label-strong']));
    });

    it('renders ticks moused over', () => {
      const wrapper = shallow(
        <Axis
          xScale={xScale}
          missingRegions={[]}
          mouseOverTime={new Date(50)}
        />
      );
      const ticks = wrapper.find({ className: styles['axis-label'] });
      expect(ticks).to.have.length(4);
      expect(ticks.at(0).childAt(0).props().className).to.eql(styles['axis-tick-label']);
      expect(ticks.at(1).childAt(0).props().className).to.eql(styles['axis-tick-label']);
      expect(ticks.at(2).childAt(0).props().className).to.eql(styles['axis-tick-label']);
      expect(ticks.at(3).children()).to.have.length(2);
      expect(ticks.at(3).childAt(0).props().className).to.eql(styles['axis-tick']);
    });
  });

  describe('missing regions', () => {
    it('0 regions', () => {
      const wrapper = shallow(
        <Axis xScale={xScale} missingRegions={[]} />
      );
      const missingRegions = wrapper.find({ className: styles['axis-styling'] });
      expect(missingRegions.children()).to.have.length(0);
    });

    describe('1 region', () => {
      it('internal', () => {
        const missingRegion = {
          startTime: new Date(5),
          endTime: new Date(55)
        };
        const wrapper = shallow(
          <Axis xScale={xScale} missingRegions={[missingRegion]} />
        );
        const missingRegions = wrapper.find({ className: styles['axis-styling'] });
        expect(missingRegions.children()).to.have.length(1);
      });

      it('leading edge', () => {
        const missingRegion = {
          startTime: new Date(0),
          endTime: new Date(55)
        };
        const wrapper = shallow(
          <Axis xScale={xScale} missingRegions={[missingRegion]} />
        );
        const missingRegions = wrapper.find({ className: styles['axis-styling'] });
        expect(missingRegions.children()).to.have.length(1);
      });

      it('trailing edge', () => {
        const missingRegion = {
          startTime: new Date(5),
          endTime: new Date(100)
        };
        const wrapper = shallow(
          <Axis xScale={xScale} missingRegions={[missingRegion]} />
        );
        const missingRegions = wrapper.find({ className: styles['axis-styling'] });
        expect(missingRegions.children()).to.have.length(1);
      });

      it('start before', () => {
        const missingRegion = {
          startTime: new Date(-1),
          endTime: new Date(5)
        };
        const wrapper = shallow(
          <Axis xScale={xScale} missingRegions={[missingRegion]} />
        );
        const missingRegions = wrapper.find({ className: styles['axis-styling'] });
        expect(missingRegions.children()).to.have.length(1);
      });

      it('end after', () => {
        const missingRegion = {
          startTime: new Date(33),
          endTime: new Date(106)
        };
        const wrapper = shallow(
          <Axis xScale={xScale} missingRegions={[missingRegion]} />
        );
        const missingRegions = wrapper.find({ className: styles['axis-styling'] });
        expect(missingRegions.children()).to.have.length(1);
      });
    });

    describe('2 regions', () => {
      it('internal', () => {
        const missingRegion1 = {
          startTime: new Date(3),
          endTime: new Date(12)
        };
        const missingRegion2 = {
          startTime: new Date(21),
          endTime: new Date(66)
        };

        const wrapper = shallow(
          <Axis xScale={xScale} missingRegions={[missingRegion1, missingRegion2]} />
        );
        const missingRegions = wrapper.find({ className: styles['axis-styling'] });
        expect(missingRegions.children()).to.have.length(2);
      });
    });
  });
});
