// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import {
  calculateDataBounds,
  aggregateCountToSlots,
  aggregateTotalCountMinMaxToSlots
} from 'components/charting/util';


describe('calculateDataBounds', () => {
  it('finds bounds, multiple elements', () => {
    expect(calculateDataBounds([10, 1, 3]).min).to.be.at.most(1);
    expect(calculateDataBounds([10, 1, 3]).max).to.be.at.least(10);
  });

  it('finds bounds, single element', () => {
    expect(calculateDataBounds([1]).min).to.be.at.most(1);
    expect(calculateDataBounds([1]).max).to.be.at.least(1);
  });

  it('finds bounds, no elements', () => {
    expect(calculateDataBounds([]).min).to.be.Infinity;
    expect(calculateDataBounds([]).max).to.be.Infinity;
  });
});

describe('aggregateCountToSlots', () => {
  it('basic case', () => {
    const data = {
      startTimeMs: 0,
      stepMs: 15000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    };
    const resampled = aggregateCountToSlots(data, 30000, [new Date(0), new Date(150000)]);
    expect(resampled).to.not.be.null;
    if (resampled == null) {
      return;
    }
    expect(resampled.startTimeMs).to.eql(0);
    expect(resampled.stepMs).to.eql(30000);
    expect(resampled.values.length).to.eql(5);
    expect(resampled.values[0]).to.almost.eql(3);
    expect(resampled.values[4]).to.almost.eql(19);
  });
});

describe('aggregateTotalCountMinMaxToSlots', () => {
  it('test many things', () => {
    const dataTCMM = {
      startTimeMs: 0,
      stepMs: 15000,
      values: {
        totalLatency: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        count: [1, 1, 1, 1, 1, 1, 1, 1, 2, 2],
        minLatency: [0, 3, 3, 0, 1, 1, 1, 1, 1, 1],
        maxLatency: [1, 4, 4, 1, 2, 2, 2, 2, 2, 2]
      }
    };
    const resampled = aggregateTotalCountMinMaxToSlots(dataTCMM, 30000, [new Date(0), new Date(150000)]);
    expect(resampled).to.not.be.null;
    if (resampled == null) {
      return;
    }
    expect(resampled.startTimeMs).to.eql(0);
    expect(resampled.stepMs).to.eql(30000);
    expect(Object.keys(resampled.values).length).to.eql(3);
    expect(resampled.values.averageLatency.length).to.eql(5);
    expect(resampled.values.averageLatency[0]).to.almost.eql(1.5);
    expect(resampled.values.averageLatency[4]).to.almost.eql(4.75);
    expect(resampled.values.minLatency[0]).to.eql(0);
    expect(resampled.values.maxLatency[0]).to.eql(4);
    expect(resampled.values.minLatency[1]).to.eql(0);
    expect(resampled.values.maxLatency[1]).to.eql(4);
    expect(resampled.values.minLatency[4]).to.eql(1);
    expect(resampled.values.maxLatency[4]).to.eql(2);
  });

  it('data beyond bounds', () => {
    const dataTCMM = {
      startTimeMs: 0,
      stepMs: 15000,
      values: {
        totalLatency: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
        count: [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
        minLatency: [0, 3, 3, 0, 1, 1, 1, 1, 1, 1, 1],
        maxLatency: [1, 4, 4, 1, 2, 2, 2, 2, 2, 2, 2]
      }
    };
    const resampled = aggregateTotalCountMinMaxToSlots(dataTCMM, 30000, [new Date(10000), new Date(150000)]);
    expect(resampled).to.not.be.null;
    if (resampled == null) {
      return;
    }
    expect(resampled.startTimeMs).to.eql(30000);
    expect(resampled.stepMs).to.eql(30000);
    expect(Object.keys(resampled.values).length).to.eql(3);
    expect(resampled.values.averageLatency.length).to.eql(4);
    expect(resampled.values.averageLatency[0]).to.almost.eql(3.5);
    expect(resampled.values.averageLatency[3]).to.almost.eql(9.5);
  });
});
