// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import { mount } from 'enzyme';

import { Label } from 'components/form/Label';
import styles from 'components/form/Label.css';

describe('<Label />', () => {
  let props;

  beforeEach(() => {
    props = {
      id: `id-Label-${Date.now()}`,
      placeholder: 'placeholder'
    };
  });

  it('renders', () => {
    const wrapper = mount(<Label {...props} />);
    expect(wrapper.find({ className: styles['label-root'] })).to.be.present();
  });

  it('passes id to <label>\'s for attribute', () => {
    const wrapper = mount(<Label {...props} />);
    expect(wrapper.find({ htmlFor: props.id })).to.have.attr('for', props.id);
  });

  describe('`disabled`', () => {
    it('true instructs the root element to have disabled styling', () => {
      props = {
        ...props,
        disabled: true
      };

      const wrapper = mount(<Label {...props} />);
      expect(wrapper.find({ className: styles['label-root-disabled'] })).to.be.present();
    });
  });

  describe('`showLabel`', () => {
    it('true instructs <label> to display as a label', () => {
      props = {
        ...props,
        showLabel: true
      };

      const wrapper = mount(<Label {...props} />);
      expect(wrapper.find({ htmlFor: props.id })).to.have.prop('className', styles['label-mode']);
    });

    it('false instructs <label> to display as a placeholder', () => {
      props = {
        ...props,
        showLabel: false
      };

      const wrapper = mount(<Label {...props} />);
      expect(wrapper.find({ htmlFor: props.id })).to.have.prop('className', styles['placeholder-mode']);
    });
  });

  describe('`staticPlaceholder`', () => {
    it('true instructs <label> to display as a placeholder ands override `showLabel`', () => {
      [
        {
          ...props,
          showLabel: false,
          staticPlaceholder: true
        },
        {
          ...props,
          showLabel: true,
          staticPlaceholder: true
        }
      ].forEach(props => {
        const wrapper = mount(<Label {...props} />);
        expect(wrapper.find({ htmlFor: props.id })).to.have.prop('className', styles['placeholder-static-mode']);
      });
    });
  });
});
