// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import { shallow } from 'enzyme';

import { getBaseFormField } from './util';
import { LabeledSwitch } from 'components/form/LabeledSwitch';
import Label from 'components/form/Label';
import { Switch } from 'components/form';

import styles from 'components/form/LabeledSwitch.css';

describe('<LabeledSwitch />', () => {
  let props;

  beforeEach(() => {
    const { reduxFormInputProps, sectionField } = getBaseFormField('switch');
    props = {
      active: false,
      reduxFormInputProps: {
        ...reduxFormInputProps,
        checked: false
      },
      sectionField: {
        ...sectionField,
        inputType: 'SWITCH'
      }
    };
  });

  it('renders', () => {
    const wrapper = shallow(<LabeledSwitch {...props} />);
    expect(wrapper).to.be.present();
  });

  describe('`field`', () => {
    it('is passed to <Switch />', () => {
      const wrapper = shallow(<LabeledSwitch {...props} />);
      expect(wrapper.find(Switch).props('field', props.reduxFormInputProps));
    });

    it('`active` instructs the inner wrapper styles', () => {
      props.active = false;

      const wrapper = shallow(<LabeledSwitch {...props} />);
      expect(wrapper.find({ className: styles['labeled-switch-inner-root'] })).to.be.present();

      wrapper.setProps({ active: true });
      expect(wrapper.find({ className: styles['labeled-switch-inner-root--active'] })).to.be.present();
    });
  });

  describe('`id`', () => {
    it('is passed to <Label />', () => {
      const wrapper = shallow(<LabeledSwitch {...props} />);
      expect(wrapper.is(Label));
      expect(wrapper.is({ id: props.sectionField.id }));
    });
  });

  describe('`label`', () => {
    it('is passed to <Label /> as `text`', () => {
      const wrapper = shallow(<LabeledSwitch {...props} />);
      expect(wrapper.is(Label));
      expect(wrapper.is({ text: props.sectionField.label }));
    });
  });

  describe('`disabled`', () => {
    beforeEach(() => {
      props = {
        ...props,
        reduxFormInputProps: {
          ...props.reduxFormInputProps,
          disabled: true
        }
      };
    });

    it('is passed to <Label />', () => {
      const wrapper = shallow(<LabeledSwitch {...props} />);
      expect(wrapper.is(Label));
      expect(wrapper.is({ disabled: props.reduxFormInputProps.disabled }));
    });

    it('is passed to <Switch />', () => {
      const wrapper = shallow(<LabeledSwitch {...props} />);
      expect(wrapper.find(Switch).is({ disabled: props.reduxFormInputProps.disabled }));
    });
  });
});
