// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import { mount } from 'enzyme';
import { spy } from 'sinon';

import {
  ConnectorIndex,
  tableHeaderHeight,
  tablePaddingHeight
} from 'components/management/ConnectorIndex';

import { SINK, SOURCE } from 'constants/management';

describe('<ConnectorIndex />', () => {
  const baseProps = {
    connectorType: SINK,
    connectors: [{
      name: 'sweet-sink',
      tasks: 0,
      topics: 'cool,topics',
      type: SINK
    }, {
      name: 'sweet-source',
      tasks: 0,
      topics: 'cool,topics',
      type: SOURCE
    }],
    rowHeight: 20
  };

  it('mounts', () => {
    const props = { ...baseProps };
    const wrapper = mount(<ConnectorIndex {...props} />);
    expect(wrapper).to.be.present();

    wrapper.unmount();
  });

  it('mounts with connectorType as SOURCE', () => {
    const props = {
      ...baseProps,
      connectorType: SOURCE
    };
    const wrapper = mount(<ConnectorIndex {...props} />);
    expect(wrapper).to.be.present();

    wrapper.unmount();
  });

  describe('setHeight', () => {
    const TEST_INNER_HEIGHT = 200;
    let _innerHeight;

    beforeEach(() => {
      _innerHeight = window.innerHeight;
      window.innerHeight = TEST_INNER_HEIGHT;
    });
    afterEach(() => {
      window.innerHeight = _innerHeight;
    });

    it('called when updated only for specific props', () => {
      const props = { ...baseProps };
      const wrapper = mount(<ConnectorIndex {...props} />);

      const instance = wrapper.instance();

      spy(instance, 'setHeight');

      expect(instance.setHeight).to.not.be.calledOnce;

      wrapper.setProps({ rowHeight: 500 });

      expect(instance.setHeight).to.be.calledOnce;

      wrapper.setProps({ randomProp: 'ha!' });

      expect(instance.setHeight).to.be.calledOnce;

      wrapper.unmount();
    });

    it('uses either innerHeight or item * rowHeight', () => {
      const props = {
        ...baseProps,
        throttleInterval: 0
      };
      const wrapper = mount(<ConnectorIndex {...props} />);

      expect(wrapper.state('height')).to.equal(props.connectors.length * props.rowHeight + tableHeaderHeight + tablePaddingHeight);

      wrapper.setProps({
        rowHeight: 500
      });

      expect(wrapper.state('height')).to.equal(TEST_INNER_HEIGHT);
      wrapper.unmount();
    });
  });
});
