// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import { mount } from 'enzyme';

import { StepLoader } from 'components/progress-indicator/StepLoader';
import styles from 'components/progress-indicator/StepLoader.css';

describe('<StepLoader />', () => {
  let props;

  beforeEach(() => {
    props = {};
  });

  it('renders', () => {
    const wrapper = mount(<StepLoader {...props} />);
    expect(wrapper.find({ className: styles.root })).to.be.present();
  });

  describe('`stepTotal`', () => {
    it('shows progress when greater than 1', () => {
      props = {
        ...props,
        stepTotal: 2
      };
      const wrapper = mount(<StepLoader {...props} />);
      expect(wrapper.find({ className: styles.progress })).to.be.present();
    });

    it('hides progress when less than 2', () => {
      const wrapper = mount(<StepLoader {...props} />);
      expect(wrapper.find({ className: styles.progress })).to.not.be.present();
    });
  });

  describe('`step`', () => {
    it('changes text', () => {
      props = {
        ...props,
        stepTotal: 2
      };

      const wrapper1 = mount(<StepLoader {...props} />);
      const progressText1 = wrapper1.find({ className: styles.progress }).text();

      props = {
        ...props,
        step: 2
      };

      const wrapper2 = mount(<StepLoader {...props} />);
      const progressText2 = wrapper2.find({ className: styles.progress }).text();

      expect(progressText1).to.be.not.equal(progressText2);
    });
  });

  describe('`text`', () => {
    it('shows the default', () => {
      const wrapper = mount(<StepLoader {...props} />);
      expect(wrapper.find({ className: styles.text }).text()).to.equal(StepLoader.defaultProps.text);
    });

    it('can be changed', () => {
      props = {
        ...props,
        text: 'testing lol'
      };
      const wrapper = mount(<StepLoader {...props} />);
      expect(wrapper.find({ className: styles.text }).text()).to.equal(props.text);
    });
  });
});
