// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import React from 'react';
import { shallow, mount } from 'enzyme';
import sinon from 'sinon';

import styles from 'components/time-picker/DefaultRanges.css';
import DefaultRanges from 'components/time-picker/DefaultRanges';
import { RangeTypes } from 'constants/monitoring/timeframe';

import type { RangeType } from 'constants/monitoring/timeframe';

describe('<DefaultRanges />', () => {
  const defaultProps = {
    selectedRangeLabel: 'selectedRangeLabel',
    getRanges: () => [
      [
        { startTime: 0, endTime: 100, rangeType: RangeTypes.ROLLING }
      ],
      [
        { startTime: 0, endTime: 100, rangeType: RangeTypes.FIXED }
      ]
    ],
    onHandleDefaultRangeClick: () => 0,
    onHandleDefaultRangeHover: () => 0,
    onHandleDefaultRangeBlur: () => 0,
    getRangeLabel: (startTime: number, endTime: number, rangeType: RangeType) => RangeTypes.ROLLING === rangeType ? 'selectedRangeLabel' : 'not-selectedRangeLabel'
  };

  it('renders', () => {
    const wrapper = shallow(
      <DefaultRanges
        {...defaultProps}
      />
    );
    expect(wrapper.find({ className: styles['default-ranges-root'] })).to.be.present();
    const range = wrapper.find({ className: styles['range'] });
    const selectedRange = wrapper.find({ className: styles['selected-range'] });
    expect(range).to.be.present();
    expect(range.text()).to.eql('not-selectedRangeLabel');
    expect(selectedRange).to.be.present();
    expect(selectedRange.text()).to.eql('selectedRangeLabel');
  });

  it('handles click', () => {
    const onClick = sinon.spy();
    const wrapper = mount(
      <DefaultRanges
        {...defaultProps}
        onHandleDefaultRangeClick={onClick}
      />
    );
    expect(wrapper.find({ className: styles['selected-range'] })).to.be.present();
    wrapper.find({ className: styles['range'] }).simulate('click');
    expect(onClick.calledOnce).to.equal(true);
    expect(onClick.calledWithExactly(0, 100, RangeTypes.FIXED)).to.equal(true);
  });
});
