// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import {
  data,
  dataTypeMismatch,
  receiveMemberList,
  receiveMemberData,
  receiveOverviewData
} from 'reducers/monitoring/data';

import { DataLayoutTypes } from 'types/monitoring/data';
import { makeAction } from 'actions/util';
import {
  CLEAR_DATA,
  RECEIVE_DATA
} from 'actions/monitoring/data';


function createState(): Object {
  return {
    rollup: 'FIFTEEN_SECONDS',
    stepMs: 15000,
    type: DataLayoutTypes.CONSUMER_GROUPS,
    overview: {
      startTimeMs: 100000,
      produced: {
        count: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        crc: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
        error: [null, null, null, null, null, null, null, null, null, null]
      },
      consumed: {
        count: [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
        minLatency: [1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000],
        maxLatency: [1001, 2001, 3001, 4001, 5001, 6001, 7001, 8001, 9001, 10001],
        totalLatency: [10000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000, 100000],
        crc: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
        error: [null, null, null, null, null, null, null, null, null, null]
      },
      partialEpoch: 101,
      fullEpoch: 102
    },
    members: [
      {
        startTimeMs: 100000,
        group: 'my-consumer-group',
        produced: {
          count: [31, 32, 33, 34, 35, 36, 37, 38, 39, 40],
          crc: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
          error: [null, null, null, null, null, null, null, null, null, null]
        },
        consumed: {
          count: [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
          minLatency: [1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000],
          maxLatency: [1001, 2001, 3001, 4001, 5001, 6001, 7001, 8001, 9001, 10001],
          totalLatency: [10000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000, 100000],
          crc: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
          error: [null, null, null, null, null, null, null, null, null, null]
        }
      }
    ],
    sources: [
      {
        lala: 1
      }
    ],
    destinations: [
      {
        data: 2
      }
    ]
  };
}

function createMemberListData(): Object {
  return {
    rollup: 'FIFTEEN_SECONDS',
    type: 'CONSUMER',
    startTimeMs: 1465932540000,
    stepMs: 15000,
    clientId: 'other-id-1',
    members: [
      {
        topic: 'test-topic1',
        partition: 1
      }
    ],
    sources: [],
    destinations: [
      {
        topic: 'test-topic',
        partition: 0
      }
    ]
  };
}

function createOverviewData(): Object {
  return {
    rollup: 'FIFTEEN_SECONDS',
    type: 'CONSUMER_GROUPS',
    startTimeMs: 115000,
    stepMs: 15000,
    produced: {
      count: [41, 42, 43, 44, 45, 46, 47, 48, 49, 50],
      crc: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
      error: [null, null, null, null, null, null, null, null, null, null]
    },
    consumed: {
      count: [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
      minLatency: [1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000],
      maxLatency: [1004, 2004, 3004, 4004, 5004, 6004, 7004, 8004, 9004, 10004],
      totalLatency: [10000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000, 100000],
      crc: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
      error: [null, null, null, null, null, null, null, null, null, null]
    }
  };
}

function createMemberData(): Object {
  return {
    rollup: 'FIFTEEN_SECONDS',
    type: 'CONSUMER_GROUPS',
    startTimeMs: 115000,
    stepMs: 15000,
    members: [
      {
        group: 'my-consumer-group',
        produced: {
          count: [41, 42, 43, 44, 45, 46, 47, 48, 49, 50],
          crc: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
          error: [null, null, null, null, null, null, null, null, null, null]
        },
        consumed: {
          count: [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
          minLatency: [1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000],
          maxLatency: [1004, 2004, 3004, 4004, 5004, 6004, 7004, 8004, 9004, 10004],
          totalLatency: [10000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000, 100000],
          crc: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
          error: [null, null, null, null, null, null, null, null, null, null]
        }
      }
    ]
  };
}


describe('CLEAR_DATA', () => {

  it('returns null', () => {
    const result = data(createState(), makeAction(CLEAR_DATA, null));
    expect(result).to.be.null;
  });

});


describe('RECEIVE_DATA', () => {

  it('bad payload', () => {
    const state = createState();
    const result = data(
      state,
      makeAction(RECEIVE_DATA, { isFullUpdate: false, test: true })
    );
    expect(result).to.eql(state);
  });

  describe('dataTypeMismatch', () => {

    it('types not equal', () => {
      const state = {
        type: DataLayoutTypes.CONSUMER_GROUP,
        group: 'mygroup',
        stepMs: 10000
      };
      const data = {
        type: DataLayoutTypes.CONSUMER_GROUPS,
        stepMs: 10000
      };
      const result = dataTypeMismatch(state, data);
      expect(result).to.be.true;
    });

    it('type ids not equal', () => {
      const state = {
        type: DataLayoutTypes.CONSUMER_GROUP,
        group: 'mygroup',
        stepMs: 10000
      };
      const data = {
        type: DataLayoutTypes.CONSUMER_GROUP,
        group: 'mygroup2',
        stepMs: 10000
      };
      const result = dataTypeMismatch(state, data);
      expect(result).to.be.true;
    });

    it('no mismatch', () => {
      const state = {
        type: DataLayoutTypes.CONSUMER_GROUP,
        group: 'mygroup',
        stepMs: 10000
      };
      const data = {
        type: DataLayoutTypes.CONSUMER_GROUP,
        group: 'mygroup',
        stepMs: 10000
      };
      const result = dataTypeMismatch(state, data);
      expect(result).to.be.false;
    });

    it('no mismatch, null id', () => {
      const state = {
        type: DataLayoutTypes.CONSUMER_GROUPS,
        stepMs: 10000
      };
      const data = {
        type: DataLayoutTypes.CONSUMER_GROUPS,
        stepMs: 10000
      };
      const result = dataTypeMismatch(state, data);
      expect(result).to.be.false;
    });

    it('stepMs different', () => {
      const state = {
        type: DataLayoutTypes.CONSUMER_GROUPS,
        stepMs: 15000
      };
      const data = {
        type: DataLayoutTypes.CONSUMER_GROUPS,
        stepMs: 10000
      };
      const result = dataTypeMismatch(state, data);
      expect(result).to.be.true;
    });

  });

  describe('receiveMemberList', () => {
    it('no data', () => {
      const state = createState();
      const result = receiveMemberList(state, null);
      expect(result).to.eql(state);
    });

    it('no state', () => {
      const state = null;
      const data = createMemberListData();
      const result = receiveMemberList(state, data);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.members).to.exist;
      expect(result.members.length).to.eql(1);
      expect(result.members[0].topic).to.eql('test-topic1');
      expect(result.destinations.length).to.eql(1);
      expect(result.destinations[0].topic).to.eql('test-topic');
      expect(result.listEpoch).to.exist;
      expect(result.sources).to.exist;
      expect(result.sources.length).to.eql(0);
      expect(result.destinations).to.exist;
      expect(result.destinations.length).to.eql(1);
    });

    it('existing state', () => {
      const state = createState();
      const data = createMemberListData();
      const result = receiveMemberList(state, data);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.members).to.exist;
      expect(result.members.length).to.eql(1);
      expect(result.members[0].topic).to.eql('test-topic1');
      expect(result.destinations.length).to.eql(1);
      expect(result.destinations[0].topic).to.eql('test-topic');
      expect(result.listEpoch).to.exist;
      expect(result.sources).to.exist;
      expect(result.sources.length).to.eql(0);
      expect(result.destinations).to.exist;
      expect(result.destinations.length).to.eql(1);
    });
  });


  describe('receiveOverviewData', () => {

    it('no data', () => {
      const state = createState();
      const data = null;
      const result = receiveOverviewData(state, data, true);
      expect(result).to.eql(state);
    });

    it('data type mismatch', () => {
      const state = createState();
      const data = createOverviewData();
      data.type = 'CONSUMER';
      const result = receiveOverviewData(state, data, false);
      expect(result).to.eql(state);
    });

    it('full update', () => {
      const state = createState();
      const data = createOverviewData();
      const result = receiveOverviewData(state, data, true);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.overview).to.exist;
      expect(result.overview.fullEpoch).to.exist;
      expect(result.overview.partialEpoch).to.exist;
      expect(result.overview.produced).to.eql(data.produced);
      expect(result.overview.consumed).to.eql(data.consumed);
      expect(result.overview.startTimeMs).to.eql(data.startTimeMs);
    });

    it('full update', () => {
      const state = createState();
      // due to implementation details, this is good to have as a separate test.
      delete state.overview;
      const data = createOverviewData();
      const result = receiveOverviewData(state, data, true);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.overview).to.exist;
      expect(result.overview.fullEpoch).to.exist;
      expect(result.overview.partialEpoch).to.exist;
      expect(result.overview.produced).to.eql(data.produced);
      expect(result.overview.consumed).to.eql(data.consumed);
      expect(result.overview.startTimeMs).to.eql(data.startTimeMs);
    });

    it('partial update (new data starts in middle of existing)', () => {
      const state = createState();
      const data = createOverviewData();
      const result = receiveOverviewData(state, data, false);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.overview).to.exist;
      expect(result.overview.fullEpoch).to.exist;
      expect(result.overview.partialEpoch).to.exist;
      expect(result.overview.produced).to.exist;
      expect(result.overview.produced.count).to.exist;
      expect(result.overview.produced.count.length).to.eql(11);
      expect(result.overview.produced.count[10]).to.eql(50);
      expect(result.overview.produced.count[0]).to.eql(1);
      expect(result.overview.consumed.maxLatency.length).to.eql(11);
      expect(result.overview.consumed.maxLatency[0]).to.eql(1001);
      expect(result.overview.consumed.maxLatency[10]).to.eql(10004);
      expect(result.overview.startTimeMs).to.eql(state.overview.startTimeMs);
    });

    it('partial update (new data starts before existing)', () => {
      const state = createState();
      const data = createOverviewData();
      data.startTimeMs = 100000 - 15000;
      const result = receiveOverviewData(state, data, false);
      expect(result).to.not.be.null;
      if (!result) return;
      // in this case, should act like full update.
      expect(result.overview).to.exist;
      expect(result.overview.fullEpoch).to.exist;
      expect(result.overview.partialEpoch).to.exist;
      expect(result.overview.produced).to.exist;
      expect(result.overview.produced.count.length).to.eql(10);
      expect(result.overview.consumed.maxLatency.length).to.eql(10);
      expect(result.overview.produced.count[0]).to.eql(41);
      expect(result.overview.startTimeMs).to.eql(data.startTimeMs);
    });

    it('partial update (new data starts after existing)', () => {
      const state = createState();
      const data = createOverviewData();
      data.startTimeMs = 100000 + 15000 * 11;
      const result = receiveOverviewData(state, data, false);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.overview).to.exist;
      expect(result.overview.fullEpoch).to.exist;
      expect(result.overview.partialEpoch).to.exist;
      expect(result.overview.produced).to.exist;
      expect(result.overview.produced.count.length).to.eql(21);
      expect(result.overview.consumed.maxLatency.length).to.eql(21);
      expect(result.overview.produced.count[0]).to.eql(1);
      expect(result.overview.produced.count[10]).to.be.null;
      expect(result.overview.produced.count[20]).to.eql(50);
      expect(result.overview.startTimeMs).to.eql(state.overview.startTimeMs);
    });

  });


  describe('receiveMemberData', () => {

    it('no data', () => {
      const state = createState();
      const data = null;
      const result = receiveMemberData(state, data, true);
      expect(result).to.eql(state);
    });

    it('data type mismatch', () => {
      const state = createState();
      const data = createMemberData();
      data.type = 'CONSUMER';
      const result = receiveMemberData(state, data, false);
      expect(result).to.eql(state);
    });

    it('received member not in state', () => {
      const state = createState();
      const data = createMemberData();
      data.members[0].group = 'non-existant-group';
      const result = receiveMemberData(state, data, true);
      expect(result).to.eql(state);
    });

    it('full update', () => {
      const state = createState();
      const data = createMemberData();
      const result = receiveMemberData(state, data, true);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.overview).to.eql(state.overview);
      expect(result.destinations).to.eql(state.destinations);
      expect(result.sources).to.eql(state.sources);
      expect(result.members.length).to.eql(1);
      expect(result.members[0].startTimeMs).to.eql(data.startTimeMs);
      expect(result.members[0].produced).to.eql(data.members[0].produced);
      expect(result.members[0].consumed).to.eql(data.members[0].consumed);
    });

    it('fullupdate, existing state with no data', () => {
      // due to implementation details, this is good to have as a separate test.
      const state = createState();
      delete state.members[0].startTimeMs;
      delete state.members[0].produced;
      delete state.members[0].consumed;
      const data = createMemberData();
      const result = receiveMemberData(state, data, true);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.overview).to.eql(state.overview);
      expect(result.destinations).to.eql(state.destinations);
      expect(result.sources).to.eql(state.sources);
      expect(result.members.length).to.eql(1);
      expect(result.members[0].startTimeMs).to.eql(data.startTimeMs);
      expect(result.members[0].produced).to.eql(data.members[0].produced);
      expect(result.members[0].consumed).to.eql(data.members[0].consumed);
    });

    it('partial update (new data starts in middle of existing)', () => {
      const state = createState();
      const data = createMemberData();
      const result = receiveMemberData(state, data, false);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.members).to.exist;
      expect(result.members.length).to.eql(1);
      expect(result.members[0].group).to.eql('my-consumer-group');
      expect(result.members[0].produced.count.length).to.eql(11);
      expect(result.members[0].produced.count[0]).to.eql(31);
      expect(result.members[0].produced.count[1]).to.eql(41);
      expect(result.members[0].produced.count[10]).to.eql(50);
    });

    it('partial update (new data starts before existing)', () => {
      const state = createState();
      const data = createMemberData();
      data.startTimeMs = state.members[0].startTimeMs - 15000;
      const result = receiveMemberData(state, data, false);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.members).to.exist;
      expect(result.members.length).to.eql(1);
      expect(result.members[0].group).to.eql('my-consumer-group');
      expect(result.members[0].produced.count.length).to.eql(10);
      expect(result.members[0].startTimeMs).to.eql(data.startTimeMs);
    });

    it('partial update (new data starts after existing)', () => {
      const state = createState();
      const data = createMemberData();
      data.startTimeMs = state.members[0].startTimeMs + 15000 * 11;
      const result = receiveMemberData(state, data, false);
      expect(result).to.not.be.null;
      if (!result) return;
      expect(result.members).to.exist;
      expect(result.members.length).to.eql(1);
      expect(result.members[0].group).to.eql('my-consumer-group');
      expect(result.members[0].produced.count.length).to.eql(21);
      expect(result.members[0].startTimeMs).to.eql(state.members[0].startTimeMs);
      expect(result.members[0].produced.count[10]).to.eql(null);
      expect(result.members[0].produced.count[0]).to.eql(31);
      expect(result.members[0].produced.count[20]).to.eql(50);
      expect(result.members[0].consumed.maxLatency[10]).to.eql(null);
    });

  });

});

