// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import {
  mergeArrayCollection
} from 'reducers/monitoring/data';

describe('mergeArrayCollection', () => {

  it('matching members (1)', () => {
    const collection1 = {
      data: [1, 2, 3, 4]
    };
    const collection2 = {
      data: [10, 20, 30, 40]
    };
    const merged = mergeArrayCollection(collection1, collection2, 4, collection2.data.length);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(1);
    expect(merged.data).to.not.be.null;
    expect(merged.data[0]).to.eql(1);
    expect(merged.data[7]).to.eql(40);
    expect(merged.data.length).to.eql(8);
  });

  it('matching members (2)', () => {
    const collection1 = {
      data: [1, 2, 3, 4]
    };
    const collection2 = {
      data: [10, 20, 30, 40]
    };
    const merged = mergeArrayCollection(collection1, collection2, 3, collection2.data.length);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(1);
    expect(merged.data).to.not.be.null;
    expect(merged.data[0]).to.eql(1);
    expect(merged.data[6]).to.eql(40);
    expect(merged.data.length).to.eql(7);
  });

  it('both empty', () => {
    const collection1 = {};
    const collection2 = {};
    const merged = mergeArrayCollection(collection1, collection2, 3, 10);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(0);
  });

  it('empty collection1', () => {
    const collection1 = {};
    const collection2 = {
      data: [1, 2, 3, 4]
    };
    const merged = mergeArrayCollection(collection1, collection2, 3, collection2.data.length);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(1);
    expect(merged.data).to.not.be.null;
    expect(merged.data[0]).to.be.null;
    expect(merged.data[3]).to.eql(1);
    expect(merged.data.length).to.eql(7);
  });

  it('empty collection2', () => {
    const collection1 = {
      data: [1, 2, 3, 4]
    };
    const collection2 = {};
    const merged = mergeArrayCollection(collection1, collection2, 3, 1000);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(1);
    expect(merged.data).to.not.be.null;
    expect(merged.data[0]).to.eql(1);
    expect(merged.data[2]).to.eql(3);
    expect(merged.data.length).to.eql(1003);
  });

  it('missing in collection2', () => {
    const collection1 = {
      data: [1, 2, 3, 4],
      data2: [5, 6, 7, 8]
    };
    const collection2 = {
      data: [10, 20, 30, 40]
    };
    const merged = mergeArrayCollection(collection1, collection2, 3, collection2.data.length);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(2);
    expect(merged.data).to.not.be.null;
    expect(merged.data2).to.not.be.null;
    expect(merged.data2[0]).to.eql(5);
    expect(merged.data[0]).to.eql(1);
  });

  it('missing in collection1', () => {
    const collection1 = {
      data: [1, 2, 3, 4]
    };
    const collection2 = {
      data: [10, 20, 30, 40],
      data2: [50, 60, 70, 80]
    };
    const merged = mergeArrayCollection(collection1, collection2, 3, collection2.data.length);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(2);
    expect(merged.data).to.not.be.null;
    expect(merged.data2).to.not.be.null;
    expect(merged.data2[0]).to.be.null;
    expect(merged.data[0]).to.eql(1);
    expect(merged.data2[3]).to.eql(50);
    expect(merged.data[3]).to.eql(10);
  });

  it('multiple', () => {
    const collection1 = {
      data: [1, 2, 3, 4],
      data2: [5, 6, 7, 8],
      data3: [9, 10, 11, 12]
    };
    const collection2 = {
      data: [10, 20, 30, 40],
      data2: [50, 60, 70, 80],
      data3: [90, 100, 110, 120]
    };
    const merged = mergeArrayCollection(collection1, collection2, 4, collection2.data.length);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(3);
    expect(merged.data).to.not.be.null;
    expect(merged.data2).to.not.be.null;
    expect(merged.data3).to.not.be.null;
    expect(merged.data.length).to.eql(8);
    expect(merged.data2.length).to.eql(8);
    expect(merged.data3.length).to.eql(8);
  });

  it('missing collection 1', () => {
    const collection1 = null;
    const collection2 = {
      data: [10, 20, 30, 40],
      data2: [50, 60, 70, 80]
    };
    const merged = mergeArrayCollection(collection1, collection2, 3, collection2.data.length);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(2);
    expect(merged.data).to.not.be.null;
    expect(merged.data2).to.not.be.null;
    expect(merged.data2[0]).to.be.null;
    expect(merged.data[0]).to.be.null;
    expect(merged.data2[3]).to.eql(50);
    expect(merged.data[3]).to.eql(10);
  });

  it('missing collection 2', () => {
    const collection1 = {
      data: [1, 2, 3, 4],
      data2: [5, 6, 7, 8]
    };
    const collection2 = null;
    const merged = mergeArrayCollection(collection1, collection2, 3, 10);
    const ks = Object.keys(merged);
    expect(ks.length).to.eql(2);
    expect(merged.data).to.not.be.null;
    expect(merged.data2).to.not.be.null;
    expect(merged.data[0]).to.eql(1);
    expect(merged.data2[0]).to.eql(5);
    expect(merged.data.length).to.eql(13);
    expect(merged.data2.length).to.eql(13);
    expect(merged.data[3]).to.be.null;
    expect(merged.data2[3]).to.be.null;
    expect(merged.data[12]).to.be.null;
    expect(merged.data2[12]).to.be.null;
  });
});
