// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import {
  mergeArrays
} from 'reducers/monitoring/data';

describe('mergeArrays', () => {

  it('both arrays null', () => {
    expect(mergeArrays(null, null, 30)).to.be.null;
  });

  it('first array null', () => {
    const merged = mergeArrays(
      null,
      [1, 2, 3, 4],
      4
    );
    expect(merged).to.not.be.null;
    if (!merged) return;
    expect(merged[0]).to.be.null;
    expect(merged[3]).to.be.null;
    expect(merged[4]).to.eql(1);
    expect(merged[7]).to.eql(4);
  });

  it('second array null (1)', () => {
    const merged = mergeArrays(
      [1, 2, 3, 4],
      null,
      3
    );
    expect(merged).to.not.be.null;
    if (!merged) return;
    expect(merged.length).to.eql(3);
    expect(merged[2]).to.eql(3);
  });

  it('second array null (2)', () => {
    const merged = mergeArrays(
      [1, 2, 3, 4],
      null,
      4
    );
    expect(merged).to.not.be.null;
    if (!merged) return;
    expect(merged.length).to.eql(4);
    expect(merged[3]).to.eql(4);
  });

  it('second array null (3)', () => {
    const merged = mergeArrays(
      [1, 2, 3, 4],
      null,
      5
    );
    expect(merged).to.not.be.null;
    if (!merged) return;
    expect(merged.length).to.eql(5);
    expect(merged[4]).to.be.null;
  });

  it('no gap', () => {
    const merged = mergeArrays(
      [1, 2, 3, 4],
      [10, 20, 30, 40],
      4
    );
    expect(merged).to.not.be.null;
    if (!merged) return;
    expect(merged.length).to.eql(8);
    expect(merged[4]).to.eql(10);
    expect(merged[0]).to.eql(1);
  });

  it('has gap', () => {
    const merged = mergeArrays(
      [1, 2, 3, 4],
      [10, 20, 30, 40],
      5
    );
    expect(merged).to.not.be.null;
    if (!merged) return;
    expect(merged.length).to.eql(9);
    expect(merged[4]).to.be.null;
    expect(merged[5]).to.eql(10);
    expect(merged[0]).to.eql(1);
  });

  it('overlap', () => {
    const merged = mergeArrays(
      [1, 2, 3, 4],
      [10, 20, 30, 40],
      3
    );
    expect(merged).to.not.be.null;
    if (!merged) return;
    expect(merged.length).to.eql(7);
    expect(merged[4]).to.eql(20);
    expect(merged[3]).to.eql(10);
    expect(merged[0]).to.eql(1);
  });

  it('replace', () => {
    const merged = mergeArrays(
      [1, 2, 3, 4],
      [10, 20],
      0
    );
    expect(merged).to.not.be.null;
    if (!merged) return;
    expect(merged.length).to.eql(2);
    expect(merged[1]).to.eql(20);
    expect(merged[0]).to.eql(10);
  });

});
