// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import {
  incrementConnectionCount,
  decrementConnectionCount,
  allowDataFetch,
  disallowDataFetch
} from 'reducers/monitoring/dataManager';

import type { DataManagerState } from 'reducers/monitoring/dataManager';


function makeState(): DataManagerState {
  return {
    connectionCount: 10,
    preventFetch: false
  };
}

describe('connectionCount', () => {

  it('increment', () => {
    const state = makeState();
    const result = incrementConnectionCount(state);
    expect(result.connectionCount).to.eql(11);
    expect(result.preventFetch).to.eql(state.preventFetch);
  });

  it('decrement', () => {
    const state = makeState();
    const result = decrementConnectionCount(state);
    expect(result.connectionCount).to.eql(9);
    expect(result.preventFetch).to.eql(state.preventFetch);
  });

});

describe('preventFetch', () => {

  it('allow', () => {
    const state = makeState();
    state.preventFetch = true;
    const result = allowDataFetch(state);
    expect(result.preventFetch).to.eql(false);
    expect(result.connectionCount).to.eql(state.connectionCount);
  });

  it('disallow', () => {
    const state = makeState();
    const result = disallowDataFetch(state);
    expect(result.preventFetch).to.eql(true);
    expect(result.connectionCount).to.eql(state.connectionCount);
  });

});
