// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.
import { cancel, fork, take } from 'redux-saga/effects';
import { createMockTask } from 'redux-saga/utils';

import { createDebounceSaga } from 'sagas/util';

describe('sagas/util', () => {
  const TEST_ACTION_TYPE = 'some-test-action-type';

  let generator;
  let next;
  let testWorker;

  describe('createDebounceSaga', () => {
    const expectStartOfLoop = next => expect(next.value).to.deep.equal(take(TEST_ACTION_TYPE));

    beforeEach(() => {
      testWorker = sinon.spy();
      const debouncedSaga = createDebounceSaga(TEST_ACTION_TYPE, testWorker);
      generator = debouncedSaga();
      next = generator.next();
    });

    it('take specified action', () => {
      expectStartOfLoop(next);
    });

    it('should fork testWorker on first pass', () => {
      next = generator.next(TEST_ACTION_TYPE);

      expect(
        next.value
      ).to.deep.equal(
        fork(testWorker, TEST_ACTION_TYPE)
      );
    });

    it('should cancel task if it is already running', () => {
      next = generator.next();

      const task = createMockTask();
      next = generator.next(task);
      next = generator.next();

      expect(
        next.value
      ).to.deep.equal(
        cancel(task)
      );

      next = generator.next();

      // start of loop again
      next = generator.next();
      expectStartOfLoop(next);
      next = generator.next();

      expect(
        next.value
      ).to.not.deep.equal(
        cancel(task)
      );

      next = generator.next();
      expectStartOfLoop(next);
    });

  });
});
