// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import { expect } from 'chai';

import {
  dropLeadingZero,
  sumArrayUntil,
  rangeLimit
} from 'utils';

describe('util', () => {
  describe('rangeLimit', () => {
    it('rangeLimit - within range', () => {
      expect(rangeLimit(5, 10)).to.eql(5);
    });

    it('rangeLimit - out of range (too low)', () => {
      expect(rangeLimit(-1, 10)).to.eql(0);
    });

    it('rangeLimit - out of range (too high)', () => {
      expect(rangeLimit(10.1, 10)).to.eql(10);
    });

    it('rangeLimit - at range limit low', () => {
      expect(rangeLimit(0, 10)).to.eql(0);
    });

    it('rangeLimit - at range limit high', () => {
      expect(rangeLimit(10, 10)).to.eql(10);
    });
  });

  describe('sumArrayUntil', () => {
    const myArray = [1, 2, 3, 4];

    it('handles negative index', () => {
      expect(sumArrayUntil(myArray, -1)).to.eql(0);
    });

    it('handles index=0', () => {
      expect(sumArrayUntil(myArray, 0)).to.eql(1);
    });

    it('handles interior index', () => {
      expect(sumArrayUntil(myArray, 1)).to.eql(3);
    });

    it('handles max index', () => {
      expect(sumArrayUntil(myArray, myArray.length)).to.eql(10);
    });

    it('handles index beyond length', () => {
      expect(sumArrayUntil(myArray, myArray.length * 2)).to.eql(10);
    });

    it('handles nulls in array', () => {
      expect(sumArrayUntil(myArray.concat([null]), myArray.length * 2)).to.eql(10);
    });
  });

  describe('dropLeadingZero', () => {
    it('drops leading zero', () => {
      expect(dropLeadingZero('0420')).to.eql('420');
    });

    it('only drops one zero', () => {
      expect(dropLeadingZero('00420')).to.eql('0420');
    });

    it('ignores nonzero leading char', () => {
      expect(dropLeadingZero('100000')).to.eql('100000');
    });
  });
});
