// @flow
// (Copyright) [2016 - 2016] Confluent, Inc.

import { expect } from 'chai';

import {
  computeIgnoreDeliveryDiscrepancyAfterIndex,
  timeSeriesSlice
} from 'utils/monitoring/data';


describe('utils/monitoring/data', () => {

  it('computeIgnoreDeliveryDiscrepancyAfterIndex all null data', () => {
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(null, null)).to.eql(Infinity);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex empty data', () => {
    const ts1 = { startTimeMs: 0, stepMs: 15000, values: [] };
    const ts2 = { startTimeMs: 0, stepMs: 15000, values: [] };
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts1, ts2)).to.eql(Infinity);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex same data', () => {
    const ts3 = {
      startTimeMs: 0,
      stepMs: 15000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    };
    const ts4 = {
      startTimeMs: 0,
      stepMs: 15000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    };
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts3, ts4)).to.eql(9);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex off by one', () => {
    const ts5 = {
      startTimeMs: 0,
      stepMs: 15000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    };
    const ts6 = {
      startTimeMs: 0,
      stepMs: 15000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 9]
    };
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts5, ts6)).to.eql(8);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex same then not', () => {
    const ts7 = {
      startTimeMs: 0,
      stepMs: 15000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    };
    const ts8 = {
      startTimeMs: 0,
      stepMs: 15000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 8, 10]
    };
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts7, ts8)).to.eql(9);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex lastDate before timeseries start', () => {
    const ts9 = {
      startTimeMs: 5000,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, null, null, null]
    };
    const ts10 = {
      startTimeMs: 5000,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, null, null, null]
    };
    const lastDate = new Date(50);
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts9, ts10, lastDate)).to.eql(0);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex lastDate sane', () => {
    const ts11 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, null, null, null]
    };
    const ts12 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, null, null, null]
    };
    const lastDate = new Date(30050);
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts11, ts12, lastDate)).to.eql(6);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex lastDate sane 2', () => {
    const ts13 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, null, null, null, null, null]
    };
    const ts14 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, null, null, null]
    };
    const lastDate = new Date(30050);
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts13, ts14, lastDate)).to.eql(4);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex lastDate sane 3', () => {
    const ts15 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, null, null, null]
    };
    const ts16 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, null, null, null]
    };
    const lastDate = new Date(30000);
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts15, ts16, lastDate)).to.eql(6);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex lastDate after timeseries start 1', () => {
    const ts17 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    };
    const ts18 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 9]
    };
    const lastDate = new Date(60000);
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts17, ts18, lastDate)).to.eql(8);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex lastDate after timeseries start 2', () => {
    const ts19 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    };
    const ts20 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 2, 3, 4, 5, 6, 7, 8, 9, 9]
    };
    const lastDate = new Date(90000);
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts19, ts20, lastDate)).to.eql(8);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex lastDate after timeseries start 3', () => {
    const ts21 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 3, 1, 3, 1, 3, 1, 3, 1, 3]
    };
    const ts22 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [2, 4, 2, 4, 2, 4, 2, 4, 2, 4]
    };
    const lastDate = new Date(90050);
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts21, ts22, lastDate)).to.eql(6);
  });

  it('computeIgnoreDeliveryDiscrepancyAfterIndex lastDate after timeseries start 4', () => {
    const ts23 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [1, 3, 1, 3, 1, 3, 1, 3, 1, 3]
    };
    const ts24 = {
      startTimeMs: 0,
      stepMs: 5000,
      values: [2, 4, 2, 4, 2, 4, 2, 4, 2, 4]
    };
    const lastDate = new Date(200000);
    expect(computeIgnoreDeliveryDiscrepancyAfterIndex(ts23, ts24, lastDate)).to.eql(9);
  });

  it('timeSeriesSlice', () => {
    const ts = {
      startTimeMs: 0,
      stepMs: 15000,
      values: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    };
    const sliced = timeSeriesSlice(ts, 2, 7);
    expect(sliced.startTimeMs).to.eql(30000);
    expect(sliced.stepMs).to.eql(15000);
    expect(sliced.values.length).to.eql(5);
    expect(sliced.values[0]).to.eql(2);
  });

});
