// (Copyright) [2016 - 2016] Confluent, Inc.
// @flow
import path from 'path';
import webpack from 'webpack';
import ExportFilesWebpackPlugin from 'export-files-webpack-plugin';
import ExtractTextPlugin from 'extract-text-webpack-plugin';
import HtmlWebpackPlugin from 'html-webpack-plugin';
import WriteFilePlugin from 'write-file-webpack-plugin';
import {
  baseWebpackConfig,
  envGlobals,
  PATHS
} from './webpack.config';

// copy base config
const devLoaders = baseWebpackConfig.module.loaders.slice();

// take out jsx loader
const devJsxLoader = devLoaders.shift();

// modify jsx loader to include react-hot
devJsxLoader.loaders.unshift('react-hot');

// add jsx loader back to devLoaders
devLoaders.unshift(devJsxLoader);

const devServer = {
  contentBase: path.resolve(__dirname, '.'),
  colors: true,
  quiet: false,
  noInfo: false,
  publicPath: 'dist/',
  historyApiFallback: false,
  host: 'localhost',
  hot: true,
  outputPath: PATHS.buildDir
};

export const webpackConfig = {
  ...baseWebpackConfig,
  devServer,
  devtool: 'source-map',
  entry: ['webpack-hot-middleware/client'].concat(baseWebpackConfig.entry),
  module: {
    ...baseWebpackConfig.module,
    loaders: devLoaders
  },
  plugins: [
    new ExtractTextPlugin('app.css', {
      allChunks: true
    }),
    new webpack.DefinePlugin(envGlobals),
    new HtmlWebpackPlugin({
      title: 'Control Center',
      hash: false,
      template: './index.html',
      filename: './dist/index.html',
      favicon: path.join(PATHS.staticDir, 'favicon.ico'),
      inject: 'body',
      minify: {
        collapseWhitespace: true
      }
    }),
    new ExportFilesWebpackPlugin('./dist/index.html'),
    new webpack.optimize.OccurenceOrderPlugin(),
    new webpack.HotModuleReplacementPlugin(),
    new webpack.NoErrorsPlugin(),
    new WriteFilePlugin()
  ]
};


if (envGlobals.__USE_REDBOX__) {
  webpackConfig.output.devtoolModuleFilenameTemplate = '/[absolute-resource-path]';
  webpackConfig.devtool = 'eval';
}
